// Copyright (c) 2022 NetEase, Inc. All rights reserved.
// Use of this source code is governed by a MIT license that can be
// found in the LICENSE file.

package com.netease.nertc_core.nertc_core_example;

import android.app.Notification;
import android.app.NotificationManager;
import android.app.Service;
import android.content.Context;
import android.content.Intent;
import android.content.pm.ServiceInfo;
import android.os.Build;
import android.os.IBinder;
import android.os.Process;
import android.util.Log;

public class CallForegroundService extends Service {

  public static final int ONGOING_NOTIFICATION_ID = 0x1314;

  private static ForegroundServiceConfig foregroundServiceConfig;

  NotificationManager notificationManager;

  static ForegroundServiceConfig getForegroundServiceConfig() {
    return foregroundServiceConfig;
  }

  public static void start(Context context, ForegroundServiceConfig config) {
    if (config == null) {
      return;
    }
    foregroundServiceConfig = config;
    if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.O) {
      Log.e("CallForegroundService", "startForegroundService");
      context.startForegroundService(new Intent(context, CallForegroundService.class));
    } else {
      Log.e("CallForegroundService", "startService");
      context.startService(new Intent(context, CallForegroundService.class));
    }
  }

  public static void cancel(Context context) {
    // stopService may throw remoteException.
    //
    // if the service had been started as foreground, but
    // being brought down before actually showing a notification.
    // That is not allowed.
    try {
      context.stopService(new Intent(context, CallForegroundService.class));
      NENotificationManager.getInstance().cancelNotification(context, ONGOING_NOTIFICATION_ID);
    } catch (Exception e) {
      Log.e("CallForegroundService", "cancel foreground service error", e);
    }
  }

  @Override
  public void onCreate() {
    Log.i("CallForegroundService", "onCreate");
    super.onCreate();
    notificationManager = (NotificationManager) getSystemService(Context.NOTIFICATION_SERVICE);
    onStartForeground();
  }

  @Override
  public int onStartCommand(Intent intent, int flags, int startId) {
    Log.i("CallForegroundService", "onStartCommand");
    onStartForeground();
    super.onStartCommand(intent, flags, startId);
    return START_NOT_STICKY;
  }

  private void onStartForeground() {
    NENotificationManager.getInstance().createForegroundNotificationChannel(this);
    // Notification ID cannot be 0.
    Notification notification =
        NENotificationManager.getInstance().buildForegroundNotification(this);
    if (notification == null) {
      Log.i("CallForegroundService", "without notification, ignore start foreground.");
      return;
    }
    if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.Q) {
      startForeground(
          ONGOING_NOTIFICATION_ID,
          notification,
          ServiceInfo.FOREGROUND_SERVICE_TYPE_MEDIA_PROJECTION);
    } else {
      startForeground(ONGOING_NOTIFICATION_ID, notification);
    }

    Log.i("CallForegroundService", "onStartForeground");
  }

  @Override
  public IBinder onBind(Intent intent) {
    return null;
  }

  @Override
  public void onTaskRemoved(Intent rootIntent) {
    super.onTaskRemoved(rootIntent);
    if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.Q) {
      Log.i("CallForegroundService", "kill self due to task removed!");
      //quit application
      stopSelf();
      android.os.Process.killProcess(Process.myPid());
      System.exit(0);
    }
  }
}
