// Copyright (c) 2022 NetEase, Inc. All rights reserved.
// Use of this source code is governed by a MIT license that can be
// found in the LICENSE file.

library nertc;

import 'dart:async';
import 'dart:convert';
import 'dart:io';

import 'package:flutter/foundation.dart';
import 'package:flutter/material.dart';
import 'package:flutter/services.dart';
import 'package:nertc_core_platform_interface/nertc_core_platform_interface.dart';
import 'package:nertc_core_platform_interface/pigeon.dart';
import 'package:path_provider/path_provider.dart';
import 'package:yunxin_alog/yunxin_alog.dart';
import 'package:crypto/crypto.dart';

export 'package:nertc_core_platform_interface/nertc_core_platform_interface.dart'
    show
        NERtcUserRole,
        NERtcClientRole,
        NERtcServerAddresses,
        NERtcJoinChannelOptions,
        NERtcConnectionType,
        NERtcConnectionState,
        NERtcNetworkQuality,
        NERtcConnectionStateChangeReason,
        NERtcAudioDevice,
        NERtcAudioDeviceState,
        NERtcVideoDeviceState,
        NERtcAudioDeviceType,
        NERtcCameraType,
        NERtcCameraPosition,
        NERtcScreenConfig,
        NERtcScreenProfile,
        NERtcSubStreamContentPrefer,
        NERtcCameraCapturePreference,
        NERtcCameraCaptureConfig,
        NERtcOptions,
        NERtcServerRecordMode,
        NERtcVideoSendMode,
        NERtcMediaCodecMode,
        NERtcAudioSessionOperationRestriction,
        NERtcAudioProfile,
        NERtcAudioScenario,
        NERtcVoiceChangerType,
        NERtcVoiceBeautifierType,
        NERtcVideoConfig,
        NERtcVideoProfile,
        NERtcVideoMirrorMode,
        NERtcLocalVideoWatermarkState,
        NERtcReverbParam,
        LastmileProbeConfig,
        NERtcVideoCorrectionConfiguration,
        NERtcVideoOutputOrientationMode,
        NERtcRemoteVideoStreamType,
        NERtcVideoViewFitType,
        NERtcVideoCropMode,
        NERtcVideoStreamType,
        NERtcAudioVolumeInfo,
        NERtcAudioDumpType,
        NERtcAudioMixingError,
        NERtcAudioRecordingConfiguration,
        NERtcAudioRecordingQuality,
        NERtcAudioRecordingPosition,
        NERtcAudioRecordingCycleTime,
        NERtcMediaPriority,
        NERtcMediaPubType,
        NERtcLastmileProbeResultState,
        NERtcChannelMediaRelayState,
        NERtcChannelMediaRelayEvent,
        NERtcChannelMediaRelayInfo,
        NERtcStreamFallbackOptions,
        NERtcAudioRecordingCode,
        NERtcChannelMediaRelayConfiguration,
        VirtualBackgroundSourceType,
        BlurDegree,
        NERtcVirtualBackgroundSource,
        NERtcVirtualBackgroundSourceStateReason,
        NERtcStreamChannelType,
        NERtcEncryptionConfig,
        NERtcEncryptionMode,
        NERtcTransportType,
        NERtcBeautyEffectType,
        NERtcAudioMixingTaskState,
        NERtcAudioStreamType,
        NERtcLiveStreamErrorCode,
        NERtcLiveStreamLayout,
        NERtcLiveStreamVideoScaleMode,
        NERtcLiveStreamUserTranscoding,
        NERtcLiveStreamImageInfo,
        NERtcLiveStreamTaskInfo,
        NERtcErrorCode,
        NERtcRuntimeError,
        NERtcNetworkStatus,
        NERtcChannelProfile,
        NERtcAudioFocusMode,
        NERtcDegradationPreference,
        NERtcVideoFrameRate,
        NERtcLiveStreamMode,
        NERtcLiveStreamState,
        NERtcAudioMixingOptions,
        NERtcAudioEffectOptions,
        NERtcAudioSendStats,
        NERtcStats,
        NERtcNetworkQualityInfo,
        NERtcAudioRecvStats,
        NERtcVideoSendStats,
        NERtcVideoRecvStats,
        NERtcVideoWatermarkConfig,
        NERtcVideoWatermarkImageConfig,
        NERtcVideoWatermarkTextConfig,
        NERtcVideoWatermarkTimestampConfig;

export 'package:nertc_core_platform_interface/pigeon.dart'
    show
        NERtcVersion,
        CGPoint,
        NERtcVideoWatermarkType,
        NERtcUserLeaveExtraInfo,
        NERtcUserJoinExtraInfo,
        NERtcCaptureExtraRotation,
        NERtcLastmileProbeResult,
        NERtcLastmileProbeOneWayResult;

part 'src/api/nertc_event_callback.dart';
part 'src/nertc_video_view.dart';
part 'src/nertc_parameters_key.dart';

part 'src/api/log_service.dart';
part 'src/api/nertc_engine.dart';
part 'src/api/nertc_device.dart';
part 'src/api/nertc_audio_effect.dart';
part 'src/api/nertc_audio_mixing.dart';
part 'src/api/nertc_video_renderer.dart';

part 'src/impl/nertc_engine_impl.dart';
part 'src/impl/nertc_device_impl.dart';
part 'src/impl/nertc_audio_effect_impl.dart';
part 'src/impl/nertc_audio_mixing_impl.dart';
part 'src/impl/nertc_video_renderer_impl.dart';
