// Copyright (c) 2022 NetEase, Inc. All rights reserved.
// Use of this source code is governed by a MIT license that can be
// found in the LICENSE file.

part of nertc;

class _NERtcAudioEffectMgrImpl extends NERtcAudioEffectManager
    with _SDKLoggerMixin, LoggingApi {
  final _platform = NERtcAudioEffectPlatform.instance;
  final _audioEffectEventCallbacks = <NERtcAudioEffectEventCallback>{};

  List<NERtcAudioEffectEventCallback> get audioEffectEventCallbacks =>
      _audioEffectEventCallbacks.toList();

  _NERtcAudioEffectMgrImpl();

  @override
  String get logTag => 'AudioEffectApi';

  @override
  Future<int> pauseAllEffect() async {
    int reply = await wrapper('pauseAllEffect', _platform.pauseAllEffect());
    return reply;
  }

  @override
  Future<int> pauseEffect(int effectId) async {
    apiLogger.i('pauseEffect#arg{effectId:$effectId}');
    int reply = await wrapper('pauseEffect', _platform.pauseEffect(effectId));
    return reply;
  }

  @override
  Future<int> playEffect(int effectId, NERtcAudioEffectOptions options) async {
    apiLogger.i('playEffect#arg{effectId:$effectId,options:$options}');
    int reply =
        await wrapper('playEffect', _platform.playEffect(effectId, options));
    return reply;
  }

  @override
  Future<int> resumeAllEffect() async {
    int reply = await wrapper('resumeAllEffect', _platform.resumeAllEffect());
    return reply;
  }

  @override
  Future<int> resumeEffect(int effectId) async {
    int reply = await wrapper('resumeEffect', _platform.resumeEffect(effectId));
    return reply;
  }

  @override
  Future<int> stopAllEffects() async {
    int reply = await wrapper('stopAllEffects', _platform.stopAllEffects());
    return reply;
  }

  @override
  Future<int> stopEffect(int effectId) async {
    int reply = await wrapper('stopEffect', _platform.stopEffect(effectId));
    return reply;
  }

  @override
  Future<int> getEffectCurrentPosition(int effectId) async {
    int reply = await wrapper('getEffectCurrentPosition',
        _platform.getEffectCurrentPosition(effectId));
    return reply;
  }

  @override
  Future<int> getEffectDuration(int effectId) async {
    int reply = await wrapper(
        'getEffectDuration', _platform.getEffectDuration(effectId));
    return reply;
  }

  @override
  Future<int> getEffectPitch(int effectId) async {
    int reply =
        await wrapper('getEffectPitch', _platform.getEffectPitch(effectId));
    return reply;
  }

  @override
  Future<int> getEffectPlaybackVolume(int effectId) async {
    int reply = await wrapper(
        'getEffectPlaybackVolume', _platform.getEffectPlaybackVolume(effectId));
    return reply;
  }

  @override
  Future<int> getEffectSendVolume(int effectId) async {
    int reply = await wrapper(
        'getEffectSendVolume', _platform.getEffectSendVolume(effectId));
    return reply;
  }

  @override
  Future<int> setEffectPitch(int effectId, int pitch) async {
    int reply = await wrapper(
        'setEffectPitch', _platform.setEffectPitch(effectId, pitch));
    return reply;
  }

  @override
  Future<int> setEffectPlaybackVolume(int effectId, int volume) async {
    int reply = await wrapper('setEffectPlaybackVolume',
        _platform.setEffectPlaybackVolume(effectId, volume));
    return reply;
  }

  @override
  Future<int> setEffectPosition(int effectId, int position) async {
    int reply = await wrapper(
        'setEffectPosition', _platform.setEffectPosition(effectId, position));
    return reply;
  }

  @override
  Future<int> setEffectSendVolume(int effectId, int volume) async {
    int reply = await wrapper(
        'setEffectSendVolume', _platform.setEffectSendVolume(effectId, volume));
    return reply;
  }

  @override
  void setEventCallback(NERtcAudioEffectEventCallback callback) {
    _audioEffectEventCallbacks.add(callback);
  }

  @override
  void removeEventCallback(NERtcAudioEffectEventCallback callback) {
    _audioEffectEventCallbacks.remove(callback);
  }
}
