// Copyright (c) 2022 NetEase, Inc. All rights reserved.
// Use of this source code is governed by a MIT license that can be
// found in the LICENSE file.

part of nertc;

class _NERtcDeviceManagerImpl extends NERtcDeviceManager
    with _SDKLoggerMixin, LoggingApi {
  final _platform = NERtcDevicePlatform.instance;

  final _deviceEventCallbacks = <NERtcDeviceEventCallback>{};

  List<NERtcDeviceEventCallback> get deviceEventCallbacks =>
      _deviceEventCallbacks.toList();

  _NERtcDeviceManagerImpl();

  @override
  String get logTag => 'DeviceManagerApi';

  @override
  void setEventCallback(NERtcDeviceEventCallback callback) {
    _deviceEventCallbacks.add(callback);
  }

  @override
  void removeEventCallback(NERtcDeviceEventCallback callback) {
    _deviceEventCallbacks.remove(callback);
  }

  @override
  Future<double> getCameraMaxZoom() async {
    double reply =
        await wrapper('getCameraMaxZoom', _platform.getCameraMaxZoom());
    return reply;
  }

  @override
  Future<bool> isCameraExposurePositionSupported() async {
    bool reply = await wrapper('isCameraExposurePositionSupported',
        _platform.isCameraExposurePositionSupported());
    return reply;
  }

  @override
  Future<bool> isCameraFocusSupported() async {
    bool reply = await wrapper(
        'isCameraFocusSupported', _platform.isCameraFocusSupported());
    return reply;
  }

  @override
  Future<bool> isCameraTorchSupported() async {
    bool reply = await wrapper(
        'isCameraTorchSupported', _platform.isCameraTorchSupported());
    return reply;
  }

  @override
  Future<bool> isCameraZoomSupported() async {
    bool reply = await wrapper(
        'isCameraZoomSupported', _platform.isCameraZoomSupported());
    return reply;
  }

  @override
  Future<bool> isPlayoutDeviceMute() async {
    bool reply =
        await wrapper('isCameraZoomSupported', _platform.isPlayoutDeviceMute());
    return reply;
  }

  @override
  Future<bool> isRecordDeviceMute() async {
    bool reply =
        await wrapper('isRecordDeviceMute', _platform.isRecordDeviceMute());
    return reply;
  }

  @override
  Future<int> setCameraFocusPosition(double x, double y) async {
    apiLogger.i('setCameraFocusPosition#arg{x:$x,y:$y}');
    int reply = await wrapper(
        'setCameraFocusPosition', _platform.setCameraFocusPosition(x, y));
    return reply;
  }

  @override
  Future<int> setCameraTorchOn(bool on) async {
    apiLogger.i('setCameraTorchOn#arg{on:$on}');
    int reply =
        await wrapper('setCameraTorchOn', _platform.setCameraTorchOn(on));
    return reply;
  }

  @override
  Future<int> setCameraZoomFactor(double factor) async {
    apiLogger.i('setCameraTorchOn#arg{factor:$factor}');
    int reply = await wrapper(
        'setCameraZoomFactor', _platform.setCameraZoomFactor(factor));
    return reply;
  }

  @override
  Future<int> setPlayoutDeviceMute(bool mute) async {
    apiLogger.i('setPlayoutDeviceMute#arg{mute:$mute}');
    int reply = await wrapper(
        'setPlayoutDeviceMute', _platform.setPlayoutDeviceMute(mute));
    return reply;
  }

  @override
  Future<int> setRecordDeviceMute(bool mute) async {
    apiLogger.i('setRecordDeviceMute#arg{mute:$mute}');
    int reply = await wrapper(
        'setRecordDeviceMute', _platform.setRecordDeviceMute(mute));
    return reply;
  }

  @override
  Future<int> switchCamera() async {
    int reply = await wrapper('switchCamera', _platform.switchCamera());
    return reply;
  }

  @override
  Future<int> enableEarback(bool enabled, int volume) async {
    apiLogger.i('enableEarback#arg{enabled:$enabled,volume:$volume}');
    int reply = await wrapper(
        'enableEarback', _platform.enableEarback(enabled, volume));
    return reply;
  }

  @override
  Future<bool> isSpeakerphoneOn() async {
    bool reply =
        await wrapper('isSpeakerphoneOn', _platform.isSpeakerphoneOn());
    return reply;
  }

  @override
  Future<int> setAudioFocusMode(int focusMode) async {
    apiLogger.i('setAudioFocusMode#arg{focusMode:$focusMode}');
    int reply = await wrapper(
        'setAudioFocusMode', _platform.setAudioFocusMode(focusMode));
    return reply;
  }

  @override
  Future<int> setEarbackVolume(int volume) async {
    apiLogger.i('setEarbackVolume#arg{volume:$volume}');
    int reply =
        await wrapper('setEarbackVolume', _platform.setEarbackVolume(volume));
    return reply;
  }

  @override
  Future<int> setSpeakerphoneOn(bool enable) async {
    apiLogger.i('setSpeakerphoneOn#arg{enable:$enable}');
    int reply =
        await wrapper('setSpeakerphoneOn', _platform.setSpeakerphoneOn(enable));
    return reply;
  }

  @override
  Future<int> getCameraCurrentZoom() async {
    int reply =
        await wrapper('getCameraCurrentZoom', _platform.getCameraCurrentZoom());
    return reply;
  }

  @override
  Future<int> getCurrentCamera() async {
    int reply = await wrapper('getCurrentCamera', _platform.getCurrentCamera());
    return reply;
  }

  @override
  Future<int> switchCameraWithPosition(int position) async {
    apiLogger.i('switchCameraWithPosition#arg{position:$position}');
    int reply = await wrapper('switchCameraWithPosition',
        _platform.switchCameraWithPosition(position));
    return reply;
  }

  @override
  Future<int> setCameraExposurePosition(double x, double y) async {
    apiLogger.i('setCameraExposurePosition#arg{x:$x,y:$y}');
    int reply = await wrapper(
        'setCameraExposurePosition', _platform.setCameraExposurePosition(x, y));
    return reply;
  }
}
