// Copyright (c) 2022 NetEase, Inc. All rights reserved.
// Use of this source code is governed by a MIT license that can be
// found in the LICENSE file.

#import "Messages.h"
#import <Flutter/Flutter.h>

#if !__has_feature(objc_arc)
#error File requires ARC to be enabled.
#endif

static NSArray *wrapResult(id result, FlutterError *error) {
  if (error) {
    return @[
      error.code ?: [NSNull null], error.message ?: [NSNull null], error.details ?: [NSNull null]
    ];
  }
  return @[ result ?: [NSNull null] ];
}
static id GetNullableObjectAtIndex(NSArray *array, NSInteger key) {
  id result = array[key];
  return (result == [NSNull null]) ? nil : result;
}

@interface NEFLTNERtcUserJoinExtraInfo ()
+ (NEFLTNERtcUserJoinExtraInfo *)fromList:(NSArray *)list;
+ (nullable NEFLTNERtcUserJoinExtraInfo *)nullableFromList:(NSArray *)list;
- (NSArray *)toList;
@end

@interface NEFLTNERtcUserLeaveExtraInfo ()
+ (NEFLTNERtcUserLeaveExtraInfo *)fromList:(NSArray *)list;
+ (nullable NEFLTNERtcUserLeaveExtraInfo *)nullableFromList:(NSArray *)list;
- (NSArray *)toList;
@end

@interface NEFLTUserJoinedEvent ()
+ (NEFLTUserJoinedEvent *)fromList:(NSArray *)list;
+ (nullable NEFLTUserJoinedEvent *)nullableFromList:(NSArray *)list;
- (NSArray *)toList;
@end

@interface NEFLTUserLeaveEvent ()
+ (NEFLTUserLeaveEvent *)fromList:(NSArray *)list;
+ (nullable NEFLTUserLeaveEvent *)nullableFromList:(NSArray *)list;
- (NSArray *)toList;
@end

@interface NEFLTUserVideoMuteEvent ()
+ (NEFLTUserVideoMuteEvent *)fromList:(NSArray *)list;
+ (nullable NEFLTUserVideoMuteEvent *)nullableFromList:(NSArray *)list;
- (NSArray *)toList;
@end

@interface NEFLTFirstVideoDataReceivedEvent ()
+ (NEFLTFirstVideoDataReceivedEvent *)fromList:(NSArray *)list;
+ (nullable NEFLTFirstVideoDataReceivedEvent *)nullableFromList:(NSArray *)list;
- (NSArray *)toList;
@end

@interface NEFLTFirstVideoFrameDecodedEvent ()
+ (NEFLTFirstVideoFrameDecodedEvent *)fromList:(NSArray *)list;
+ (nullable NEFLTFirstVideoFrameDecodedEvent *)nullableFromList:(NSArray *)list;
- (NSArray *)toList;
@end

@interface NEFLTVirtualBackgroundSourceEnabledEvent ()
+ (NEFLTVirtualBackgroundSourceEnabledEvent *)fromList:(NSArray *)list;
+ (nullable NEFLTVirtualBackgroundSourceEnabledEvent *)nullableFromList:(NSArray *)list;
- (NSArray *)toList;
@end

@interface NEFLTAudioVolumeInfo ()
+ (NEFLTAudioVolumeInfo *)fromList:(NSArray *)list;
+ (nullable NEFLTAudioVolumeInfo *)nullableFromList:(NSArray *)list;
- (NSArray *)toList;
@end

@interface NEFLTRemoteAudioVolumeIndicationEvent ()
+ (NEFLTRemoteAudioVolumeIndicationEvent *)fromList:(NSArray *)list;
+ (nullable NEFLTRemoteAudioVolumeIndicationEvent *)nullableFromList:(NSArray *)list;
- (NSArray *)toList;
@end

@interface NEFLTNERtcLastmileProbeResult ()
+ (NEFLTNERtcLastmileProbeResult *)fromList:(NSArray *)list;
+ (nullable NEFLTNERtcLastmileProbeResult *)nullableFromList:(NSArray *)list;
- (NSArray *)toList;
@end

@interface NEFLTNERtcLastmileProbeOneWayResult ()
+ (NEFLTNERtcLastmileProbeOneWayResult *)fromList:(NSArray *)list;
+ (nullable NEFLTNERtcLastmileProbeOneWayResult *)nullableFromList:(NSArray *)list;
- (NSArray *)toList;
@end

@interface NEFLTRtcServerAddresses ()
+ (NEFLTRtcServerAddresses *)fromList:(NSArray *)list;
+ (nullable NEFLTRtcServerAddresses *)nullableFromList:(NSArray *)list;
- (NSArray *)toList;
@end

@interface NEFLTCreateEngineRequest ()
+ (NEFLTCreateEngineRequest *)fromList:(NSArray *)list;
+ (nullable NEFLTCreateEngineRequest *)nullableFromList:(NSArray *)list;
- (NSArray *)toList;
@end

@interface NEFLTJoinChannelOptions ()
+ (NEFLTJoinChannelOptions *)fromList:(NSArray *)list;
+ (nullable NEFLTJoinChannelOptions *)nullableFromList:(NSArray *)list;
- (NSArray *)toList;
@end

@interface NEFLTJoinChannelRequest ()
+ (NEFLTJoinChannelRequest *)fromList:(NSArray *)list;
+ (nullable NEFLTJoinChannelRequest *)nullableFromList:(NSArray *)list;
- (NSArray *)toList;
@end

@interface NEFLTSubscribeRemoteAudioRequest ()
+ (NEFLTSubscribeRemoteAudioRequest *)fromList:(NSArray *)list;
+ (nullable NEFLTSubscribeRemoteAudioRequest *)nullableFromList:(NSArray *)list;
- (NSArray *)toList;
@end

@interface NEFLTEnableLocalVideoRequest ()
+ (NEFLTEnableLocalVideoRequest *)fromList:(NSArray *)list;
+ (nullable NEFLTEnableLocalVideoRequest *)nullableFromList:(NSArray *)list;
- (NSArray *)toList;
@end

@interface NEFLTSetAudioProfileRequest ()
+ (NEFLTSetAudioProfileRequest *)fromList:(NSArray *)list;
+ (nullable NEFLTSetAudioProfileRequest *)nullableFromList:(NSArray *)list;
- (NSArray *)toList;
@end

@interface NEFLTSetLocalVideoConfigRequest ()
+ (NEFLTSetLocalVideoConfigRequest *)fromList:(NSArray *)list;
+ (nullable NEFLTSetLocalVideoConfigRequest *)nullableFromList:(NSArray *)list;
- (NSArray *)toList;
@end

@interface NEFLTSetCameraCaptureConfigRequest ()
+ (NEFLTSetCameraCaptureConfigRequest *)fromList:(NSArray *)list;
+ (nullable NEFLTSetCameraCaptureConfigRequest *)nullableFromList:(NSArray *)list;
- (NSArray *)toList;
@end

@interface NEFLTStartorStopVideoPreviewRequest ()
+ (NEFLTStartorStopVideoPreviewRequest *)fromList:(NSArray *)list;
+ (nullable NEFLTStartorStopVideoPreviewRequest *)nullableFromList:(NSArray *)list;
- (NSArray *)toList;
@end

@interface NEFLTStartScreenCaptureRequest ()
+ (NEFLTStartScreenCaptureRequest *)fromList:(NSArray *)list;
+ (nullable NEFLTStartScreenCaptureRequest *)nullableFromList:(NSArray *)list;
- (NSArray *)toList;
@end

@interface NEFLTSubscribeRemoteVideoStreamRequest ()
+ (NEFLTSubscribeRemoteVideoStreamRequest *)fromList:(NSArray *)list;
+ (nullable NEFLTSubscribeRemoteVideoStreamRequest *)nullableFromList:(NSArray *)list;
- (NSArray *)toList;
@end

@interface NEFLTSubscribeRemoteSubStreamVideoRequest ()
+ (NEFLTSubscribeRemoteSubStreamVideoRequest *)fromList:(NSArray *)list;
+ (nullable NEFLTSubscribeRemoteSubStreamVideoRequest *)nullableFromList:(NSArray *)list;
- (NSArray *)toList;
@end

@interface NEFLTEnableAudioVolumeIndicationRequest ()
+ (NEFLTEnableAudioVolumeIndicationRequest *)fromList:(NSArray *)list;
+ (nullable NEFLTEnableAudioVolumeIndicationRequest *)nullableFromList:(NSArray *)list;
- (NSArray *)toList;
@end

@interface NEFLTSubscribeRemoteSubStreamAudioRequest ()
+ (NEFLTSubscribeRemoteSubStreamAudioRequest *)fromList:(NSArray *)list;
+ (nullable NEFLTSubscribeRemoteSubStreamAudioRequest *)nullableFromList:(NSArray *)list;
- (NSArray *)toList;
@end

@interface NEFLTSetAudioSubscribeOnlyByRequest ()
+ (NEFLTSetAudioSubscribeOnlyByRequest *)fromList:(NSArray *)list;
+ (nullable NEFLTSetAudioSubscribeOnlyByRequest *)nullableFromList:(NSArray *)list;
- (NSArray *)toList;
@end

@interface NEFLTStartAudioMixingRequest ()
+ (NEFLTStartAudioMixingRequest *)fromList:(NSArray *)list;
+ (nullable NEFLTStartAudioMixingRequest *)nullableFromList:(NSArray *)list;
- (NSArray *)toList;
@end

@interface NEFLTPlayEffectRequest ()
+ (NEFLTPlayEffectRequest *)fromList:(NSArray *)list;
+ (nullable NEFLTPlayEffectRequest *)nullableFromList:(NSArray *)list;
- (NSArray *)toList;
@end

@interface NEFLTSetCameraPositionRequest ()
+ (NEFLTSetCameraPositionRequest *)fromList:(NSArray *)list;
+ (nullable NEFLTSetCameraPositionRequest *)nullableFromList:(NSArray *)list;
- (NSArray *)toList;
@end

@interface NEFLTAddOrUpdateLiveStreamTaskRequest ()
+ (NEFLTAddOrUpdateLiveStreamTaskRequest *)fromList:(NSArray *)list;
+ (nullable NEFLTAddOrUpdateLiveStreamTaskRequest *)nullableFromList:(NSArray *)list;
- (NSArray *)toList;
@end

@interface NEFLTDeleteLiveStreamTaskRequest ()
+ (NEFLTDeleteLiveStreamTaskRequest *)fromList:(NSArray *)list;
+ (nullable NEFLTDeleteLiveStreamTaskRequest *)nullableFromList:(NSArray *)list;
- (NSArray *)toList;
@end

@interface NEFLTSendSEIMsgRequest ()
+ (NEFLTSendSEIMsgRequest *)fromList:(NSArray *)list;
+ (nullable NEFLTSendSEIMsgRequest *)nullableFromList:(NSArray *)list;
- (NSArray *)toList;
@end

@interface NEFLTSetLocalVoiceEqualizationRequest ()
+ (NEFLTSetLocalVoiceEqualizationRequest *)fromList:(NSArray *)list;
+ (nullable NEFLTSetLocalVoiceEqualizationRequest *)nullableFromList:(NSArray *)list;
- (NSArray *)toList;
@end

@interface NEFLTSwitchChannelRequest ()
+ (NEFLTSwitchChannelRequest *)fromList:(NSArray *)list;
+ (nullable NEFLTSwitchChannelRequest *)nullableFromList:(NSArray *)list;
- (NSArray *)toList;
@end

@interface NEFLTStartAudioRecordingRequest ()
+ (NEFLTStartAudioRecordingRequest *)fromList:(NSArray *)list;
+ (nullable NEFLTStartAudioRecordingRequest *)nullableFromList:(NSArray *)list;
- (NSArray *)toList;
@end

@interface NEFLTAudioRecordingConfigurationRequest ()
+ (NEFLTAudioRecordingConfigurationRequest *)fromList:(NSArray *)list;
+ (nullable NEFLTAudioRecordingConfigurationRequest *)nullableFromList:(NSArray *)list;
- (NSArray *)toList;
@end

@interface NEFLTSetLocalMediaPriorityRequest ()
+ (NEFLTSetLocalMediaPriorityRequest *)fromList:(NSArray *)list;
+ (nullable NEFLTSetLocalMediaPriorityRequest *)nullableFromList:(NSArray *)list;
- (NSArray *)toList;
@end

@interface NEFLTStartOrUpdateChannelMediaRelayRequest ()
+ (NEFLTStartOrUpdateChannelMediaRelayRequest *)fromList:(NSArray *)list;
+ (nullable NEFLTStartOrUpdateChannelMediaRelayRequest *)nullableFromList:(NSArray *)list;
- (NSArray *)toList;
@end

@interface NEFLTAdjustUserPlaybackSignalVolumeRequest ()
+ (NEFLTAdjustUserPlaybackSignalVolumeRequest *)fromList:(NSArray *)list;
+ (nullable NEFLTAdjustUserPlaybackSignalVolumeRequest *)nullableFromList:(NSArray *)list;
- (NSArray *)toList;
@end

@interface NEFLTEnableEncryptionRequest ()
+ (NEFLTEnableEncryptionRequest *)fromList:(NSArray *)list;
+ (nullable NEFLTEnableEncryptionRequest *)nullableFromList:(NSArray *)list;
- (NSArray *)toList;
@end

@interface NEFLTSetLocalVoiceReverbParamRequest ()
+ (NEFLTSetLocalVoiceReverbParamRequest *)fromList:(NSArray *)list;
+ (nullable NEFLTSetLocalVoiceReverbParamRequest *)nullableFromList:(NSArray *)list;
- (NSArray *)toList;
@end

@interface NEFLTReportCustomEventRequest ()
+ (NEFLTReportCustomEventRequest *)fromList:(NSArray *)list;
+ (nullable NEFLTReportCustomEventRequest *)nullableFromList:(NSArray *)list;
- (NSArray *)toList;
@end

@interface NEFLTStartLastmileProbeTestRequest ()
+ (NEFLTStartLastmileProbeTestRequest *)fromList:(NSArray *)list;
+ (nullable NEFLTStartLastmileProbeTestRequest *)nullableFromList:(NSArray *)list;
- (NSArray *)toList;
@end

@interface NEFLTCGPoint ()
+ (NEFLTCGPoint *)fromList:(NSArray *)list;
+ (nullable NEFLTCGPoint *)nullableFromList:(NSArray *)list;
- (NSArray *)toList;
@end

@interface NEFLTSetVideoCorrectionConfigRequest ()
+ (NEFLTSetVideoCorrectionConfigRequest *)fromList:(NSArray *)list;
+ (nullable NEFLTSetVideoCorrectionConfigRequest *)nullableFromList:(NSArray *)list;
- (NSArray *)toList;
@end

@interface NEFLTEnableVirtualBackgroundRequest ()
+ (NEFLTEnableVirtualBackgroundRequest *)fromList:(NSArray *)list;
+ (nullable NEFLTEnableVirtualBackgroundRequest *)nullableFromList:(NSArray *)list;
- (NSArray *)toList;
@end

@interface NEFLTSetRemoteHighPriorityAudioStreamRequest ()
+ (NEFLTSetRemoteHighPriorityAudioStreamRequest *)fromList:(NSArray *)list;
+ (nullable NEFLTSetRemoteHighPriorityAudioStreamRequest *)nullableFromList:(NSArray *)list;
- (NSArray *)toList;
@end

@interface NEFLTVideoWatermarkImageConfig ()
+ (NEFLTVideoWatermarkImageConfig *)fromList:(NSArray *)list;
+ (nullable NEFLTVideoWatermarkImageConfig *)nullableFromList:(NSArray *)list;
- (NSArray *)toList;
@end

@interface NEFLTVideoWatermarkTextConfig ()
+ (NEFLTVideoWatermarkTextConfig *)fromList:(NSArray *)list;
+ (nullable NEFLTVideoWatermarkTextConfig *)nullableFromList:(NSArray *)list;
- (NSArray *)toList;
@end

@interface NEFLTVideoWatermarkTimestampConfig ()
+ (NEFLTVideoWatermarkTimestampConfig *)fromList:(NSArray *)list;
+ (nullable NEFLTVideoWatermarkTimestampConfig *)nullableFromList:(NSArray *)list;
- (NSArray *)toList;
@end

@interface NEFLTVideoWatermarkConfig ()
+ (NEFLTVideoWatermarkConfig *)fromList:(NSArray *)list;
+ (nullable NEFLTVideoWatermarkConfig *)nullableFromList:(NSArray *)list;
- (NSArray *)toList;
@end

@interface NEFLTSetLocalVideoWatermarkConfigsRequest ()
+ (NEFLTSetLocalVideoWatermarkConfigsRequest *)fromList:(NSArray *)list;
+ (nullable NEFLTSetLocalVideoWatermarkConfigsRequest *)nullableFromList:(NSArray *)list;
- (NSArray *)toList;
@end

@interface NEFLTNERtcVersion ()
+ (NEFLTNERtcVersion *)fromList:(NSArray *)list;
+ (nullable NEFLTNERtcVersion *)nullableFromList:(NSArray *)list;
- (NSArray *)toList;
@end

@implementation NEFLTNERtcUserJoinExtraInfo
+ (instancetype)makeWithCustomInfo:(NSString *)customInfo {
  NEFLTNERtcUserJoinExtraInfo *pigeonResult = [[NEFLTNERtcUserJoinExtraInfo alloc] init];
  pigeonResult.customInfo = customInfo;
  return pigeonResult;
}
+ (NEFLTNERtcUserJoinExtraInfo *)fromList:(NSArray *)list {
  NEFLTNERtcUserJoinExtraInfo *pigeonResult = [[NEFLTNERtcUserJoinExtraInfo alloc] init];
  pigeonResult.customInfo = GetNullableObjectAtIndex(list, 0);
  NSAssert(pigeonResult.customInfo != nil, @"");
  return pigeonResult;
}
+ (nullable NEFLTNERtcUserJoinExtraInfo *)nullableFromList:(NSArray *)list {
  return (list) ? [NEFLTNERtcUserJoinExtraInfo fromList:list] : nil;
}
- (NSArray *)toList {
  return @[
    (self.customInfo ?: [NSNull null]),
  ];
}
@end

@implementation NEFLTNERtcUserLeaveExtraInfo
+ (instancetype)makeWithCustomInfo:(NSString *)customInfo {
  NEFLTNERtcUserLeaveExtraInfo *pigeonResult = [[NEFLTNERtcUserLeaveExtraInfo alloc] init];
  pigeonResult.customInfo = customInfo;
  return pigeonResult;
}
+ (NEFLTNERtcUserLeaveExtraInfo *)fromList:(NSArray *)list {
  NEFLTNERtcUserLeaveExtraInfo *pigeonResult = [[NEFLTNERtcUserLeaveExtraInfo alloc] init];
  pigeonResult.customInfo = GetNullableObjectAtIndex(list, 0);
  NSAssert(pigeonResult.customInfo != nil, @"");
  return pigeonResult;
}
+ (nullable NEFLTNERtcUserLeaveExtraInfo *)nullableFromList:(NSArray *)list {
  return (list) ? [NEFLTNERtcUserLeaveExtraInfo fromList:list] : nil;
}
- (NSArray *)toList {
  return @[
    (self.customInfo ?: [NSNull null]),
  ];
}
@end

@implementation NEFLTUserJoinedEvent
+ (instancetype)makeWithUid:(NSNumber *)uid
              joinExtraInfo:(nullable NEFLTNERtcUserJoinExtraInfo *)joinExtraInfo {
  NEFLTUserJoinedEvent *pigeonResult = [[NEFLTUserJoinedEvent alloc] init];
  pigeonResult.uid = uid;
  pigeonResult.joinExtraInfo = joinExtraInfo;
  return pigeonResult;
}
+ (NEFLTUserJoinedEvent *)fromList:(NSArray *)list {
  NEFLTUserJoinedEvent *pigeonResult = [[NEFLTUserJoinedEvent alloc] init];
  pigeonResult.uid = GetNullableObjectAtIndex(list, 0);
  NSAssert(pigeonResult.uid != nil, @"");
  pigeonResult.joinExtraInfo =
      [NEFLTNERtcUserJoinExtraInfo nullableFromList:(GetNullableObjectAtIndex(list, 1))];
  return pigeonResult;
}
+ (nullable NEFLTUserJoinedEvent *)nullableFromList:(NSArray *)list {
  return (list) ? [NEFLTUserJoinedEvent fromList:list] : nil;
}
- (NSArray *)toList {
  return @[
    (self.uid ?: [NSNull null]),
    (self.joinExtraInfo ? [self.joinExtraInfo toList] : [NSNull null]),
  ];
}
@end

@implementation NEFLTUserLeaveEvent
+ (instancetype)makeWithUid:(NSNumber *)uid
                     reason:(NSNumber *)reason
             leaveExtraInfo:(nullable NEFLTNERtcUserLeaveExtraInfo *)leaveExtraInfo {
  NEFLTUserLeaveEvent *pigeonResult = [[NEFLTUserLeaveEvent alloc] init];
  pigeonResult.uid = uid;
  pigeonResult.reason = reason;
  pigeonResult.leaveExtraInfo = leaveExtraInfo;
  return pigeonResult;
}
+ (NEFLTUserLeaveEvent *)fromList:(NSArray *)list {
  NEFLTUserLeaveEvent *pigeonResult = [[NEFLTUserLeaveEvent alloc] init];
  pigeonResult.uid = GetNullableObjectAtIndex(list, 0);
  NSAssert(pigeonResult.uid != nil, @"");
  pigeonResult.reason = GetNullableObjectAtIndex(list, 1);
  NSAssert(pigeonResult.reason != nil, @"");
  pigeonResult.leaveExtraInfo =
      [NEFLTNERtcUserLeaveExtraInfo nullableFromList:(GetNullableObjectAtIndex(list, 2))];
  return pigeonResult;
}
+ (nullable NEFLTUserLeaveEvent *)nullableFromList:(NSArray *)list {
  return (list) ? [NEFLTUserLeaveEvent fromList:list] : nil;
}
- (NSArray *)toList {
  return @[
    (self.uid ?: [NSNull null]),
    (self.reason ?: [NSNull null]),
    (self.leaveExtraInfo ? [self.leaveExtraInfo toList] : [NSNull null]),
  ];
}
@end

@implementation NEFLTUserVideoMuteEvent
+ (instancetype)makeWithUid:(NSNumber *)uid
                      muted:(NSNumber *)muted
                 streamType:(nullable NSNumber *)streamType {
  NEFLTUserVideoMuteEvent *pigeonResult = [[NEFLTUserVideoMuteEvent alloc] init];
  pigeonResult.uid = uid;
  pigeonResult.muted = muted;
  pigeonResult.streamType = streamType;
  return pigeonResult;
}
+ (NEFLTUserVideoMuteEvent *)fromList:(NSArray *)list {
  NEFLTUserVideoMuteEvent *pigeonResult = [[NEFLTUserVideoMuteEvent alloc] init];
  pigeonResult.uid = GetNullableObjectAtIndex(list, 0);
  NSAssert(pigeonResult.uid != nil, @"");
  pigeonResult.muted = GetNullableObjectAtIndex(list, 1);
  NSAssert(pigeonResult.muted != nil, @"");
  pigeonResult.streamType = GetNullableObjectAtIndex(list, 2);
  return pigeonResult;
}
+ (nullable NEFLTUserVideoMuteEvent *)nullableFromList:(NSArray *)list {
  return (list) ? [NEFLTUserVideoMuteEvent fromList:list] : nil;
}
- (NSArray *)toList {
  return @[
    (self.uid ?: [NSNull null]),
    (self.muted ?: [NSNull null]),
    (self.streamType ?: [NSNull null]),
  ];
}
@end

@implementation NEFLTFirstVideoDataReceivedEvent
+ (instancetype)makeWithUid:(NSNumber *)uid streamType:(nullable NSNumber *)streamType {
  NEFLTFirstVideoDataReceivedEvent *pigeonResult = [[NEFLTFirstVideoDataReceivedEvent alloc] init];
  pigeonResult.uid = uid;
  pigeonResult.streamType = streamType;
  return pigeonResult;
}
+ (NEFLTFirstVideoDataReceivedEvent *)fromList:(NSArray *)list {
  NEFLTFirstVideoDataReceivedEvent *pigeonResult = [[NEFLTFirstVideoDataReceivedEvent alloc] init];
  pigeonResult.uid = GetNullableObjectAtIndex(list, 0);
  NSAssert(pigeonResult.uid != nil, @"");
  pigeonResult.streamType = GetNullableObjectAtIndex(list, 1);
  return pigeonResult;
}
+ (nullable NEFLTFirstVideoDataReceivedEvent *)nullableFromList:(NSArray *)list {
  return (list) ? [NEFLTFirstVideoDataReceivedEvent fromList:list] : nil;
}
- (NSArray *)toList {
  return @[
    (self.uid ?: [NSNull null]),
    (self.streamType ?: [NSNull null]),
  ];
}
@end

@implementation NEFLTFirstVideoFrameDecodedEvent
+ (instancetype)makeWithUid:(NSNumber *)uid
                      width:(NSNumber *)width
                     height:(NSNumber *)height
                 streamType:(nullable NSNumber *)streamType {
  NEFLTFirstVideoFrameDecodedEvent *pigeonResult = [[NEFLTFirstVideoFrameDecodedEvent alloc] init];
  pigeonResult.uid = uid;
  pigeonResult.width = width;
  pigeonResult.height = height;
  pigeonResult.streamType = streamType;
  return pigeonResult;
}
+ (NEFLTFirstVideoFrameDecodedEvent *)fromList:(NSArray *)list {
  NEFLTFirstVideoFrameDecodedEvent *pigeonResult = [[NEFLTFirstVideoFrameDecodedEvent alloc] init];
  pigeonResult.uid = GetNullableObjectAtIndex(list, 0);
  NSAssert(pigeonResult.uid != nil, @"");
  pigeonResult.width = GetNullableObjectAtIndex(list, 1);
  NSAssert(pigeonResult.width != nil, @"");
  pigeonResult.height = GetNullableObjectAtIndex(list, 2);
  NSAssert(pigeonResult.height != nil, @"");
  pigeonResult.streamType = GetNullableObjectAtIndex(list, 3);
  return pigeonResult;
}
+ (nullable NEFLTFirstVideoFrameDecodedEvent *)nullableFromList:(NSArray *)list {
  return (list) ? [NEFLTFirstVideoFrameDecodedEvent fromList:list] : nil;
}
- (NSArray *)toList {
  return @[
    (self.uid ?: [NSNull null]),
    (self.width ?: [NSNull null]),
    (self.height ?: [NSNull null]),
    (self.streamType ?: [NSNull null]),
  ];
}
@end

@implementation NEFLTVirtualBackgroundSourceEnabledEvent
+ (instancetype)makeWithEnabled:(NSNumber *)enabled reason:(NSNumber *)reason {
  NEFLTVirtualBackgroundSourceEnabledEvent *pigeonResult =
      [[NEFLTVirtualBackgroundSourceEnabledEvent alloc] init];
  pigeonResult.enabled = enabled;
  pigeonResult.reason = reason;
  return pigeonResult;
}
+ (NEFLTVirtualBackgroundSourceEnabledEvent *)fromList:(NSArray *)list {
  NEFLTVirtualBackgroundSourceEnabledEvent *pigeonResult =
      [[NEFLTVirtualBackgroundSourceEnabledEvent alloc] init];
  pigeonResult.enabled = GetNullableObjectAtIndex(list, 0);
  NSAssert(pigeonResult.enabled != nil, @"");
  pigeonResult.reason = GetNullableObjectAtIndex(list, 1);
  NSAssert(pigeonResult.reason != nil, @"");
  return pigeonResult;
}
+ (nullable NEFLTVirtualBackgroundSourceEnabledEvent *)nullableFromList:(NSArray *)list {
  return (list) ? [NEFLTVirtualBackgroundSourceEnabledEvent fromList:list] : nil;
}
- (NSArray *)toList {
  return @[
    (self.enabled ?: [NSNull null]),
    (self.reason ?: [NSNull null]),
  ];
}
@end

@implementation NEFLTAudioVolumeInfo
+ (instancetype)makeWithUid:(NSNumber *)uid
                     volume:(NSNumber *)volume
            subStreamVolume:(NSNumber *)subStreamVolume {
  NEFLTAudioVolumeInfo *pigeonResult = [[NEFLTAudioVolumeInfo alloc] init];
  pigeonResult.uid = uid;
  pigeonResult.volume = volume;
  pigeonResult.subStreamVolume = subStreamVolume;
  return pigeonResult;
}
+ (NEFLTAudioVolumeInfo *)fromList:(NSArray *)list {
  NEFLTAudioVolumeInfo *pigeonResult = [[NEFLTAudioVolumeInfo alloc] init];
  pigeonResult.uid = GetNullableObjectAtIndex(list, 0);
  NSAssert(pigeonResult.uid != nil, @"");
  pigeonResult.volume = GetNullableObjectAtIndex(list, 1);
  NSAssert(pigeonResult.volume != nil, @"");
  pigeonResult.subStreamVolume = GetNullableObjectAtIndex(list, 2);
  NSAssert(pigeonResult.subStreamVolume != nil, @"");
  return pigeonResult;
}
+ (nullable NEFLTAudioVolumeInfo *)nullableFromList:(NSArray *)list {
  return (list) ? [NEFLTAudioVolumeInfo fromList:list] : nil;
}
- (NSArray *)toList {
  return @[
    (self.uid ?: [NSNull null]),
    (self.volume ?: [NSNull null]),
    (self.subStreamVolume ?: [NSNull null]),
  ];
}
@end

@implementation NEFLTRemoteAudioVolumeIndicationEvent
+ (instancetype)makeWithVolumeList:(nullable NSArray<NEFLTAudioVolumeInfo *> *)volumeList
                       totalVolume:(NSNumber *)totalVolume {
  NEFLTRemoteAudioVolumeIndicationEvent *pigeonResult =
      [[NEFLTRemoteAudioVolumeIndicationEvent alloc] init];
  pigeonResult.volumeList = volumeList;
  pigeonResult.totalVolume = totalVolume;
  return pigeonResult;
}
+ (NEFLTRemoteAudioVolumeIndicationEvent *)fromList:(NSArray *)list {
  NEFLTRemoteAudioVolumeIndicationEvent *pigeonResult =
      [[NEFLTRemoteAudioVolumeIndicationEvent alloc] init];
  pigeonResult.volumeList = GetNullableObjectAtIndex(list, 0);
  pigeonResult.totalVolume = GetNullableObjectAtIndex(list, 1);
  NSAssert(pigeonResult.totalVolume != nil, @"");
  return pigeonResult;
}
+ (nullable NEFLTRemoteAudioVolumeIndicationEvent *)nullableFromList:(NSArray *)list {
  return (list) ? [NEFLTRemoteAudioVolumeIndicationEvent fromList:list] : nil;
}
- (NSArray *)toList {
  return @[
    (self.volumeList ?: [NSNull null]),
    (self.totalVolume ?: [NSNull null]),
  ];
}
@end

@implementation NEFLTNERtcLastmileProbeResult
+ (instancetype)makeWithState:(NSNumber *)state
                          rtt:(NSNumber *)rtt
                 uplinkReport:(NEFLTNERtcLastmileProbeOneWayResult *)uplinkReport
               downlinkReport:(NEFLTNERtcLastmileProbeOneWayResult *)downlinkReport {
  NEFLTNERtcLastmileProbeResult *pigeonResult = [[NEFLTNERtcLastmileProbeResult alloc] init];
  pigeonResult.state = state;
  pigeonResult.rtt = rtt;
  pigeonResult.uplinkReport = uplinkReport;
  pigeonResult.downlinkReport = downlinkReport;
  return pigeonResult;
}
+ (NEFLTNERtcLastmileProbeResult *)fromList:(NSArray *)list {
  NEFLTNERtcLastmileProbeResult *pigeonResult = [[NEFLTNERtcLastmileProbeResult alloc] init];
  pigeonResult.state = GetNullableObjectAtIndex(list, 0);
  NSAssert(pigeonResult.state != nil, @"");
  pigeonResult.rtt = GetNullableObjectAtIndex(list, 1);
  NSAssert(pigeonResult.rtt != nil, @"");
  pigeonResult.uplinkReport =
      [NEFLTNERtcLastmileProbeOneWayResult nullableFromList:(GetNullableObjectAtIndex(list, 2))];
  NSAssert(pigeonResult.uplinkReport != nil, @"");
  pigeonResult.downlinkReport =
      [NEFLTNERtcLastmileProbeOneWayResult nullableFromList:(GetNullableObjectAtIndex(list, 3))];
  NSAssert(pigeonResult.downlinkReport != nil, @"");
  return pigeonResult;
}
+ (nullable NEFLTNERtcLastmileProbeResult *)nullableFromList:(NSArray *)list {
  return (list) ? [NEFLTNERtcLastmileProbeResult fromList:list] : nil;
}
- (NSArray *)toList {
  return @[
    (self.state ?: [NSNull null]),
    (self.rtt ?: [NSNull null]),
    (self.uplinkReport ? [self.uplinkReport toList] : [NSNull null]),
    (self.downlinkReport ? [self.downlinkReport toList] : [NSNull null]),
  ];
}
@end

@implementation NEFLTNERtcLastmileProbeOneWayResult
+ (instancetype)makeWithPacketLossRate:(NSNumber *)packetLossRate
                                jitter:(NSNumber *)jitter
                    availableBandwidth:(NSNumber *)availableBandwidth {
  NEFLTNERtcLastmileProbeOneWayResult *pigeonResult =
      [[NEFLTNERtcLastmileProbeOneWayResult alloc] init];
  pigeonResult.packetLossRate = packetLossRate;
  pigeonResult.jitter = jitter;
  pigeonResult.availableBandwidth = availableBandwidth;
  return pigeonResult;
}
+ (NEFLTNERtcLastmileProbeOneWayResult *)fromList:(NSArray *)list {
  NEFLTNERtcLastmileProbeOneWayResult *pigeonResult =
      [[NEFLTNERtcLastmileProbeOneWayResult alloc] init];
  pigeonResult.packetLossRate = GetNullableObjectAtIndex(list, 0);
  NSAssert(pigeonResult.packetLossRate != nil, @"");
  pigeonResult.jitter = GetNullableObjectAtIndex(list, 1);
  NSAssert(pigeonResult.jitter != nil, @"");
  pigeonResult.availableBandwidth = GetNullableObjectAtIndex(list, 2);
  NSAssert(pigeonResult.availableBandwidth != nil, @"");
  return pigeonResult;
}
+ (nullable NEFLTNERtcLastmileProbeOneWayResult *)nullableFromList:(NSArray *)list {
  return (list) ? [NEFLTNERtcLastmileProbeOneWayResult fromList:list] : nil;
}
- (NSArray *)toList {
  return @[
    (self.packetLossRate ?: [NSNull null]),
    (self.jitter ?: [NSNull null]),
    (self.availableBandwidth ?: [NSNull null]),
  ];
}
@end

@implementation NEFLTRtcServerAddresses
+ (instancetype)makeWithValid:(nullable NSNumber *)valid
                channelServer:(nullable NSString *)channelServer
             statisticsServer:(nullable NSString *)statisticsServer
                   roomServer:(nullable NSString *)roomServer
                 compatServer:(nullable NSString *)compatServer
                 nosLbsServer:(nullable NSString *)nosLbsServer
               nosUploadSever:(nullable NSString *)nosUploadSever
               nosTokenServer:(nullable NSString *)nosTokenServer
                      useIPv6:(nullable NSNumber *)useIPv6 {
  NEFLTRtcServerAddresses *pigeonResult = [[NEFLTRtcServerAddresses alloc] init];
  pigeonResult.valid = valid;
  pigeonResult.channelServer = channelServer;
  pigeonResult.statisticsServer = statisticsServer;
  pigeonResult.roomServer = roomServer;
  pigeonResult.compatServer = compatServer;
  pigeonResult.nosLbsServer = nosLbsServer;
  pigeonResult.nosUploadSever = nosUploadSever;
  pigeonResult.nosTokenServer = nosTokenServer;
  pigeonResult.useIPv6 = useIPv6;
  return pigeonResult;
}
+ (NEFLTRtcServerAddresses *)fromList:(NSArray *)list {
  NEFLTRtcServerAddresses *pigeonResult = [[NEFLTRtcServerAddresses alloc] init];
  pigeonResult.valid = GetNullableObjectAtIndex(list, 0);
  pigeonResult.channelServer = GetNullableObjectAtIndex(list, 1);
  pigeonResult.statisticsServer = GetNullableObjectAtIndex(list, 2);
  pigeonResult.roomServer = GetNullableObjectAtIndex(list, 3);
  pigeonResult.compatServer = GetNullableObjectAtIndex(list, 4);
  pigeonResult.nosLbsServer = GetNullableObjectAtIndex(list, 5);
  pigeonResult.nosUploadSever = GetNullableObjectAtIndex(list, 6);
  pigeonResult.nosTokenServer = GetNullableObjectAtIndex(list, 7);
  pigeonResult.useIPv6 = GetNullableObjectAtIndex(list, 8);
  return pigeonResult;
}
+ (nullable NEFLTRtcServerAddresses *)nullableFromList:(NSArray *)list {
  return (list) ? [NEFLTRtcServerAddresses fromList:list] : nil;
}
- (NSArray *)toList {
  return @[
    (self.valid ?: [NSNull null]),
    (self.channelServer ?: [NSNull null]),
    (self.statisticsServer ?: [NSNull null]),
    (self.roomServer ?: [NSNull null]),
    (self.compatServer ?: [NSNull null]),
    (self.nosLbsServer ?: [NSNull null]),
    (self.nosUploadSever ?: [NSNull null]),
    (self.nosTokenServer ?: [NSNull null]),
    (self.useIPv6 ?: [NSNull null]),
  ];
}
@end

@implementation NEFLTCreateEngineRequest
+ (instancetype)makeWithAppKey:(nullable NSString *)appKey
                                   logDir:(nullable NSString *)logDir
                          serverAddresses:(nullable NEFLTRtcServerAddresses *)serverAddresses
                                 logLevel:(nullable NSNumber *)logLevel
                       audioAutoSubscribe:(nullable NSNumber *)audioAutoSubscribe
                       videoAutoSubscribe:(nullable NSNumber *)videoAutoSubscribe
        disableFirstJoinUserCreateChannel:(nullable NSNumber *)disableFirstJoinUserCreateChannel
    audioDisableOverrideSpeakerOnReceiver:(nullable NSNumber *)audioDisableOverrideSpeakerOnReceiver
               audioDisableSWAECOnHeadset:(nullable NSNumber *)audioDisableSWAECOnHeadset
                         audioAINSEnabled:(nullable NSNumber *)audioAINSEnabled
                        serverRecordAudio:(nullable NSNumber *)serverRecordAudio
                        serverRecordVideo:(nullable NSNumber *)serverRecordVideo
                         serverRecordMode:(nullable NSNumber *)serverRecordMode
                      serverRecordSpeaker:(nullable NSNumber *)serverRecordSpeaker
                        publishSelfStream:(nullable NSNumber *)publishSelfStream
              videoCaptureObserverEnabled:(nullable NSNumber *)videoCaptureObserverEnabled
                          videoEncodeMode:(nullable NSNumber *)videoEncodeMode
                          videoDecodeMode:(nullable NSNumber *)videoDecodeMode
                            videoSendMode:(nullable NSNumber *)videoSendMode
                         videoH265Enabled:(nullable NSNumber *)videoH265Enabled
                           mode1v1Enabled:(nullable NSNumber *)mode1v1Enabled {
  NEFLTCreateEngineRequest *pigeonResult = [[NEFLTCreateEngineRequest alloc] init];
  pigeonResult.appKey = appKey;
  pigeonResult.logDir = logDir;
  pigeonResult.serverAddresses = serverAddresses;
  pigeonResult.logLevel = logLevel;
  pigeonResult.audioAutoSubscribe = audioAutoSubscribe;
  pigeonResult.videoAutoSubscribe = videoAutoSubscribe;
  pigeonResult.disableFirstJoinUserCreateChannel = disableFirstJoinUserCreateChannel;
  pigeonResult.audioDisableOverrideSpeakerOnReceiver = audioDisableOverrideSpeakerOnReceiver;
  pigeonResult.audioDisableSWAECOnHeadset = audioDisableSWAECOnHeadset;
  pigeonResult.audioAINSEnabled = audioAINSEnabled;
  pigeonResult.serverRecordAudio = serverRecordAudio;
  pigeonResult.serverRecordVideo = serverRecordVideo;
  pigeonResult.serverRecordMode = serverRecordMode;
  pigeonResult.serverRecordSpeaker = serverRecordSpeaker;
  pigeonResult.publishSelfStream = publishSelfStream;
  pigeonResult.videoCaptureObserverEnabled = videoCaptureObserverEnabled;
  pigeonResult.videoEncodeMode = videoEncodeMode;
  pigeonResult.videoDecodeMode = videoDecodeMode;
  pigeonResult.videoSendMode = videoSendMode;
  pigeonResult.videoH265Enabled = videoH265Enabled;
  pigeonResult.mode1v1Enabled = mode1v1Enabled;
  return pigeonResult;
}
+ (NEFLTCreateEngineRequest *)fromList:(NSArray *)list {
  NEFLTCreateEngineRequest *pigeonResult = [[NEFLTCreateEngineRequest alloc] init];
  pigeonResult.appKey = GetNullableObjectAtIndex(list, 0);
  pigeonResult.logDir = GetNullableObjectAtIndex(list, 1);
  pigeonResult.serverAddresses =
      [NEFLTRtcServerAddresses nullableFromList:(GetNullableObjectAtIndex(list, 2))];
  pigeonResult.logLevel = GetNullableObjectAtIndex(list, 3);
  pigeonResult.audioAutoSubscribe = GetNullableObjectAtIndex(list, 4);
  pigeonResult.videoAutoSubscribe = GetNullableObjectAtIndex(list, 5);
  pigeonResult.disableFirstJoinUserCreateChannel = GetNullableObjectAtIndex(list, 6);
  pigeonResult.audioDisableOverrideSpeakerOnReceiver = GetNullableObjectAtIndex(list, 7);
  pigeonResult.audioDisableSWAECOnHeadset = GetNullableObjectAtIndex(list, 8);
  pigeonResult.audioAINSEnabled = GetNullableObjectAtIndex(list, 9);
  pigeonResult.serverRecordAudio = GetNullableObjectAtIndex(list, 10);
  pigeonResult.serverRecordVideo = GetNullableObjectAtIndex(list, 11);
  pigeonResult.serverRecordMode = GetNullableObjectAtIndex(list, 12);
  pigeonResult.serverRecordSpeaker = GetNullableObjectAtIndex(list, 13);
  pigeonResult.publishSelfStream = GetNullableObjectAtIndex(list, 14);
  pigeonResult.videoCaptureObserverEnabled = GetNullableObjectAtIndex(list, 15);
  pigeonResult.videoEncodeMode = GetNullableObjectAtIndex(list, 16);
  pigeonResult.videoDecodeMode = GetNullableObjectAtIndex(list, 17);
  pigeonResult.videoSendMode = GetNullableObjectAtIndex(list, 18);
  pigeonResult.videoH265Enabled = GetNullableObjectAtIndex(list, 19);
  pigeonResult.mode1v1Enabled = GetNullableObjectAtIndex(list, 20);
  return pigeonResult;
}
+ (nullable NEFLTCreateEngineRequest *)nullableFromList:(NSArray *)list {
  return (list) ? [NEFLTCreateEngineRequest fromList:list] : nil;
}
- (NSArray *)toList {
  return @[
    (self.appKey ?: [NSNull null]),
    (self.logDir ?: [NSNull null]),
    (self.serverAddresses ? [self.serverAddresses toList] : [NSNull null]),
    (self.logLevel ?: [NSNull null]),
    (self.audioAutoSubscribe ?: [NSNull null]),
    (self.videoAutoSubscribe ?: [NSNull null]),
    (self.disableFirstJoinUserCreateChannel ?: [NSNull null]),
    (self.audioDisableOverrideSpeakerOnReceiver ?: [NSNull null]),
    (self.audioDisableSWAECOnHeadset ?: [NSNull null]),
    (self.audioAINSEnabled ?: [NSNull null]),
    (self.serverRecordAudio ?: [NSNull null]),
    (self.serverRecordVideo ?: [NSNull null]),
    (self.serverRecordMode ?: [NSNull null]),
    (self.serverRecordSpeaker ?: [NSNull null]),
    (self.publishSelfStream ?: [NSNull null]),
    (self.videoCaptureObserverEnabled ?: [NSNull null]),
    (self.videoEncodeMode ?: [NSNull null]),
    (self.videoDecodeMode ?: [NSNull null]),
    (self.videoSendMode ?: [NSNull null]),
    (self.videoH265Enabled ?: [NSNull null]),
    (self.mode1v1Enabled ?: [NSNull null]),
  ];
}
@end

@implementation NEFLTJoinChannelOptions
+ (instancetype)makeWithCustomInfo:(nullable NSString *)customInfo
                     permissionKey:(nullable NSString *)permissionKey {
  NEFLTJoinChannelOptions *pigeonResult = [[NEFLTJoinChannelOptions alloc] init];
  pigeonResult.customInfo = customInfo;
  pigeonResult.permissionKey = permissionKey;
  return pigeonResult;
}
+ (NEFLTJoinChannelOptions *)fromList:(NSArray *)list {
  NEFLTJoinChannelOptions *pigeonResult = [[NEFLTJoinChannelOptions alloc] init];
  pigeonResult.customInfo = GetNullableObjectAtIndex(list, 0);
  pigeonResult.permissionKey = GetNullableObjectAtIndex(list, 1);
  return pigeonResult;
}
+ (nullable NEFLTJoinChannelOptions *)nullableFromList:(NSArray *)list {
  return (list) ? [NEFLTJoinChannelOptions fromList:list] : nil;
}
- (NSArray *)toList {
  return @[
    (self.customInfo ?: [NSNull null]),
    (self.permissionKey ?: [NSNull null]),
  ];
}
@end

@implementation NEFLTJoinChannelRequest
+ (instancetype)makeWithToken:(nullable NSString *)token
                  channelName:(NSString *)channelName
                          uid:(NSNumber *)uid
               channelOptions:(nullable NEFLTJoinChannelOptions *)channelOptions {
  NEFLTJoinChannelRequest *pigeonResult = [[NEFLTJoinChannelRequest alloc] init];
  pigeonResult.token = token;
  pigeonResult.channelName = channelName;
  pigeonResult.uid = uid;
  pigeonResult.channelOptions = channelOptions;
  return pigeonResult;
}
+ (NEFLTJoinChannelRequest *)fromList:(NSArray *)list {
  NEFLTJoinChannelRequest *pigeonResult = [[NEFLTJoinChannelRequest alloc] init];
  pigeonResult.token = GetNullableObjectAtIndex(list, 0);
  pigeonResult.channelName = GetNullableObjectAtIndex(list, 1);
  NSAssert(pigeonResult.channelName != nil, @"");
  pigeonResult.uid = GetNullableObjectAtIndex(list, 2);
  NSAssert(pigeonResult.uid != nil, @"");
  pigeonResult.channelOptions =
      [NEFLTJoinChannelOptions nullableFromList:(GetNullableObjectAtIndex(list, 3))];
  return pigeonResult;
}
+ (nullable NEFLTJoinChannelRequest *)nullableFromList:(NSArray *)list {
  return (list) ? [NEFLTJoinChannelRequest fromList:list] : nil;
}
- (NSArray *)toList {
  return @[
    (self.token ?: [NSNull null]),
    (self.channelName ?: [NSNull null]),
    (self.uid ?: [NSNull null]),
    (self.channelOptions ? [self.channelOptions toList] : [NSNull null]),
  ];
}
@end

@implementation NEFLTSubscribeRemoteAudioRequest
+ (instancetype)makeWithUid:(nullable NSNumber *)uid subscribe:(nullable NSNumber *)subscribe {
  NEFLTSubscribeRemoteAudioRequest *pigeonResult = [[NEFLTSubscribeRemoteAudioRequest alloc] init];
  pigeonResult.uid = uid;
  pigeonResult.subscribe = subscribe;
  return pigeonResult;
}
+ (NEFLTSubscribeRemoteAudioRequest *)fromList:(NSArray *)list {
  NEFLTSubscribeRemoteAudioRequest *pigeonResult = [[NEFLTSubscribeRemoteAudioRequest alloc] init];
  pigeonResult.uid = GetNullableObjectAtIndex(list, 0);
  pigeonResult.subscribe = GetNullableObjectAtIndex(list, 1);
  return pigeonResult;
}
+ (nullable NEFLTSubscribeRemoteAudioRequest *)nullableFromList:(NSArray *)list {
  return (list) ? [NEFLTSubscribeRemoteAudioRequest fromList:list] : nil;
}
- (NSArray *)toList {
  return @[
    (self.uid ?: [NSNull null]),
    (self.subscribe ?: [NSNull null]),
  ];
}
@end

@implementation NEFLTEnableLocalVideoRequest
+ (instancetype)makeWithEnable:(nullable NSNumber *)enable
                    streamType:(nullable NSNumber *)streamType {
  NEFLTEnableLocalVideoRequest *pigeonResult = [[NEFLTEnableLocalVideoRequest alloc] init];
  pigeonResult.enable = enable;
  pigeonResult.streamType = streamType;
  return pigeonResult;
}
+ (NEFLTEnableLocalVideoRequest *)fromList:(NSArray *)list {
  NEFLTEnableLocalVideoRequest *pigeonResult = [[NEFLTEnableLocalVideoRequest alloc] init];
  pigeonResult.enable = GetNullableObjectAtIndex(list, 0);
  pigeonResult.streamType = GetNullableObjectAtIndex(list, 1);
  return pigeonResult;
}
+ (nullable NEFLTEnableLocalVideoRequest *)nullableFromList:(NSArray *)list {
  return (list) ? [NEFLTEnableLocalVideoRequest fromList:list] : nil;
}
- (NSArray *)toList {
  return @[
    (self.enable ?: [NSNull null]),
    (self.streamType ?: [NSNull null]),
  ];
}
@end

@implementation NEFLTSetAudioProfileRequest
+ (instancetype)makeWithProfile:(nullable NSNumber *)profile
                       scenario:(nullable NSNumber *)scenario {
  NEFLTSetAudioProfileRequest *pigeonResult = [[NEFLTSetAudioProfileRequest alloc] init];
  pigeonResult.profile = profile;
  pigeonResult.scenario = scenario;
  return pigeonResult;
}
+ (NEFLTSetAudioProfileRequest *)fromList:(NSArray *)list {
  NEFLTSetAudioProfileRequest *pigeonResult = [[NEFLTSetAudioProfileRequest alloc] init];
  pigeonResult.profile = GetNullableObjectAtIndex(list, 0);
  pigeonResult.scenario = GetNullableObjectAtIndex(list, 1);
  return pigeonResult;
}
+ (nullable NEFLTSetAudioProfileRequest *)nullableFromList:(NSArray *)list {
  return (list) ? [NEFLTSetAudioProfileRequest fromList:list] : nil;
}
- (NSArray *)toList {
  return @[
    (self.profile ?: [NSNull null]),
    (self.scenario ?: [NSNull null]),
  ];
}
@end

@implementation NEFLTSetLocalVideoConfigRequest
+ (instancetype)makeWithVideoProfile:(nullable NSNumber *)videoProfile
                       videoCropMode:(nullable NSNumber *)videoCropMode
                         frontCamera:(nullable NSNumber *)frontCamera
                           frameRate:(nullable NSNumber *)frameRate
                        minFrameRate:(nullable NSNumber *)minFrameRate
                             bitrate:(nullable NSNumber *)bitrate
                          minBitrate:(nullable NSNumber *)minBitrate
                   degradationPrefer:(nullable NSNumber *)degradationPrefer
                               width:(nullable NSNumber *)width
                              height:(nullable NSNumber *)height
                          cameraType:(nullable NSNumber *)cameraType
                          mirrorMode:(nullable NSNumber *)mirrorMode
                     orientationMode:(nullable NSNumber *)orientationMode
                          streamType:(nullable NSNumber *)streamType {
  NEFLTSetLocalVideoConfigRequest *pigeonResult = [[NEFLTSetLocalVideoConfigRequest alloc] init];
  pigeonResult.videoProfile = videoProfile;
  pigeonResult.videoCropMode = videoCropMode;
  pigeonResult.frontCamera = frontCamera;
  pigeonResult.frameRate = frameRate;
  pigeonResult.minFrameRate = minFrameRate;
  pigeonResult.bitrate = bitrate;
  pigeonResult.minBitrate = minBitrate;
  pigeonResult.degradationPrefer = degradationPrefer;
  pigeonResult.width = width;
  pigeonResult.height = height;
  pigeonResult.cameraType = cameraType;
  pigeonResult.mirrorMode = mirrorMode;
  pigeonResult.orientationMode = orientationMode;
  pigeonResult.streamType = streamType;
  return pigeonResult;
}
+ (NEFLTSetLocalVideoConfigRequest *)fromList:(NSArray *)list {
  NEFLTSetLocalVideoConfigRequest *pigeonResult = [[NEFLTSetLocalVideoConfigRequest alloc] init];
  pigeonResult.videoProfile = GetNullableObjectAtIndex(list, 0);
  pigeonResult.videoCropMode = GetNullableObjectAtIndex(list, 1);
  pigeonResult.frontCamera = GetNullableObjectAtIndex(list, 2);
  pigeonResult.frameRate = GetNullableObjectAtIndex(list, 3);
  pigeonResult.minFrameRate = GetNullableObjectAtIndex(list, 4);
  pigeonResult.bitrate = GetNullableObjectAtIndex(list, 5);
  pigeonResult.minBitrate = GetNullableObjectAtIndex(list, 6);
  pigeonResult.degradationPrefer = GetNullableObjectAtIndex(list, 7);
  pigeonResult.width = GetNullableObjectAtIndex(list, 8);
  pigeonResult.height = GetNullableObjectAtIndex(list, 9);
  pigeonResult.cameraType = GetNullableObjectAtIndex(list, 10);
  pigeonResult.mirrorMode = GetNullableObjectAtIndex(list, 11);
  pigeonResult.orientationMode = GetNullableObjectAtIndex(list, 12);
  pigeonResult.streamType = GetNullableObjectAtIndex(list, 13);
  return pigeonResult;
}
+ (nullable NEFLTSetLocalVideoConfigRequest *)nullableFromList:(NSArray *)list {
  return (list) ? [NEFLTSetLocalVideoConfigRequest fromList:list] : nil;
}
- (NSArray *)toList {
  return @[
    (self.videoProfile ?: [NSNull null]),
    (self.videoCropMode ?: [NSNull null]),
    (self.frontCamera ?: [NSNull null]),
    (self.frameRate ?: [NSNull null]),
    (self.minFrameRate ?: [NSNull null]),
    (self.bitrate ?: [NSNull null]),
    (self.minBitrate ?: [NSNull null]),
    (self.degradationPrefer ?: [NSNull null]),
    (self.width ?: [NSNull null]),
    (self.height ?: [NSNull null]),
    (self.cameraType ?: [NSNull null]),
    (self.mirrorMode ?: [NSNull null]),
    (self.orientationMode ?: [NSNull null]),
    (self.streamType ?: [NSNull null]),
  ];
}
@end

@implementation NEFLTSetCameraCaptureConfigRequest
+ (instancetype)makeWithExtraRotation:(NEFLTNERtcCaptureExtraRotation)extraRotation
                         captureWidth:(nullable NSNumber *)captureWidth
                        captureHeight:(nullable NSNumber *)captureHeight
                           streamType:(nullable NSNumber *)streamType {
  NEFLTSetCameraCaptureConfigRequest *pigeonResult =
      [[NEFLTSetCameraCaptureConfigRequest alloc] init];
  pigeonResult.extraRotation = extraRotation;
  pigeonResult.captureWidth = captureWidth;
  pigeonResult.captureHeight = captureHeight;
  pigeonResult.streamType = streamType;
  return pigeonResult;
}
+ (NEFLTSetCameraCaptureConfigRequest *)fromList:(NSArray *)list {
  NEFLTSetCameraCaptureConfigRequest *pigeonResult =
      [[NEFLTSetCameraCaptureConfigRequest alloc] init];
  pigeonResult.extraRotation = [GetNullableObjectAtIndex(list, 0) integerValue];
  pigeonResult.captureWidth = GetNullableObjectAtIndex(list, 1);
  pigeonResult.captureHeight = GetNullableObjectAtIndex(list, 2);
  pigeonResult.streamType = GetNullableObjectAtIndex(list, 3);
  return pigeonResult;
}
+ (nullable NEFLTSetCameraCaptureConfigRequest *)nullableFromList:(NSArray *)list {
  return (list) ? [NEFLTSetCameraCaptureConfigRequest fromList:list] : nil;
}
- (NSArray *)toList {
  return @[
    @(self.extraRotation),
    (self.captureWidth ?: [NSNull null]),
    (self.captureHeight ?: [NSNull null]),
    (self.streamType ?: [NSNull null]),
  ];
}
@end

@implementation NEFLTStartorStopVideoPreviewRequest
+ (instancetype)makeWithStreamType:(nullable NSNumber *)streamType {
  NEFLTStartorStopVideoPreviewRequest *pigeonResult =
      [[NEFLTStartorStopVideoPreviewRequest alloc] init];
  pigeonResult.streamType = streamType;
  return pigeonResult;
}
+ (NEFLTStartorStopVideoPreviewRequest *)fromList:(NSArray *)list {
  NEFLTStartorStopVideoPreviewRequest *pigeonResult =
      [[NEFLTStartorStopVideoPreviewRequest alloc] init];
  pigeonResult.streamType = GetNullableObjectAtIndex(list, 0);
  return pigeonResult;
}
+ (nullable NEFLTStartorStopVideoPreviewRequest *)nullableFromList:(NSArray *)list {
  return (list) ? [NEFLTStartorStopVideoPreviewRequest fromList:list] : nil;
}
- (NSArray *)toList {
  return @[
    (self.streamType ?: [NSNull null]),
  ];
}
@end

@implementation NEFLTStartScreenCaptureRequest
+ (instancetype)makeWithContentPrefer:(nullable NSNumber *)contentPrefer
                         videoProfile:(nullable NSNumber *)videoProfile
                            frameRate:(nullable NSNumber *)frameRate
                         minFrameRate:(nullable NSNumber *)minFrameRate
                              bitrate:(nullable NSNumber *)bitrate
                           minBitrate:(nullable NSNumber *)minBitrate {
  NEFLTStartScreenCaptureRequest *pigeonResult = [[NEFLTStartScreenCaptureRequest alloc] init];
  pigeonResult.contentPrefer = contentPrefer;
  pigeonResult.videoProfile = videoProfile;
  pigeonResult.frameRate = frameRate;
  pigeonResult.minFrameRate = minFrameRate;
  pigeonResult.bitrate = bitrate;
  pigeonResult.minBitrate = minBitrate;
  return pigeonResult;
}
+ (NEFLTStartScreenCaptureRequest *)fromList:(NSArray *)list {
  NEFLTStartScreenCaptureRequest *pigeonResult = [[NEFLTStartScreenCaptureRequest alloc] init];
  pigeonResult.contentPrefer = GetNullableObjectAtIndex(list, 0);
  pigeonResult.videoProfile = GetNullableObjectAtIndex(list, 1);
  pigeonResult.frameRate = GetNullableObjectAtIndex(list, 2);
  pigeonResult.minFrameRate = GetNullableObjectAtIndex(list, 3);
  pigeonResult.bitrate = GetNullableObjectAtIndex(list, 4);
  pigeonResult.minBitrate = GetNullableObjectAtIndex(list, 5);
  return pigeonResult;
}
+ (nullable NEFLTStartScreenCaptureRequest *)nullableFromList:(NSArray *)list {
  return (list) ? [NEFLTStartScreenCaptureRequest fromList:list] : nil;
}
- (NSArray *)toList {
  return @[
    (self.contentPrefer ?: [NSNull null]),
    (self.videoProfile ?: [NSNull null]),
    (self.frameRate ?: [NSNull null]),
    (self.minFrameRate ?: [NSNull null]),
    (self.bitrate ?: [NSNull null]),
    (self.minBitrate ?: [NSNull null]),
  ];
}
@end

@implementation NEFLTSubscribeRemoteVideoStreamRequest
+ (instancetype)makeWithUid:(nullable NSNumber *)uid
                 streamType:(nullable NSNumber *)streamType
                  subscribe:(nullable NSNumber *)subscribe {
  NEFLTSubscribeRemoteVideoStreamRequest *pigeonResult =
      [[NEFLTSubscribeRemoteVideoStreamRequest alloc] init];
  pigeonResult.uid = uid;
  pigeonResult.streamType = streamType;
  pigeonResult.subscribe = subscribe;
  return pigeonResult;
}
+ (NEFLTSubscribeRemoteVideoStreamRequest *)fromList:(NSArray *)list {
  NEFLTSubscribeRemoteVideoStreamRequest *pigeonResult =
      [[NEFLTSubscribeRemoteVideoStreamRequest alloc] init];
  pigeonResult.uid = GetNullableObjectAtIndex(list, 0);
  pigeonResult.streamType = GetNullableObjectAtIndex(list, 1);
  pigeonResult.subscribe = GetNullableObjectAtIndex(list, 2);
  return pigeonResult;
}
+ (nullable NEFLTSubscribeRemoteVideoStreamRequest *)nullableFromList:(NSArray *)list {
  return (list) ? [NEFLTSubscribeRemoteVideoStreamRequest fromList:list] : nil;
}
- (NSArray *)toList {
  return @[
    (self.uid ?: [NSNull null]),
    (self.streamType ?: [NSNull null]),
    (self.subscribe ?: [NSNull null]),
  ];
}
@end

@implementation NEFLTSubscribeRemoteSubStreamVideoRequest
+ (instancetype)makeWithUid:(nullable NSNumber *)uid subscribe:(nullable NSNumber *)subscribe {
  NEFLTSubscribeRemoteSubStreamVideoRequest *pigeonResult =
      [[NEFLTSubscribeRemoteSubStreamVideoRequest alloc] init];
  pigeonResult.uid = uid;
  pigeonResult.subscribe = subscribe;
  return pigeonResult;
}
+ (NEFLTSubscribeRemoteSubStreamVideoRequest *)fromList:(NSArray *)list {
  NEFLTSubscribeRemoteSubStreamVideoRequest *pigeonResult =
      [[NEFLTSubscribeRemoteSubStreamVideoRequest alloc] init];
  pigeonResult.uid = GetNullableObjectAtIndex(list, 0);
  pigeonResult.subscribe = GetNullableObjectAtIndex(list, 1);
  return pigeonResult;
}
+ (nullable NEFLTSubscribeRemoteSubStreamVideoRequest *)nullableFromList:(NSArray *)list {
  return (list) ? [NEFLTSubscribeRemoteSubStreamVideoRequest fromList:list] : nil;
}
- (NSArray *)toList {
  return @[
    (self.uid ?: [NSNull null]),
    (self.subscribe ?: [NSNull null]),
  ];
}
@end

@implementation NEFLTEnableAudioVolumeIndicationRequest
+ (instancetype)makeWithEnable:(nullable NSNumber *)enable
                      interval:(nullable NSNumber *)interval
                           vad:(nullable NSNumber *)vad {
  NEFLTEnableAudioVolumeIndicationRequest *pigeonResult =
      [[NEFLTEnableAudioVolumeIndicationRequest alloc] init];
  pigeonResult.enable = enable;
  pigeonResult.interval = interval;
  pigeonResult.vad = vad;
  return pigeonResult;
}
+ (NEFLTEnableAudioVolumeIndicationRequest *)fromList:(NSArray *)list {
  NEFLTEnableAudioVolumeIndicationRequest *pigeonResult =
      [[NEFLTEnableAudioVolumeIndicationRequest alloc] init];
  pigeonResult.enable = GetNullableObjectAtIndex(list, 0);
  pigeonResult.interval = GetNullableObjectAtIndex(list, 1);
  pigeonResult.vad = GetNullableObjectAtIndex(list, 2);
  return pigeonResult;
}
+ (nullable NEFLTEnableAudioVolumeIndicationRequest *)nullableFromList:(NSArray *)list {
  return (list) ? [NEFLTEnableAudioVolumeIndicationRequest fromList:list] : nil;
}
- (NSArray *)toList {
  return @[
    (self.enable ?: [NSNull null]),
    (self.interval ?: [NSNull null]),
    (self.vad ?: [NSNull null]),
  ];
}
@end

@implementation NEFLTSubscribeRemoteSubStreamAudioRequest
+ (instancetype)makeWithSubscribe:(nullable NSNumber *)subscribe uid:(nullable NSNumber *)uid {
  NEFLTSubscribeRemoteSubStreamAudioRequest *pigeonResult =
      [[NEFLTSubscribeRemoteSubStreamAudioRequest alloc] init];
  pigeonResult.subscribe = subscribe;
  pigeonResult.uid = uid;
  return pigeonResult;
}
+ (NEFLTSubscribeRemoteSubStreamAudioRequest *)fromList:(NSArray *)list {
  NEFLTSubscribeRemoteSubStreamAudioRequest *pigeonResult =
      [[NEFLTSubscribeRemoteSubStreamAudioRequest alloc] init];
  pigeonResult.subscribe = GetNullableObjectAtIndex(list, 0);
  pigeonResult.uid = GetNullableObjectAtIndex(list, 1);
  return pigeonResult;
}
+ (nullable NEFLTSubscribeRemoteSubStreamAudioRequest *)nullableFromList:(NSArray *)list {
  return (list) ? [NEFLTSubscribeRemoteSubStreamAudioRequest fromList:list] : nil;
}
- (NSArray *)toList {
  return @[
    (self.subscribe ?: [NSNull null]),
    (self.uid ?: [NSNull null]),
  ];
}
@end

@implementation NEFLTSetAudioSubscribeOnlyByRequest
+ (instancetype)makeWithUidArray:(nullable NSArray<NSNumber *> *)uidArray {
  NEFLTSetAudioSubscribeOnlyByRequest *pigeonResult =
      [[NEFLTSetAudioSubscribeOnlyByRequest alloc] init];
  pigeonResult.uidArray = uidArray;
  return pigeonResult;
}
+ (NEFLTSetAudioSubscribeOnlyByRequest *)fromList:(NSArray *)list {
  NEFLTSetAudioSubscribeOnlyByRequest *pigeonResult =
      [[NEFLTSetAudioSubscribeOnlyByRequest alloc] init];
  pigeonResult.uidArray = GetNullableObjectAtIndex(list, 0);
  return pigeonResult;
}
+ (nullable NEFLTSetAudioSubscribeOnlyByRequest *)nullableFromList:(NSArray *)list {
  return (list) ? [NEFLTSetAudioSubscribeOnlyByRequest fromList:list] : nil;
}
- (NSArray *)toList {
  return @[
    (self.uidArray ?: [NSNull null]),
  ];
}
@end

@implementation NEFLTStartAudioMixingRequest
+ (instancetype)makeWithPath:(nullable NSString *)path
                   loopCount:(nullable NSNumber *)loopCount
                 sendEnabled:(nullable NSNumber *)sendEnabled
                  sendVolume:(nullable NSNumber *)sendVolume
             playbackEnabled:(nullable NSNumber *)playbackEnabled
              playbackVolume:(nullable NSNumber *)playbackVolume
              startTimeStamp:(nullable NSNumber *)startTimeStamp
           sendWithAudioType:(nullable NSNumber *)sendWithAudioType
            progressInterval:(nullable NSNumber *)progressInterval {
  NEFLTStartAudioMixingRequest *pigeonResult = [[NEFLTStartAudioMixingRequest alloc] init];
  pigeonResult.path = path;
  pigeonResult.loopCount = loopCount;
  pigeonResult.sendEnabled = sendEnabled;
  pigeonResult.sendVolume = sendVolume;
  pigeonResult.playbackEnabled = playbackEnabled;
  pigeonResult.playbackVolume = playbackVolume;
  pigeonResult.startTimeStamp = startTimeStamp;
  pigeonResult.sendWithAudioType = sendWithAudioType;
  pigeonResult.progressInterval = progressInterval;
  return pigeonResult;
}
+ (NEFLTStartAudioMixingRequest *)fromList:(NSArray *)list {
  NEFLTStartAudioMixingRequest *pigeonResult = [[NEFLTStartAudioMixingRequest alloc] init];
  pigeonResult.path = GetNullableObjectAtIndex(list, 0);
  pigeonResult.loopCount = GetNullableObjectAtIndex(list, 1);
  pigeonResult.sendEnabled = GetNullableObjectAtIndex(list, 2);
  pigeonResult.sendVolume = GetNullableObjectAtIndex(list, 3);
  pigeonResult.playbackEnabled = GetNullableObjectAtIndex(list, 4);
  pigeonResult.playbackVolume = GetNullableObjectAtIndex(list, 5);
  pigeonResult.startTimeStamp = GetNullableObjectAtIndex(list, 6);
  pigeonResult.sendWithAudioType = GetNullableObjectAtIndex(list, 7);
  pigeonResult.progressInterval = GetNullableObjectAtIndex(list, 8);
  return pigeonResult;
}
+ (nullable NEFLTStartAudioMixingRequest *)nullableFromList:(NSArray *)list {
  return (list) ? [NEFLTStartAudioMixingRequest fromList:list] : nil;
}
- (NSArray *)toList {
  return @[
    (self.path ?: [NSNull null]),
    (self.loopCount ?: [NSNull null]),
    (self.sendEnabled ?: [NSNull null]),
    (self.sendVolume ?: [NSNull null]),
    (self.playbackEnabled ?: [NSNull null]),
    (self.playbackVolume ?: [NSNull null]),
    (self.startTimeStamp ?: [NSNull null]),
    (self.sendWithAudioType ?: [NSNull null]),
    (self.progressInterval ?: [NSNull null]),
  ];
}
@end

@implementation NEFLTPlayEffectRequest
+ (instancetype)makeWithEffectId:(nullable NSNumber *)effectId
                            path:(nullable NSString *)path
                       loopCount:(nullable NSNumber *)loopCount
                     sendEnabled:(nullable NSNumber *)sendEnabled
                      sendVolume:(nullable NSNumber *)sendVolume
                 playbackEnabled:(nullable NSNumber *)playbackEnabled
                  playbackVolume:(nullable NSNumber *)playbackVolume
                  startTimestamp:(nullable NSNumber *)startTimestamp
               sendWithAudioType:(nullable NSNumber *)sendWithAudioType
                progressInterval:(nullable NSNumber *)progressInterval {
  NEFLTPlayEffectRequest *pigeonResult = [[NEFLTPlayEffectRequest alloc] init];
  pigeonResult.effectId = effectId;
  pigeonResult.path = path;
  pigeonResult.loopCount = loopCount;
  pigeonResult.sendEnabled = sendEnabled;
  pigeonResult.sendVolume = sendVolume;
  pigeonResult.playbackEnabled = playbackEnabled;
  pigeonResult.playbackVolume = playbackVolume;
  pigeonResult.startTimestamp = startTimestamp;
  pigeonResult.sendWithAudioType = sendWithAudioType;
  pigeonResult.progressInterval = progressInterval;
  return pigeonResult;
}
+ (NEFLTPlayEffectRequest *)fromList:(NSArray *)list {
  NEFLTPlayEffectRequest *pigeonResult = [[NEFLTPlayEffectRequest alloc] init];
  pigeonResult.effectId = GetNullableObjectAtIndex(list, 0);
  pigeonResult.path = GetNullableObjectAtIndex(list, 1);
  pigeonResult.loopCount = GetNullableObjectAtIndex(list, 2);
  pigeonResult.sendEnabled = GetNullableObjectAtIndex(list, 3);
  pigeonResult.sendVolume = GetNullableObjectAtIndex(list, 4);
  pigeonResult.playbackEnabled = GetNullableObjectAtIndex(list, 5);
  pigeonResult.playbackVolume = GetNullableObjectAtIndex(list, 6);
  pigeonResult.startTimestamp = GetNullableObjectAtIndex(list, 7);
  pigeonResult.sendWithAudioType = GetNullableObjectAtIndex(list, 8);
  pigeonResult.progressInterval = GetNullableObjectAtIndex(list, 9);
  return pigeonResult;
}
+ (nullable NEFLTPlayEffectRequest *)nullableFromList:(NSArray *)list {
  return (list) ? [NEFLTPlayEffectRequest fromList:list] : nil;
}
- (NSArray *)toList {
  return @[
    (self.effectId ?: [NSNull null]),
    (self.path ?: [NSNull null]),
    (self.loopCount ?: [NSNull null]),
    (self.sendEnabled ?: [NSNull null]),
    (self.sendVolume ?: [NSNull null]),
    (self.playbackEnabled ?: [NSNull null]),
    (self.playbackVolume ?: [NSNull null]),
    (self.startTimestamp ?: [NSNull null]),
    (self.sendWithAudioType ?: [NSNull null]),
    (self.progressInterval ?: [NSNull null]),
  ];
}
@end

@implementation NEFLTSetCameraPositionRequest
+ (instancetype)makeWithX:(nullable NSNumber *)x y:(nullable NSNumber *)y {
  NEFLTSetCameraPositionRequest *pigeonResult = [[NEFLTSetCameraPositionRequest alloc] init];
  pigeonResult.x = x;
  pigeonResult.y = y;
  return pigeonResult;
}
+ (NEFLTSetCameraPositionRequest *)fromList:(NSArray *)list {
  NEFLTSetCameraPositionRequest *pigeonResult = [[NEFLTSetCameraPositionRequest alloc] init];
  pigeonResult.x = GetNullableObjectAtIndex(list, 0);
  pigeonResult.y = GetNullableObjectAtIndex(list, 1);
  return pigeonResult;
}
+ (nullable NEFLTSetCameraPositionRequest *)nullableFromList:(NSArray *)list {
  return (list) ? [NEFLTSetCameraPositionRequest fromList:list] : nil;
}
- (NSArray *)toList {
  return @[
    (self.x ?: [NSNull null]),
    (self.y ?: [NSNull null]),
  ];
}
@end

@implementation NEFLTAddOrUpdateLiveStreamTaskRequest
+ (instancetype)makeWithSerial:(nullable NSNumber *)serial
                        taskId:(nullable NSString *)taskId
                           url:(nullable NSString *)url
           serverRecordEnabled:(nullable NSNumber *)serverRecordEnabled
                      liveMode:(nullable NSNumber *)liveMode
                   layoutWidth:(nullable NSNumber *)layoutWidth
                  layoutHeight:(nullable NSNumber *)layoutHeight
         layoutBackgroundColor:(nullable NSNumber *)layoutBackgroundColor
                layoutImageUrl:(nullable NSString *)layoutImageUrl
                  layoutImageX:(nullable NSNumber *)layoutImageX
                  layoutImageY:(nullable NSNumber *)layoutImageY
              layoutImageWidth:(nullable NSNumber *)layoutImageWidth
             layoutImageHeight:(nullable NSNumber *)layoutImageHeight
     layoutUserTranscodingList:(nullable NSArray *)layoutUserTranscodingList {
  NEFLTAddOrUpdateLiveStreamTaskRequest *pigeonResult =
      [[NEFLTAddOrUpdateLiveStreamTaskRequest alloc] init];
  pigeonResult.serial = serial;
  pigeonResult.taskId = taskId;
  pigeonResult.url = url;
  pigeonResult.serverRecordEnabled = serverRecordEnabled;
  pigeonResult.liveMode = liveMode;
  pigeonResult.layoutWidth = layoutWidth;
  pigeonResult.layoutHeight = layoutHeight;
  pigeonResult.layoutBackgroundColor = layoutBackgroundColor;
  pigeonResult.layoutImageUrl = layoutImageUrl;
  pigeonResult.layoutImageX = layoutImageX;
  pigeonResult.layoutImageY = layoutImageY;
  pigeonResult.layoutImageWidth = layoutImageWidth;
  pigeonResult.layoutImageHeight = layoutImageHeight;
  pigeonResult.layoutUserTranscodingList = layoutUserTranscodingList;
  return pigeonResult;
}
+ (NEFLTAddOrUpdateLiveStreamTaskRequest *)fromList:(NSArray *)list {
  NEFLTAddOrUpdateLiveStreamTaskRequest *pigeonResult =
      [[NEFLTAddOrUpdateLiveStreamTaskRequest alloc] init];
  pigeonResult.serial = GetNullableObjectAtIndex(list, 0);
  pigeonResult.taskId = GetNullableObjectAtIndex(list, 1);
  pigeonResult.url = GetNullableObjectAtIndex(list, 2);
  pigeonResult.serverRecordEnabled = GetNullableObjectAtIndex(list, 3);
  pigeonResult.liveMode = GetNullableObjectAtIndex(list, 4);
  pigeonResult.layoutWidth = GetNullableObjectAtIndex(list, 5);
  pigeonResult.layoutHeight = GetNullableObjectAtIndex(list, 6);
  pigeonResult.layoutBackgroundColor = GetNullableObjectAtIndex(list, 7);
  pigeonResult.layoutImageUrl = GetNullableObjectAtIndex(list, 8);
  pigeonResult.layoutImageX = GetNullableObjectAtIndex(list, 9);
  pigeonResult.layoutImageY = GetNullableObjectAtIndex(list, 10);
  pigeonResult.layoutImageWidth = GetNullableObjectAtIndex(list, 11);
  pigeonResult.layoutImageHeight = GetNullableObjectAtIndex(list, 12);
  pigeonResult.layoutUserTranscodingList = GetNullableObjectAtIndex(list, 13);
  return pigeonResult;
}
+ (nullable NEFLTAddOrUpdateLiveStreamTaskRequest *)nullableFromList:(NSArray *)list {
  return (list) ? [NEFLTAddOrUpdateLiveStreamTaskRequest fromList:list] : nil;
}
- (NSArray *)toList {
  return @[
    (self.serial ?: [NSNull null]),
    (self.taskId ?: [NSNull null]),
    (self.url ?: [NSNull null]),
    (self.serverRecordEnabled ?: [NSNull null]),
    (self.liveMode ?: [NSNull null]),
    (self.layoutWidth ?: [NSNull null]),
    (self.layoutHeight ?: [NSNull null]),
    (self.layoutBackgroundColor ?: [NSNull null]),
    (self.layoutImageUrl ?: [NSNull null]),
    (self.layoutImageX ?: [NSNull null]),
    (self.layoutImageY ?: [NSNull null]),
    (self.layoutImageWidth ?: [NSNull null]),
    (self.layoutImageHeight ?: [NSNull null]),
    (self.layoutUserTranscodingList ?: [NSNull null]),
  ];
}
@end

@implementation NEFLTDeleteLiveStreamTaskRequest
+ (instancetype)makeWithSerial:(nullable NSNumber *)serial taskId:(nullable NSString *)taskId {
  NEFLTDeleteLiveStreamTaskRequest *pigeonResult = [[NEFLTDeleteLiveStreamTaskRequest alloc] init];
  pigeonResult.serial = serial;
  pigeonResult.taskId = taskId;
  return pigeonResult;
}
+ (NEFLTDeleteLiveStreamTaskRequest *)fromList:(NSArray *)list {
  NEFLTDeleteLiveStreamTaskRequest *pigeonResult = [[NEFLTDeleteLiveStreamTaskRequest alloc] init];
  pigeonResult.serial = GetNullableObjectAtIndex(list, 0);
  pigeonResult.taskId = GetNullableObjectAtIndex(list, 1);
  return pigeonResult;
}
+ (nullable NEFLTDeleteLiveStreamTaskRequest *)nullableFromList:(NSArray *)list {
  return (list) ? [NEFLTDeleteLiveStreamTaskRequest fromList:list] : nil;
}
- (NSArray *)toList {
  return @[
    (self.serial ?: [NSNull null]),
    (self.taskId ?: [NSNull null]),
  ];
}
@end

@implementation NEFLTSendSEIMsgRequest
+ (instancetype)makeWithSeiMsg:(nullable NSString *)seiMsg
                    streamType:(nullable NSNumber *)streamType {
  NEFLTSendSEIMsgRequest *pigeonResult = [[NEFLTSendSEIMsgRequest alloc] init];
  pigeonResult.seiMsg = seiMsg;
  pigeonResult.streamType = streamType;
  return pigeonResult;
}
+ (NEFLTSendSEIMsgRequest *)fromList:(NSArray *)list {
  NEFLTSendSEIMsgRequest *pigeonResult = [[NEFLTSendSEIMsgRequest alloc] init];
  pigeonResult.seiMsg = GetNullableObjectAtIndex(list, 0);
  pigeonResult.streamType = GetNullableObjectAtIndex(list, 1);
  return pigeonResult;
}
+ (nullable NEFLTSendSEIMsgRequest *)nullableFromList:(NSArray *)list {
  return (list) ? [NEFLTSendSEIMsgRequest fromList:list] : nil;
}
- (NSArray *)toList {
  return @[
    (self.seiMsg ?: [NSNull null]),
    (self.streamType ?: [NSNull null]),
  ];
}
@end

@implementation NEFLTSetLocalVoiceEqualizationRequest
+ (instancetype)makeWithBandFrequency:(nullable NSNumber *)bandFrequency
                             bandGain:(nullable NSNumber *)bandGain {
  NEFLTSetLocalVoiceEqualizationRequest *pigeonResult =
      [[NEFLTSetLocalVoiceEqualizationRequest alloc] init];
  pigeonResult.bandFrequency = bandFrequency;
  pigeonResult.bandGain = bandGain;
  return pigeonResult;
}
+ (NEFLTSetLocalVoiceEqualizationRequest *)fromList:(NSArray *)list {
  NEFLTSetLocalVoiceEqualizationRequest *pigeonResult =
      [[NEFLTSetLocalVoiceEqualizationRequest alloc] init];
  pigeonResult.bandFrequency = GetNullableObjectAtIndex(list, 0);
  pigeonResult.bandGain = GetNullableObjectAtIndex(list, 1);
  return pigeonResult;
}
+ (nullable NEFLTSetLocalVoiceEqualizationRequest *)nullableFromList:(NSArray *)list {
  return (list) ? [NEFLTSetLocalVoiceEqualizationRequest fromList:list] : nil;
}
- (NSArray *)toList {
  return @[
    (self.bandFrequency ?: [NSNull null]),
    (self.bandGain ?: [NSNull null]),
  ];
}
@end

@implementation NEFLTSwitchChannelRequest
+ (instancetype)makeWithToken:(nullable NSString *)token
                  channelName:(nullable NSString *)channelName
               channelOptions:(nullable NEFLTJoinChannelOptions *)channelOptions {
  NEFLTSwitchChannelRequest *pigeonResult = [[NEFLTSwitchChannelRequest alloc] init];
  pigeonResult.token = token;
  pigeonResult.channelName = channelName;
  pigeonResult.channelOptions = channelOptions;
  return pigeonResult;
}
+ (NEFLTSwitchChannelRequest *)fromList:(NSArray *)list {
  NEFLTSwitchChannelRequest *pigeonResult = [[NEFLTSwitchChannelRequest alloc] init];
  pigeonResult.token = GetNullableObjectAtIndex(list, 0);
  pigeonResult.channelName = GetNullableObjectAtIndex(list, 1);
  pigeonResult.channelOptions =
      [NEFLTJoinChannelOptions nullableFromList:(GetNullableObjectAtIndex(list, 2))];
  return pigeonResult;
}
+ (nullable NEFLTSwitchChannelRequest *)nullableFromList:(NSArray *)list {
  return (list) ? [NEFLTSwitchChannelRequest fromList:list] : nil;
}
- (NSArray *)toList {
  return @[
    (self.token ?: [NSNull null]),
    (self.channelName ?: [NSNull null]),
    (self.channelOptions ? [self.channelOptions toList] : [NSNull null]),
  ];
}
@end

@implementation NEFLTStartAudioRecordingRequest
+ (instancetype)makeWithFilePath:(nullable NSString *)filePath
                      sampleRate:(nullable NSNumber *)sampleRate
                         quality:(nullable NSNumber *)quality {
  NEFLTStartAudioRecordingRequest *pigeonResult = [[NEFLTStartAudioRecordingRequest alloc] init];
  pigeonResult.filePath = filePath;
  pigeonResult.sampleRate = sampleRate;
  pigeonResult.quality = quality;
  return pigeonResult;
}
+ (NEFLTStartAudioRecordingRequest *)fromList:(NSArray *)list {
  NEFLTStartAudioRecordingRequest *pigeonResult = [[NEFLTStartAudioRecordingRequest alloc] init];
  pigeonResult.filePath = GetNullableObjectAtIndex(list, 0);
  pigeonResult.sampleRate = GetNullableObjectAtIndex(list, 1);
  pigeonResult.quality = GetNullableObjectAtIndex(list, 2);
  return pigeonResult;
}
+ (nullable NEFLTStartAudioRecordingRequest *)nullableFromList:(NSArray *)list {
  return (list) ? [NEFLTStartAudioRecordingRequest fromList:list] : nil;
}
- (NSArray *)toList {
  return @[
    (self.filePath ?: [NSNull null]),
    (self.sampleRate ?: [NSNull null]),
    (self.quality ?: [NSNull null]),
  ];
}
@end

@implementation NEFLTAudioRecordingConfigurationRequest
+ (instancetype)makeWithFilePath:(nullable NSString *)filePath
                      sampleRate:(nullable NSNumber *)sampleRate
                         quality:(nullable NSNumber *)quality
                        position:(nullable NSNumber *)position
                       cycleTime:(nullable NSNumber *)cycleTime {
  NEFLTAudioRecordingConfigurationRequest *pigeonResult =
      [[NEFLTAudioRecordingConfigurationRequest alloc] init];
  pigeonResult.filePath = filePath;
  pigeonResult.sampleRate = sampleRate;
  pigeonResult.quality = quality;
  pigeonResult.position = position;
  pigeonResult.cycleTime = cycleTime;
  return pigeonResult;
}
+ (NEFLTAudioRecordingConfigurationRequest *)fromList:(NSArray *)list {
  NEFLTAudioRecordingConfigurationRequest *pigeonResult =
      [[NEFLTAudioRecordingConfigurationRequest alloc] init];
  pigeonResult.filePath = GetNullableObjectAtIndex(list, 0);
  pigeonResult.sampleRate = GetNullableObjectAtIndex(list, 1);
  pigeonResult.quality = GetNullableObjectAtIndex(list, 2);
  pigeonResult.position = GetNullableObjectAtIndex(list, 3);
  pigeonResult.cycleTime = GetNullableObjectAtIndex(list, 4);
  return pigeonResult;
}
+ (nullable NEFLTAudioRecordingConfigurationRequest *)nullableFromList:(NSArray *)list {
  return (list) ? [NEFLTAudioRecordingConfigurationRequest fromList:list] : nil;
}
- (NSArray *)toList {
  return @[
    (self.filePath ?: [NSNull null]),
    (self.sampleRate ?: [NSNull null]),
    (self.quality ?: [NSNull null]),
    (self.position ?: [NSNull null]),
    (self.cycleTime ?: [NSNull null]),
  ];
}
@end

@implementation NEFLTSetLocalMediaPriorityRequest
+ (instancetype)makeWithPriority:(nullable NSNumber *)priority
                    isPreemptive:(nullable NSNumber *)isPreemptive {
  NEFLTSetLocalMediaPriorityRequest *pigeonResult =
      [[NEFLTSetLocalMediaPriorityRequest alloc] init];
  pigeonResult.priority = priority;
  pigeonResult.isPreemptive = isPreemptive;
  return pigeonResult;
}
+ (NEFLTSetLocalMediaPriorityRequest *)fromList:(NSArray *)list {
  NEFLTSetLocalMediaPriorityRequest *pigeonResult =
      [[NEFLTSetLocalMediaPriorityRequest alloc] init];
  pigeonResult.priority = GetNullableObjectAtIndex(list, 0);
  pigeonResult.isPreemptive = GetNullableObjectAtIndex(list, 1);
  return pigeonResult;
}
+ (nullable NEFLTSetLocalMediaPriorityRequest *)nullableFromList:(NSArray *)list {
  return (list) ? [NEFLTSetLocalMediaPriorityRequest fromList:list] : nil;
}
- (NSArray *)toList {
  return @[
    (self.priority ?: [NSNull null]),
    (self.isPreemptive ?: [NSNull null]),
  ];
}
@end

@implementation NEFLTStartOrUpdateChannelMediaRelayRequest
+ (instancetype)makeWithSourceMediaInfo:(nullable NSDictionary<id, id> *)sourceMediaInfo
                          destMediaInfo:
                              (nullable NSDictionary<id, NSDictionary<id, id> *> *)destMediaInfo {
  NEFLTStartOrUpdateChannelMediaRelayRequest *pigeonResult =
      [[NEFLTStartOrUpdateChannelMediaRelayRequest alloc] init];
  pigeonResult.sourceMediaInfo = sourceMediaInfo;
  pigeonResult.destMediaInfo = destMediaInfo;
  return pigeonResult;
}
+ (NEFLTStartOrUpdateChannelMediaRelayRequest *)fromList:(NSArray *)list {
  NEFLTStartOrUpdateChannelMediaRelayRequest *pigeonResult =
      [[NEFLTStartOrUpdateChannelMediaRelayRequest alloc] init];
  pigeonResult.sourceMediaInfo = GetNullableObjectAtIndex(list, 0);
  pigeonResult.destMediaInfo = GetNullableObjectAtIndex(list, 1);
  return pigeonResult;
}
+ (nullable NEFLTStartOrUpdateChannelMediaRelayRequest *)nullableFromList:(NSArray *)list {
  return (list) ? [NEFLTStartOrUpdateChannelMediaRelayRequest fromList:list] : nil;
}
- (NSArray *)toList {
  return @[
    (self.sourceMediaInfo ?: [NSNull null]),
    (self.destMediaInfo ?: [NSNull null]),
  ];
}
@end

@implementation NEFLTAdjustUserPlaybackSignalVolumeRequest
+ (instancetype)makeWithUid:(nullable NSNumber *)uid volume:(nullable NSNumber *)volume {
  NEFLTAdjustUserPlaybackSignalVolumeRequest *pigeonResult =
      [[NEFLTAdjustUserPlaybackSignalVolumeRequest alloc] init];
  pigeonResult.uid = uid;
  pigeonResult.volume = volume;
  return pigeonResult;
}
+ (NEFLTAdjustUserPlaybackSignalVolumeRequest *)fromList:(NSArray *)list {
  NEFLTAdjustUserPlaybackSignalVolumeRequest *pigeonResult =
      [[NEFLTAdjustUserPlaybackSignalVolumeRequest alloc] init];
  pigeonResult.uid = GetNullableObjectAtIndex(list, 0);
  pigeonResult.volume = GetNullableObjectAtIndex(list, 1);
  return pigeonResult;
}
+ (nullable NEFLTAdjustUserPlaybackSignalVolumeRequest *)nullableFromList:(NSArray *)list {
  return (list) ? [NEFLTAdjustUserPlaybackSignalVolumeRequest fromList:list] : nil;
}
- (NSArray *)toList {
  return @[
    (self.uid ?: [NSNull null]),
    (self.volume ?: [NSNull null]),
  ];
}
@end

@implementation NEFLTEnableEncryptionRequest
+ (instancetype)makeWithKey:(nullable NSString *)key
                       mode:(nullable NSNumber *)mode
                     enable:(nullable NSNumber *)enable {
  NEFLTEnableEncryptionRequest *pigeonResult = [[NEFLTEnableEncryptionRequest alloc] init];
  pigeonResult.key = key;
  pigeonResult.mode = mode;
  pigeonResult.enable = enable;
  return pigeonResult;
}
+ (NEFLTEnableEncryptionRequest *)fromList:(NSArray *)list {
  NEFLTEnableEncryptionRequest *pigeonResult = [[NEFLTEnableEncryptionRequest alloc] init];
  pigeonResult.key = GetNullableObjectAtIndex(list, 0);
  pigeonResult.mode = GetNullableObjectAtIndex(list, 1);
  pigeonResult.enable = GetNullableObjectAtIndex(list, 2);
  return pigeonResult;
}
+ (nullable NEFLTEnableEncryptionRequest *)nullableFromList:(NSArray *)list {
  return (list) ? [NEFLTEnableEncryptionRequest fromList:list] : nil;
}
- (NSArray *)toList {
  return @[
    (self.key ?: [NSNull null]),
    (self.mode ?: [NSNull null]),
    (self.enable ?: [NSNull null]),
  ];
}
@end

@implementation NEFLTSetLocalVoiceReverbParamRequest
+ (instancetype)makeWithWetGain:(nullable NSNumber *)wetGain
                        dryGain:(nullable NSNumber *)dryGain
                        damping:(nullable NSNumber *)damping
                       roomSize:(nullable NSNumber *)roomSize
                      decayTime:(nullable NSNumber *)decayTime
                       preDelay:(nullable NSNumber *)preDelay {
  NEFLTSetLocalVoiceReverbParamRequest *pigeonResult =
      [[NEFLTSetLocalVoiceReverbParamRequest alloc] init];
  pigeonResult.wetGain = wetGain;
  pigeonResult.dryGain = dryGain;
  pigeonResult.damping = damping;
  pigeonResult.roomSize = roomSize;
  pigeonResult.decayTime = decayTime;
  pigeonResult.preDelay = preDelay;
  return pigeonResult;
}
+ (NEFLTSetLocalVoiceReverbParamRequest *)fromList:(NSArray *)list {
  NEFLTSetLocalVoiceReverbParamRequest *pigeonResult =
      [[NEFLTSetLocalVoiceReverbParamRequest alloc] init];
  pigeonResult.wetGain = GetNullableObjectAtIndex(list, 0);
  pigeonResult.dryGain = GetNullableObjectAtIndex(list, 1);
  pigeonResult.damping = GetNullableObjectAtIndex(list, 2);
  pigeonResult.roomSize = GetNullableObjectAtIndex(list, 3);
  pigeonResult.decayTime = GetNullableObjectAtIndex(list, 4);
  pigeonResult.preDelay = GetNullableObjectAtIndex(list, 5);
  return pigeonResult;
}
+ (nullable NEFLTSetLocalVoiceReverbParamRequest *)nullableFromList:(NSArray *)list {
  return (list) ? [NEFLTSetLocalVoiceReverbParamRequest fromList:list] : nil;
}
- (NSArray *)toList {
  return @[
    (self.wetGain ?: [NSNull null]),
    (self.dryGain ?: [NSNull null]),
    (self.damping ?: [NSNull null]),
    (self.roomSize ?: [NSNull null]),
    (self.decayTime ?: [NSNull null]),
    (self.preDelay ?: [NSNull null]),
  ];
}
@end

@implementation NEFLTReportCustomEventRequest
+ (instancetype)makeWithEventName:(nullable NSString *)eventName
                   customIdentify:(nullable NSString *)customIdentify
                            param:(nullable NSDictionary<NSString *, id> *)param {
  NEFLTReportCustomEventRequest *pigeonResult = [[NEFLTReportCustomEventRequest alloc] init];
  pigeonResult.eventName = eventName;
  pigeonResult.customIdentify = customIdentify;
  pigeonResult.param = param;
  return pigeonResult;
}
+ (NEFLTReportCustomEventRequest *)fromList:(NSArray *)list {
  NEFLTReportCustomEventRequest *pigeonResult = [[NEFLTReportCustomEventRequest alloc] init];
  pigeonResult.eventName = GetNullableObjectAtIndex(list, 0);
  pigeonResult.customIdentify = GetNullableObjectAtIndex(list, 1);
  pigeonResult.param = GetNullableObjectAtIndex(list, 2);
  return pigeonResult;
}
+ (nullable NEFLTReportCustomEventRequest *)nullableFromList:(NSArray *)list {
  return (list) ? [NEFLTReportCustomEventRequest fromList:list] : nil;
}
- (NSArray *)toList {
  return @[
    (self.eventName ?: [NSNull null]),
    (self.customIdentify ?: [NSNull null]),
    (self.param ?: [NSNull null]),
  ];
}
@end

@implementation NEFLTStartLastmileProbeTestRequest
+ (instancetype)makeWithProbeUplink:(nullable NSNumber *)probeUplink
                      probeDownlink:(nullable NSNumber *)probeDownlink
              expectedUplinkBitrate:(nullable NSNumber *)expectedUplinkBitrate
            expectedDownlinkBitrate:(nullable NSNumber *)expectedDownlinkBitrate {
  NEFLTStartLastmileProbeTestRequest *pigeonResult =
      [[NEFLTStartLastmileProbeTestRequest alloc] init];
  pigeonResult.probeUplink = probeUplink;
  pigeonResult.probeDownlink = probeDownlink;
  pigeonResult.expectedUplinkBitrate = expectedUplinkBitrate;
  pigeonResult.expectedDownlinkBitrate = expectedDownlinkBitrate;
  return pigeonResult;
}
+ (NEFLTStartLastmileProbeTestRequest *)fromList:(NSArray *)list {
  NEFLTStartLastmileProbeTestRequest *pigeonResult =
      [[NEFLTStartLastmileProbeTestRequest alloc] init];
  pigeonResult.probeUplink = GetNullableObjectAtIndex(list, 0);
  pigeonResult.probeDownlink = GetNullableObjectAtIndex(list, 1);
  pigeonResult.expectedUplinkBitrate = GetNullableObjectAtIndex(list, 2);
  pigeonResult.expectedDownlinkBitrate = GetNullableObjectAtIndex(list, 3);
  return pigeonResult;
}
+ (nullable NEFLTStartLastmileProbeTestRequest *)nullableFromList:(NSArray *)list {
  return (list) ? [NEFLTStartLastmileProbeTestRequest fromList:list] : nil;
}
- (NSArray *)toList {
  return @[
    (self.probeUplink ?: [NSNull null]),
    (self.probeDownlink ?: [NSNull null]),
    (self.expectedUplinkBitrate ?: [NSNull null]),
    (self.expectedDownlinkBitrate ?: [NSNull null]),
  ];
}
@end

@implementation NEFLTCGPoint
+ (instancetype)makeWithX:(NSNumber *)x y:(NSNumber *)y {
  NEFLTCGPoint *pigeonResult = [[NEFLTCGPoint alloc] init];
  pigeonResult.x = x;
  pigeonResult.y = y;
  return pigeonResult;
}
+ (NEFLTCGPoint *)fromList:(NSArray *)list {
  NEFLTCGPoint *pigeonResult = [[NEFLTCGPoint alloc] init];
  pigeonResult.x = GetNullableObjectAtIndex(list, 0);
  NSAssert(pigeonResult.x != nil, @"");
  pigeonResult.y = GetNullableObjectAtIndex(list, 1);
  NSAssert(pigeonResult.y != nil, @"");
  return pigeonResult;
}
+ (nullable NEFLTCGPoint *)nullableFromList:(NSArray *)list {
  return (list) ? [NEFLTCGPoint fromList:list] : nil;
}
- (NSArray *)toList {
  return @[
    (self.x ?: [NSNull null]),
    (self.y ?: [NSNull null]),
  ];
}
@end

@implementation NEFLTSetVideoCorrectionConfigRequest
+ (instancetype)makeWithTopLeft:(nullable NEFLTCGPoint *)topLeft
                       topRight:(nullable NEFLTCGPoint *)topRight
                     bottomLeft:(nullable NEFLTCGPoint *)bottomLeft
                    bottomRight:(nullable NEFLTCGPoint *)bottomRight
                    canvasWidth:(nullable NSNumber *)canvasWidth
                   canvasHeight:(nullable NSNumber *)canvasHeight
                   enableMirror:(nullable NSNumber *)enableMirror {
  NEFLTSetVideoCorrectionConfigRequest *pigeonResult =
      [[NEFLTSetVideoCorrectionConfigRequest alloc] init];
  pigeonResult.topLeft = topLeft;
  pigeonResult.topRight = topRight;
  pigeonResult.bottomLeft = bottomLeft;
  pigeonResult.bottomRight = bottomRight;
  pigeonResult.canvasWidth = canvasWidth;
  pigeonResult.canvasHeight = canvasHeight;
  pigeonResult.enableMirror = enableMirror;
  return pigeonResult;
}
+ (NEFLTSetVideoCorrectionConfigRequest *)fromList:(NSArray *)list {
  NEFLTSetVideoCorrectionConfigRequest *pigeonResult =
      [[NEFLTSetVideoCorrectionConfigRequest alloc] init];
  pigeonResult.topLeft = [NEFLTCGPoint nullableFromList:(GetNullableObjectAtIndex(list, 0))];
  pigeonResult.topRight = [NEFLTCGPoint nullableFromList:(GetNullableObjectAtIndex(list, 1))];
  pigeonResult.bottomLeft = [NEFLTCGPoint nullableFromList:(GetNullableObjectAtIndex(list, 2))];
  pigeonResult.bottomRight = [NEFLTCGPoint nullableFromList:(GetNullableObjectAtIndex(list, 3))];
  pigeonResult.canvasWidth = GetNullableObjectAtIndex(list, 4);
  pigeonResult.canvasHeight = GetNullableObjectAtIndex(list, 5);
  pigeonResult.enableMirror = GetNullableObjectAtIndex(list, 6);
  return pigeonResult;
}
+ (nullable NEFLTSetVideoCorrectionConfigRequest *)nullableFromList:(NSArray *)list {
  return (list) ? [NEFLTSetVideoCorrectionConfigRequest fromList:list] : nil;
}
- (NSArray *)toList {
  return @[
    (self.topLeft ? [self.topLeft toList] : [NSNull null]),
    (self.topRight ? [self.topRight toList] : [NSNull null]),
    (self.bottomLeft ? [self.bottomLeft toList] : [NSNull null]),
    (self.bottomRight ? [self.bottomRight toList] : [NSNull null]),
    (self.canvasWidth ?: [NSNull null]),
    (self.canvasHeight ?: [NSNull null]),
    (self.enableMirror ?: [NSNull null]),
  ];
}
@end

@implementation NEFLTEnableVirtualBackgroundRequest
+ (instancetype)makeWithEnabled:(nullable NSNumber *)enabled
           backgroundSourceType:(nullable NSNumber *)backgroundSourceType
                          color:(nullable NSNumber *)color
                         source:(nullable NSString *)source
                    blur_degree:(nullable NSNumber *)blur_degree {
  NEFLTEnableVirtualBackgroundRequest *pigeonResult =
      [[NEFLTEnableVirtualBackgroundRequest alloc] init];
  pigeonResult.enabled = enabled;
  pigeonResult.backgroundSourceType = backgroundSourceType;
  pigeonResult.color = color;
  pigeonResult.source = source;
  pigeonResult.blur_degree = blur_degree;
  return pigeonResult;
}
+ (NEFLTEnableVirtualBackgroundRequest *)fromList:(NSArray *)list {
  NEFLTEnableVirtualBackgroundRequest *pigeonResult =
      [[NEFLTEnableVirtualBackgroundRequest alloc] init];
  pigeonResult.enabled = GetNullableObjectAtIndex(list, 0);
  pigeonResult.backgroundSourceType = GetNullableObjectAtIndex(list, 1);
  pigeonResult.color = GetNullableObjectAtIndex(list, 2);
  pigeonResult.source = GetNullableObjectAtIndex(list, 3);
  pigeonResult.blur_degree = GetNullableObjectAtIndex(list, 4);
  return pigeonResult;
}
+ (nullable NEFLTEnableVirtualBackgroundRequest *)nullableFromList:(NSArray *)list {
  return (list) ? [NEFLTEnableVirtualBackgroundRequest fromList:list] : nil;
}
- (NSArray *)toList {
  return @[
    (self.enabled ?: [NSNull null]),
    (self.backgroundSourceType ?: [NSNull null]),
    (self.color ?: [NSNull null]),
    (self.source ?: [NSNull null]),
    (self.blur_degree ?: [NSNull null]),
  ];
}
@end

@implementation NEFLTSetRemoteHighPriorityAudioStreamRequest
+ (instancetype)makeWithEnabled:(nullable NSNumber *)enabled
                            uid:(nullable NSNumber *)uid
                     streamType:(nullable NSNumber *)streamType {
  NEFLTSetRemoteHighPriorityAudioStreamRequest *pigeonResult =
      [[NEFLTSetRemoteHighPriorityAudioStreamRequest alloc] init];
  pigeonResult.enabled = enabled;
  pigeonResult.uid = uid;
  pigeonResult.streamType = streamType;
  return pigeonResult;
}
+ (NEFLTSetRemoteHighPriorityAudioStreamRequest *)fromList:(NSArray *)list {
  NEFLTSetRemoteHighPriorityAudioStreamRequest *pigeonResult =
      [[NEFLTSetRemoteHighPriorityAudioStreamRequest alloc] init];
  pigeonResult.enabled = GetNullableObjectAtIndex(list, 0);
  pigeonResult.uid = GetNullableObjectAtIndex(list, 1);
  pigeonResult.streamType = GetNullableObjectAtIndex(list, 2);
  return pigeonResult;
}
+ (nullable NEFLTSetRemoteHighPriorityAudioStreamRequest *)nullableFromList:(NSArray *)list {
  return (list) ? [NEFLTSetRemoteHighPriorityAudioStreamRequest fromList:list] : nil;
}
- (NSArray *)toList {
  return @[
    (self.enabled ?: [NSNull null]),
    (self.uid ?: [NSNull null]),
    (self.streamType ?: [NSNull null]),
  ];
}
@end

@implementation NEFLTVideoWatermarkImageConfig
+ (instancetype)makeWithWmAlpha:(nullable NSNumber *)wmAlpha
                        wmWidth:(nullable NSNumber *)wmWidth
                       wmHeight:(nullable NSNumber *)wmHeight
                        offsetX:(nullable NSNumber *)offsetX
                        offsetY:(nullable NSNumber *)offsetY
                     imagePaths:(nullable NSArray<NSString *> *)imagePaths
                            fps:(nullable NSNumber *)fps
                           loop:(nullable NSNumber *)loop {
  NEFLTVideoWatermarkImageConfig *pigeonResult = [[NEFLTVideoWatermarkImageConfig alloc] init];
  pigeonResult.wmAlpha = wmAlpha;
  pigeonResult.wmWidth = wmWidth;
  pigeonResult.wmHeight = wmHeight;
  pigeonResult.offsetX = offsetX;
  pigeonResult.offsetY = offsetY;
  pigeonResult.imagePaths = imagePaths;
  pigeonResult.fps = fps;
  pigeonResult.loop = loop;
  return pigeonResult;
}
+ (NEFLTVideoWatermarkImageConfig *)fromList:(NSArray *)list {
  NEFLTVideoWatermarkImageConfig *pigeonResult = [[NEFLTVideoWatermarkImageConfig alloc] init];
  pigeonResult.wmAlpha = GetNullableObjectAtIndex(list, 0);
  pigeonResult.wmWidth = GetNullableObjectAtIndex(list, 1);
  pigeonResult.wmHeight = GetNullableObjectAtIndex(list, 2);
  pigeonResult.offsetX = GetNullableObjectAtIndex(list, 3);
  pigeonResult.offsetY = GetNullableObjectAtIndex(list, 4);
  pigeonResult.imagePaths = GetNullableObjectAtIndex(list, 5);
  pigeonResult.fps = GetNullableObjectAtIndex(list, 6);
  pigeonResult.loop = GetNullableObjectAtIndex(list, 7);
  return pigeonResult;
}
+ (nullable NEFLTVideoWatermarkImageConfig *)nullableFromList:(NSArray *)list {
  return (list) ? [NEFLTVideoWatermarkImageConfig fromList:list] : nil;
}
- (NSArray *)toList {
  return @[
    (self.wmAlpha ?: [NSNull null]),
    (self.wmWidth ?: [NSNull null]),
    (self.wmHeight ?: [NSNull null]),
    (self.offsetX ?: [NSNull null]),
    (self.offsetY ?: [NSNull null]),
    (self.imagePaths ?: [NSNull null]),
    (self.fps ?: [NSNull null]),
    (self.loop ?: [NSNull null]),
  ];
}
@end

@implementation NEFLTVideoWatermarkTextConfig
+ (instancetype)makeWithWmAlpha:(nullable NSNumber *)wmAlpha
                        wmWidth:(nullable NSNumber *)wmWidth
                       wmHeight:(nullable NSNumber *)wmHeight
                        offsetX:(nullable NSNumber *)offsetX
                        offsetY:(nullable NSNumber *)offsetY
                        wmColor:(nullable NSNumber *)wmColor
                       fontSize:(nullable NSNumber *)fontSize
                      fontColor:(nullable NSNumber *)fontColor
                 fontNameOrPath:(nullable NSString *)fontNameOrPath
                        content:(nullable NSString *)content {
  NEFLTVideoWatermarkTextConfig *pigeonResult = [[NEFLTVideoWatermarkTextConfig alloc] init];
  pigeonResult.wmAlpha = wmAlpha;
  pigeonResult.wmWidth = wmWidth;
  pigeonResult.wmHeight = wmHeight;
  pigeonResult.offsetX = offsetX;
  pigeonResult.offsetY = offsetY;
  pigeonResult.wmColor = wmColor;
  pigeonResult.fontSize = fontSize;
  pigeonResult.fontColor = fontColor;
  pigeonResult.fontNameOrPath = fontNameOrPath;
  pigeonResult.content = content;
  return pigeonResult;
}
+ (NEFLTVideoWatermarkTextConfig *)fromList:(NSArray *)list {
  NEFLTVideoWatermarkTextConfig *pigeonResult = [[NEFLTVideoWatermarkTextConfig alloc] init];
  pigeonResult.wmAlpha = GetNullableObjectAtIndex(list, 0);
  pigeonResult.wmWidth = GetNullableObjectAtIndex(list, 1);
  pigeonResult.wmHeight = GetNullableObjectAtIndex(list, 2);
  pigeonResult.offsetX = GetNullableObjectAtIndex(list, 3);
  pigeonResult.offsetY = GetNullableObjectAtIndex(list, 4);
  pigeonResult.wmColor = GetNullableObjectAtIndex(list, 5);
  pigeonResult.fontSize = GetNullableObjectAtIndex(list, 6);
  pigeonResult.fontColor = GetNullableObjectAtIndex(list, 7);
  pigeonResult.fontNameOrPath = GetNullableObjectAtIndex(list, 8);
  pigeonResult.content = GetNullableObjectAtIndex(list, 9);
  return pigeonResult;
}
+ (nullable NEFLTVideoWatermarkTextConfig *)nullableFromList:(NSArray *)list {
  return (list) ? [NEFLTVideoWatermarkTextConfig fromList:list] : nil;
}
- (NSArray *)toList {
  return @[
    (self.wmAlpha ?: [NSNull null]),
    (self.wmWidth ?: [NSNull null]),
    (self.wmHeight ?: [NSNull null]),
    (self.offsetX ?: [NSNull null]),
    (self.offsetY ?: [NSNull null]),
    (self.wmColor ?: [NSNull null]),
    (self.fontSize ?: [NSNull null]),
    (self.fontColor ?: [NSNull null]),
    (self.fontNameOrPath ?: [NSNull null]),
    (self.content ?: [NSNull null]),
  ];
}
@end

@implementation NEFLTVideoWatermarkTimestampConfig
+ (instancetype)makeWithWmAlpha:(nullable NSNumber *)wmAlpha
                        wmWidth:(nullable NSNumber *)wmWidth
                       wmHeight:(nullable NSNumber *)wmHeight
                        offsetX:(nullable NSNumber *)offsetX
                        offsetY:(nullable NSNumber *)offsetY
                        wmColor:(nullable NSNumber *)wmColor
                       fontSize:(nullable NSNumber *)fontSize
                      fontColor:(nullable NSNumber *)fontColor
                 fontNameOrPath:(nullable NSString *)fontNameOrPath {
  NEFLTVideoWatermarkTimestampConfig *pigeonResult =
      [[NEFLTVideoWatermarkTimestampConfig alloc] init];
  pigeonResult.wmAlpha = wmAlpha;
  pigeonResult.wmWidth = wmWidth;
  pigeonResult.wmHeight = wmHeight;
  pigeonResult.offsetX = offsetX;
  pigeonResult.offsetY = offsetY;
  pigeonResult.wmColor = wmColor;
  pigeonResult.fontSize = fontSize;
  pigeonResult.fontColor = fontColor;
  pigeonResult.fontNameOrPath = fontNameOrPath;
  return pigeonResult;
}
+ (NEFLTVideoWatermarkTimestampConfig *)fromList:(NSArray *)list {
  NEFLTVideoWatermarkTimestampConfig *pigeonResult =
      [[NEFLTVideoWatermarkTimestampConfig alloc] init];
  pigeonResult.wmAlpha = GetNullableObjectAtIndex(list, 0);
  pigeonResult.wmWidth = GetNullableObjectAtIndex(list, 1);
  pigeonResult.wmHeight = GetNullableObjectAtIndex(list, 2);
  pigeonResult.offsetX = GetNullableObjectAtIndex(list, 3);
  pigeonResult.offsetY = GetNullableObjectAtIndex(list, 4);
  pigeonResult.wmColor = GetNullableObjectAtIndex(list, 5);
  pigeonResult.fontSize = GetNullableObjectAtIndex(list, 6);
  pigeonResult.fontColor = GetNullableObjectAtIndex(list, 7);
  pigeonResult.fontNameOrPath = GetNullableObjectAtIndex(list, 8);
  return pigeonResult;
}
+ (nullable NEFLTVideoWatermarkTimestampConfig *)nullableFromList:(NSArray *)list {
  return (list) ? [NEFLTVideoWatermarkTimestampConfig fromList:list] : nil;
}
- (NSArray *)toList {
  return @[
    (self.wmAlpha ?: [NSNull null]),
    (self.wmWidth ?: [NSNull null]),
    (self.wmHeight ?: [NSNull null]),
    (self.offsetX ?: [NSNull null]),
    (self.offsetY ?: [NSNull null]),
    (self.wmColor ?: [NSNull null]),
    (self.fontSize ?: [NSNull null]),
    (self.fontColor ?: [NSNull null]),
    (self.fontNameOrPath ?: [NSNull null]),
  ];
}
@end

@implementation NEFLTVideoWatermarkConfig
+ (instancetype)makeWithWatermarkType:(NEFLTNERtcVideoWatermarkType)WatermarkType
                       imageWatermark:(nullable NEFLTVideoWatermarkImageConfig *)imageWatermark
                        textWatermark:(nullable NEFLTVideoWatermarkTextConfig *)textWatermark
                   timestampWatermark:
                       (nullable NEFLTVideoWatermarkTimestampConfig *)timestampWatermark {
  NEFLTVideoWatermarkConfig *pigeonResult = [[NEFLTVideoWatermarkConfig alloc] init];
  pigeonResult.WatermarkType = WatermarkType;
  pigeonResult.imageWatermark = imageWatermark;
  pigeonResult.textWatermark = textWatermark;
  pigeonResult.timestampWatermark = timestampWatermark;
  return pigeonResult;
}
+ (NEFLTVideoWatermarkConfig *)fromList:(NSArray *)list {
  NEFLTVideoWatermarkConfig *pigeonResult = [[NEFLTVideoWatermarkConfig alloc] init];
  pigeonResult.WatermarkType = [GetNullableObjectAtIndex(list, 0) integerValue];
  pigeonResult.imageWatermark =
      [NEFLTVideoWatermarkImageConfig nullableFromList:(GetNullableObjectAtIndex(list, 1))];
  pigeonResult.textWatermark =
      [NEFLTVideoWatermarkTextConfig nullableFromList:(GetNullableObjectAtIndex(list, 2))];
  pigeonResult.timestampWatermark =
      [NEFLTVideoWatermarkTimestampConfig nullableFromList:(GetNullableObjectAtIndex(list, 3))];
  return pigeonResult;
}
+ (nullable NEFLTVideoWatermarkConfig *)nullableFromList:(NSArray *)list {
  return (list) ? [NEFLTVideoWatermarkConfig fromList:list] : nil;
}
- (NSArray *)toList {
  return @[
    @(self.WatermarkType),
    (self.imageWatermark ? [self.imageWatermark toList] : [NSNull null]),
    (self.textWatermark ? [self.textWatermark toList] : [NSNull null]),
    (self.timestampWatermark ? [self.timestampWatermark toList] : [NSNull null]),
  ];
}
@end

@implementation NEFLTSetLocalVideoWatermarkConfigsRequest
+ (instancetype)makeWithType:(NSNumber *)type config:(nullable NEFLTVideoWatermarkConfig *)config {
  NEFLTSetLocalVideoWatermarkConfigsRequest *pigeonResult =
      [[NEFLTSetLocalVideoWatermarkConfigsRequest alloc] init];
  pigeonResult.type = type;
  pigeonResult.config = config;
  return pigeonResult;
}
+ (NEFLTSetLocalVideoWatermarkConfigsRequest *)fromList:(NSArray *)list {
  NEFLTSetLocalVideoWatermarkConfigsRequest *pigeonResult =
      [[NEFLTSetLocalVideoWatermarkConfigsRequest alloc] init];
  pigeonResult.type = GetNullableObjectAtIndex(list, 0);
  NSAssert(pigeonResult.type != nil, @"");
  pigeonResult.config =
      [NEFLTVideoWatermarkConfig nullableFromList:(GetNullableObjectAtIndex(list, 1))];
  return pigeonResult;
}
+ (nullable NEFLTSetLocalVideoWatermarkConfigsRequest *)nullableFromList:(NSArray *)list {
  return (list) ? [NEFLTSetLocalVideoWatermarkConfigsRequest fromList:list] : nil;
}
- (NSArray *)toList {
  return @[
    (self.type ?: [NSNull null]),
    (self.config ? [self.config toList] : [NSNull null]),
  ];
}
@end

@implementation NEFLTNERtcVersion
+ (instancetype)makeWithVersionName:(nullable NSString *)versionName
                        versionCode:(nullable NSNumber *)versionCode
                          buildType:(nullable NSString *)buildType
                          buildDate:(nullable NSString *)buildDate
                      buildRevision:(nullable NSString *)buildRevision
                          buildHost:(nullable NSString *)buildHost
                          serverEnv:(nullable NSString *)serverEnv
                        buildBranch:(nullable NSString *)buildBranch
                     engineRevision:(nullable NSString *)engineRevision {
  NEFLTNERtcVersion *pigeonResult = [[NEFLTNERtcVersion alloc] init];
  pigeonResult.versionName = versionName;
  pigeonResult.versionCode = versionCode;
  pigeonResult.buildType = buildType;
  pigeonResult.buildDate = buildDate;
  pigeonResult.buildRevision = buildRevision;
  pigeonResult.buildHost = buildHost;
  pigeonResult.serverEnv = serverEnv;
  pigeonResult.buildBranch = buildBranch;
  pigeonResult.engineRevision = engineRevision;
  return pigeonResult;
}
+ (NEFLTNERtcVersion *)fromList:(NSArray *)list {
  NEFLTNERtcVersion *pigeonResult = [[NEFLTNERtcVersion alloc] init];
  pigeonResult.versionName = GetNullableObjectAtIndex(list, 0);
  pigeonResult.versionCode = GetNullableObjectAtIndex(list, 1);
  pigeonResult.buildType = GetNullableObjectAtIndex(list, 2);
  pigeonResult.buildDate = GetNullableObjectAtIndex(list, 3);
  pigeonResult.buildRevision = GetNullableObjectAtIndex(list, 4);
  pigeonResult.buildHost = GetNullableObjectAtIndex(list, 5);
  pigeonResult.serverEnv = GetNullableObjectAtIndex(list, 6);
  pigeonResult.buildBranch = GetNullableObjectAtIndex(list, 7);
  pigeonResult.engineRevision = GetNullableObjectAtIndex(list, 8);
  return pigeonResult;
}
+ (nullable NEFLTNERtcVersion *)nullableFromList:(NSArray *)list {
  return (list) ? [NEFLTNERtcVersion fromList:list] : nil;
}
- (NSArray *)toList {
  return @[
    (self.versionName ?: [NSNull null]),
    (self.versionCode ?: [NSNull null]),
    (self.buildType ?: [NSNull null]),
    (self.buildDate ?: [NSNull null]),
    (self.buildRevision ?: [NSNull null]),
    (self.buildHost ?: [NSNull null]),
    (self.serverEnv ?: [NSNull null]),
    (self.buildBranch ?: [NSNull null]),
    (self.engineRevision ?: [NSNull null]),
  ];
}
@end

@interface NEFLTNERtcChannelEventSinkCodecReader : FlutterStandardReader
@end
@implementation NEFLTNERtcChannelEventSinkCodecReader
- (nullable id)readValueOfType:(UInt8)type {
  switch (type) {
    case 128:
      return [NEFLTAudioVolumeInfo fromList:[self readValue]];
    case 129:
      return [NEFLTFirstVideoDataReceivedEvent fromList:[self readValue]];
    case 130:
      return [NEFLTFirstVideoFrameDecodedEvent fromList:[self readValue]];
    case 131:
      return [NEFLTNERtcLastmileProbeOneWayResult fromList:[self readValue]];
    case 132:
      return [NEFLTNERtcLastmileProbeResult fromList:[self readValue]];
    case 133:
      return [NEFLTNERtcUserJoinExtraInfo fromList:[self readValue]];
    case 134:
      return [NEFLTNERtcUserLeaveExtraInfo fromList:[self readValue]];
    case 135:
      return [NEFLTRemoteAudioVolumeIndicationEvent fromList:[self readValue]];
    case 136:
      return [NEFLTUserJoinedEvent fromList:[self readValue]];
    case 137:
      return [NEFLTUserLeaveEvent fromList:[self readValue]];
    case 138:
      return [NEFLTUserVideoMuteEvent fromList:[self readValue]];
    case 139:
      return [NEFLTVirtualBackgroundSourceEnabledEvent fromList:[self readValue]];
    default:
      return [super readValueOfType:type];
  }
}
@end

@interface NEFLTNERtcChannelEventSinkCodecWriter : FlutterStandardWriter
@end
@implementation NEFLTNERtcChannelEventSinkCodecWriter
- (void)writeValue:(id)value {
  if ([value isKindOfClass:[NEFLTAudioVolumeInfo class]]) {
    [self writeByte:128];
    [self writeValue:[value toList]];
  } else if ([value isKindOfClass:[NEFLTFirstVideoDataReceivedEvent class]]) {
    [self writeByte:129];
    [self writeValue:[value toList]];
  } else if ([value isKindOfClass:[NEFLTFirstVideoFrameDecodedEvent class]]) {
    [self writeByte:130];
    [self writeValue:[value toList]];
  } else if ([value isKindOfClass:[NEFLTNERtcLastmileProbeOneWayResult class]]) {
    [self writeByte:131];
    [self writeValue:[value toList]];
  } else if ([value isKindOfClass:[NEFLTNERtcLastmileProbeResult class]]) {
    [self writeByte:132];
    [self writeValue:[value toList]];
  } else if ([value isKindOfClass:[NEFLTNERtcUserJoinExtraInfo class]]) {
    [self writeByte:133];
    [self writeValue:[value toList]];
  } else if ([value isKindOfClass:[NEFLTNERtcUserLeaveExtraInfo class]]) {
    [self writeByte:134];
    [self writeValue:[value toList]];
  } else if ([value isKindOfClass:[NEFLTRemoteAudioVolumeIndicationEvent class]]) {
    [self writeByte:135];
    [self writeValue:[value toList]];
  } else if ([value isKindOfClass:[NEFLTUserJoinedEvent class]]) {
    [self writeByte:136];
    [self writeValue:[value toList]];
  } else if ([value isKindOfClass:[NEFLTUserLeaveEvent class]]) {
    [self writeByte:137];
    [self writeValue:[value toList]];
  } else if ([value isKindOfClass:[NEFLTUserVideoMuteEvent class]]) {
    [self writeByte:138];
    [self writeValue:[value toList]];
  } else if ([value isKindOfClass:[NEFLTVirtualBackgroundSourceEnabledEvent class]]) {
    [self writeByte:139];
    [self writeValue:[value toList]];
  } else {
    [super writeValue:value];
  }
}
@end

@interface NEFLTNERtcChannelEventSinkCodecReaderWriter : FlutterStandardReaderWriter
@end
@implementation NEFLTNERtcChannelEventSinkCodecReaderWriter
- (FlutterStandardWriter *)writerWithData:(NSMutableData *)data {
  return [[NEFLTNERtcChannelEventSinkCodecWriter alloc] initWithData:data];
}
- (FlutterStandardReader *)readerWithData:(NSData *)data {
  return [[NEFLTNERtcChannelEventSinkCodecReader alloc] initWithData:data];
}
@end

NSObject<FlutterMessageCodec> *NEFLTNERtcChannelEventSinkGetCodec() {
  static FlutterStandardMessageCodec *sSharedObject = nil;
  static dispatch_once_t sPred = 0;
  dispatch_once(&sPred, ^{
    NEFLTNERtcChannelEventSinkCodecReaderWriter *readerWriter =
        [[NEFLTNERtcChannelEventSinkCodecReaderWriter alloc] init];
    sSharedObject = [FlutterStandardMessageCodec codecWithReaderWriter:readerWriter];
  });
  return sSharedObject;
}

@interface NEFLTNERtcChannelEventSink ()
@property(nonatomic, strong) NSObject<FlutterBinaryMessenger> *binaryMessenger;
@end

@implementation NEFLTNERtcChannelEventSink

- (instancetype)initWithBinaryMessenger:(NSObject<FlutterBinaryMessenger> *)binaryMessenger {
  self = [super init];
  if (self) {
    _binaryMessenger = binaryMessenger;
  }
  return self;
}
- (void)onJoinChannelResult:(NSNumber *)arg_result
                  channelId:(NSNumber *)arg_channelId
                    elapsed:(NSNumber *)arg_elapsed
                        uid:(NSNumber *)arg_uid
                 completion:(void (^)(FlutterError *_Nullable))completion {
  FlutterBasicMessageChannel *channel = [FlutterBasicMessageChannel
      messageChannelWithName:@"dev.flutter.pigeon.NERtcChannelEventSink.onJoinChannel"
             binaryMessenger:self.binaryMessenger
                       codec:NEFLTNERtcChannelEventSinkGetCodec()];
  [channel sendMessage:@[
    arg_result ?: [NSNull null], arg_channelId ?: [NSNull null], arg_elapsed ?: [NSNull null],
    arg_uid ?: [NSNull null]
  ]
                 reply:^(id reply) {
                   completion(nil);
                 }];
}
- (void)onLeaveChannelResult:(NSNumber *)arg_result
                  completion:(void (^)(FlutterError *_Nullable))completion {
  FlutterBasicMessageChannel *channel = [FlutterBasicMessageChannel
      messageChannelWithName:@"dev.flutter.pigeon.NERtcChannelEventSink.onLeaveChannel"
             binaryMessenger:self.binaryMessenger
                       codec:NEFLTNERtcChannelEventSinkGetCodec()];
  [channel sendMessage:@[ arg_result ?: [NSNull null] ]
                 reply:^(id reply) {
                   completion(nil);
                 }];
}
- (void)onUserJoinedEvent:(NEFLTUserJoinedEvent *)arg_event
               completion:(void (^)(FlutterError *_Nullable))completion {
  FlutterBasicMessageChannel *channel = [FlutterBasicMessageChannel
      messageChannelWithName:@"dev.flutter.pigeon.NERtcChannelEventSink.onUserJoined"
             binaryMessenger:self.binaryMessenger
                       codec:NEFLTNERtcChannelEventSinkGetCodec()];
  [channel sendMessage:@[ arg_event ?: [NSNull null] ]
                 reply:^(id reply) {
                   completion(nil);
                 }];
}
- (void)onUserLeaveEvent:(NEFLTUserLeaveEvent *)arg_event
              completion:(void (^)(FlutterError *_Nullable))completion {
  FlutterBasicMessageChannel *channel = [FlutterBasicMessageChannel
      messageChannelWithName:@"dev.flutter.pigeon.NERtcChannelEventSink.onUserLeave"
             binaryMessenger:self.binaryMessenger
                       codec:NEFLTNERtcChannelEventSinkGetCodec()];
  [channel sendMessage:@[ arg_event ?: [NSNull null] ]
                 reply:^(id reply) {
                   completion(nil);
                 }];
}
- (void)onUserAudioStartUid:(NSNumber *)arg_uid
                 completion:(void (^)(FlutterError *_Nullable))completion {
  FlutterBasicMessageChannel *channel = [FlutterBasicMessageChannel
      messageChannelWithName:@"dev.flutter.pigeon.NERtcChannelEventSink.onUserAudioStart"
             binaryMessenger:self.binaryMessenger
                       codec:NEFLTNERtcChannelEventSinkGetCodec()];
  [channel sendMessage:@[ arg_uid ?: [NSNull null] ]
                 reply:^(id reply) {
                   completion(nil);
                 }];
}
- (void)onUserSubStreamAudioStartUid:(NSNumber *)arg_uid
                          completion:(void (^)(FlutterError *_Nullable))completion {
  FlutterBasicMessageChannel *channel = [FlutterBasicMessageChannel
      messageChannelWithName:@"dev.flutter.pigeon.NERtcChannelEventSink.onUserSubStreamAudioStart"
             binaryMessenger:self.binaryMessenger
                       codec:NEFLTNERtcChannelEventSinkGetCodec()];
  [channel sendMessage:@[ arg_uid ?: [NSNull null] ]
                 reply:^(id reply) {
                   completion(nil);
                 }];
}
- (void)onUserAudioStopUid:(NSNumber *)arg_uid
                completion:(void (^)(FlutterError *_Nullable))completion {
  FlutterBasicMessageChannel *channel = [FlutterBasicMessageChannel
      messageChannelWithName:@"dev.flutter.pigeon.NERtcChannelEventSink.onUserAudioStop"
             binaryMessenger:self.binaryMessenger
                       codec:NEFLTNERtcChannelEventSinkGetCodec()];
  [channel sendMessage:@[ arg_uid ?: [NSNull null] ]
                 reply:^(id reply) {
                   completion(nil);
                 }];
}
- (void)onUserSubStreamAudioStopUid:(NSNumber *)arg_uid
                         completion:(void (^)(FlutterError *_Nullable))completion {
  FlutterBasicMessageChannel *channel = [FlutterBasicMessageChannel
      messageChannelWithName:@"dev.flutter.pigeon.NERtcChannelEventSink.onUserSubStreamAudioStop"
             binaryMessenger:self.binaryMessenger
                       codec:NEFLTNERtcChannelEventSinkGetCodec()];
  [channel sendMessage:@[ arg_uid ?: [NSNull null] ]
                 reply:^(id reply) {
                   completion(nil);
                 }];
}
- (void)onUserVideoStartUid:(NSNumber *)arg_uid
                 maxProfile:(NSNumber *)arg_maxProfile
                 completion:(void (^)(FlutterError *_Nullable))completion {
  FlutterBasicMessageChannel *channel = [FlutterBasicMessageChannel
      messageChannelWithName:@"dev.flutter.pigeon.NERtcChannelEventSink.onUserVideoStart"
             binaryMessenger:self.binaryMessenger
                       codec:NEFLTNERtcChannelEventSinkGetCodec()];
  [channel sendMessage:@[ arg_uid ?: [NSNull null], arg_maxProfile ?: [NSNull null] ]
                 reply:^(id reply) {
                   completion(nil);
                 }];
}
- (void)onUserVideoStopUid:(NSNumber *)arg_uid
                completion:(void (^)(FlutterError *_Nullable))completion {
  FlutterBasicMessageChannel *channel = [FlutterBasicMessageChannel
      messageChannelWithName:@"dev.flutter.pigeon.NERtcChannelEventSink.onUserVideoStop"
             binaryMessenger:self.binaryMessenger
                       codec:NEFLTNERtcChannelEventSinkGetCodec()];
  [channel sendMessage:@[ arg_uid ?: [NSNull null] ]
                 reply:^(id reply) {
                   completion(nil);
                 }];
}
- (void)onDisconnectReason:(NSNumber *)arg_reason
                completion:(void (^)(FlutterError *_Nullable))completion {
  FlutterBasicMessageChannel *channel = [FlutterBasicMessageChannel
      messageChannelWithName:@"dev.flutter.pigeon.NERtcChannelEventSink.onDisconnect"
             binaryMessenger:self.binaryMessenger
                       codec:NEFLTNERtcChannelEventSinkGetCodec()];
  [channel sendMessage:@[ arg_reason ?: [NSNull null] ]
                 reply:^(id reply) {
                   completion(nil);
                 }];
}
- (void)onUserAudioMuteUid:(NSNumber *)arg_uid
                     muted:(NSNumber *)arg_muted
                completion:(void (^)(FlutterError *_Nullable))completion {
  FlutterBasicMessageChannel *channel = [FlutterBasicMessageChannel
      messageChannelWithName:@"dev.flutter.pigeon.NERtcChannelEventSink.onUserAudioMute"
             binaryMessenger:self.binaryMessenger
                       codec:NEFLTNERtcChannelEventSinkGetCodec()];
  [channel sendMessage:@[ arg_uid ?: [NSNull null], arg_muted ?: [NSNull null] ]
                 reply:^(id reply) {
                   completion(nil);
                 }];
}
- (void)onUserVideoMuteEvent:(NEFLTUserVideoMuteEvent *)arg_event
                  completion:(void (^)(FlutterError *_Nullable))completion {
  FlutterBasicMessageChannel *channel = [FlutterBasicMessageChannel
      messageChannelWithName:@"dev.flutter.pigeon.NERtcChannelEventSink.onUserVideoMute"
             binaryMessenger:self.binaryMessenger
                       codec:NEFLTNERtcChannelEventSinkGetCodec()];
  [channel sendMessage:@[ arg_event ?: [NSNull null] ]
                 reply:^(id reply) {
                   completion(nil);
                 }];
}
- (void)onUserSubStreamAudioMuteUid:(NSNumber *)arg_uid
                              muted:(NSNumber *)arg_muted
                         completion:(void (^)(FlutterError *_Nullable))completion {
  FlutterBasicMessageChannel *channel = [FlutterBasicMessageChannel
      messageChannelWithName:@"dev.flutter.pigeon.NERtcChannelEventSink.onUserSubStreamAudioMute"
             binaryMessenger:self.binaryMessenger
                       codec:NEFLTNERtcChannelEventSinkGetCodec()];
  [channel sendMessage:@[ arg_uid ?: [NSNull null], arg_muted ?: [NSNull null] ]
                 reply:^(id reply) {
                   completion(nil);
                 }];
}
- (void)onFirstAudioDataReceivedUid:(NSNumber *)arg_uid
                         completion:(void (^)(FlutterError *_Nullable))completion {
  FlutterBasicMessageChannel *channel = [FlutterBasicMessageChannel
      messageChannelWithName:@"dev.flutter.pigeon.NERtcChannelEventSink.onFirstAudioDataReceived"
             binaryMessenger:self.binaryMessenger
                       codec:NEFLTNERtcChannelEventSinkGetCodec()];
  [channel sendMessage:@[ arg_uid ?: [NSNull null] ]
                 reply:^(id reply) {
                   completion(nil);
                 }];
}
- (void)onFirstVideoDataReceivedEvent:(NEFLTFirstVideoDataReceivedEvent *)arg_event
                           completion:(void (^)(FlutterError *_Nullable))completion {
  FlutterBasicMessageChannel *channel = [FlutterBasicMessageChannel
      messageChannelWithName:@"dev.flutter.pigeon.NERtcChannelEventSink.onFirstVideoDataReceived"
             binaryMessenger:self.binaryMessenger
                       codec:NEFLTNERtcChannelEventSinkGetCodec()];
  [channel sendMessage:@[ arg_event ?: [NSNull null] ]
                 reply:^(id reply) {
                   completion(nil);
                 }];
}
- (void)onFirstAudioFrameDecodedUid:(NSNumber *)arg_uid
                         completion:(void (^)(FlutterError *_Nullable))completion {
  FlutterBasicMessageChannel *channel = [FlutterBasicMessageChannel
      messageChannelWithName:@"dev.flutter.pigeon.NERtcChannelEventSink.onFirstAudioFrameDecoded"
             binaryMessenger:self.binaryMessenger
                       codec:NEFLTNERtcChannelEventSinkGetCodec()];
  [channel sendMessage:@[ arg_uid ?: [NSNull null] ]
                 reply:^(id reply) {
                   completion(nil);
                 }];
}
- (void)onFirstVideoFrameDecodedEvent:(NEFLTFirstVideoFrameDecodedEvent *)arg_event
                           completion:(void (^)(FlutterError *_Nullable))completion {
  FlutterBasicMessageChannel *channel = [FlutterBasicMessageChannel
      messageChannelWithName:@"dev.flutter.pigeon.NERtcChannelEventSink.onFirstVideoFrameDecoded"
             binaryMessenger:self.binaryMessenger
                       codec:NEFLTNERtcChannelEventSinkGetCodec()];
  [channel sendMessage:@[ arg_event ?: [NSNull null] ]
                 reply:^(id reply) {
                   completion(nil);
                 }];
}
- (void)onVirtualBackgroundSourceEnabledEvent:(NEFLTVirtualBackgroundSourceEnabledEvent *)arg_event
                                   completion:(void (^)(FlutterError *_Nullable))completion {
  FlutterBasicMessageChannel *channel = [FlutterBasicMessageChannel
      messageChannelWithName:
          @"dev.flutter.pigeon.NERtcChannelEventSink.onVirtualBackgroundSourceEnabled"
             binaryMessenger:self.binaryMessenger
                       codec:NEFLTNERtcChannelEventSinkGetCodec()];
  [channel sendMessage:@[ arg_event ?: [NSNull null] ]
                 reply:^(id reply) {
                   completion(nil);
                 }];
}
- (void)onConnectionTypeChangedNewConnectionType:(NSNumber *)arg_newConnectionType
                                      completion:(void (^)(FlutterError *_Nullable))completion {
  FlutterBasicMessageChannel *channel = [FlutterBasicMessageChannel
      messageChannelWithName:@"dev.flutter.pigeon.NERtcChannelEventSink.onConnectionTypeChanged"
             binaryMessenger:self.binaryMessenger
                       codec:NEFLTNERtcChannelEventSinkGetCodec()];
  [channel sendMessage:@[ arg_newConnectionType ?: [NSNull null] ]
                 reply:^(id reply) {
                   completion(nil);
                 }];
}
- (void)onReconnectingStartWithCompletion:(void (^)(FlutterError *_Nullable))completion {
  FlutterBasicMessageChannel *channel = [FlutterBasicMessageChannel
      messageChannelWithName:@"dev.flutter.pigeon.NERtcChannelEventSink.onReconnectingStart"
             binaryMessenger:self.binaryMessenger
                       codec:NEFLTNERtcChannelEventSinkGetCodec()];
  [channel sendMessage:nil
                 reply:^(id reply) {
                   completion(nil);
                 }];
}
- (void)onReJoinChannelResult:(NSNumber *)arg_result
                    channelId:(NSNumber *)arg_channelId
                   completion:(void (^)(FlutterError *_Nullable))completion {
  FlutterBasicMessageChannel *channel = [FlutterBasicMessageChannel
      messageChannelWithName:@"dev.flutter.pigeon.NERtcChannelEventSink.onReJoinChannel"
             binaryMessenger:self.binaryMessenger
                       codec:NEFLTNERtcChannelEventSinkGetCodec()];
  [channel sendMessage:@[ arg_result ?: [NSNull null], arg_channelId ?: [NSNull null] ]
                 reply:^(id reply) {
                   completion(nil);
                 }];
}
- (void)onConnectionStateChangedState:(NSNumber *)arg_state
                               reason:(NSNumber *)arg_reason
                           completion:(void (^)(FlutterError *_Nullable))completion {
  FlutterBasicMessageChannel *channel = [FlutterBasicMessageChannel
      messageChannelWithName:@"dev.flutter.pigeon.NERtcChannelEventSink.onConnectionStateChanged"
             binaryMessenger:self.binaryMessenger
                       codec:NEFLTNERtcChannelEventSinkGetCodec()];
  [channel sendMessage:@[ arg_state ?: [NSNull null], arg_reason ?: [NSNull null] ]
                 reply:^(id reply) {
                   completion(nil);
                 }];
}
- (void)onLocalAudioVolumeIndicationVolume:(NSNumber *)arg_volume
                                   vadFlag:(NSNumber *)arg_vadFlag
                                completion:(void (^)(FlutterError *_Nullable))completion {
  FlutterBasicMessageChannel *channel = [FlutterBasicMessageChannel
      messageChannelWithName:
          @"dev.flutter.pigeon.NERtcChannelEventSink.onLocalAudioVolumeIndication"
             binaryMessenger:self.binaryMessenger
                       codec:NEFLTNERtcChannelEventSinkGetCodec()];
  [channel sendMessage:@[ arg_volume ?: [NSNull null], arg_vadFlag ?: [NSNull null] ]
                 reply:^(id reply) {
                   completion(nil);
                 }];
}
- (void)onRemoteAudioVolumeIndicationEvent:(NEFLTRemoteAudioVolumeIndicationEvent *)arg_event
                                completion:(void (^)(FlutterError *_Nullable))completion {
  FlutterBasicMessageChannel *channel = [FlutterBasicMessageChannel
      messageChannelWithName:
          @"dev.flutter.pigeon.NERtcChannelEventSink.onRemoteAudioVolumeIndication"
             binaryMessenger:self.binaryMessenger
                       codec:NEFLTNERtcChannelEventSinkGetCodec()];
  [channel sendMessage:@[ arg_event ?: [NSNull null] ]
                 reply:^(id reply) {
                   completion(nil);
                 }];
}
- (void)onLiveStreamStateTaskId:(NSString *)arg_taskId
                        pushUrl:(NSString *)arg_pushUrl
                      liveState:(NSNumber *)arg_liveState
                     completion:(void (^)(FlutterError *_Nullable))completion {
  FlutterBasicMessageChannel *channel = [FlutterBasicMessageChannel
      messageChannelWithName:@"dev.flutter.pigeon.NERtcChannelEventSink.onLiveStreamState"
             binaryMessenger:self.binaryMessenger
                       codec:NEFLTNERtcChannelEventSinkGetCodec()];
  [channel sendMessage:@[
    arg_taskId ?: [NSNull null], arg_pushUrl ?: [NSNull null], arg_liveState ?: [NSNull null]
  ]
                 reply:^(id reply) {
                   completion(nil);
                 }];
}
- (void)onClientRoleChangeOldRole:(NSNumber *)arg_oldRole
                          newRole:(NSNumber *)arg_newRole
                       completion:(void (^)(FlutterError *_Nullable))completion {
  FlutterBasicMessageChannel *channel = [FlutterBasicMessageChannel
      messageChannelWithName:@"dev.flutter.pigeon.NERtcChannelEventSink.onClientRoleChange"
             binaryMessenger:self.binaryMessenger
                       codec:NEFLTNERtcChannelEventSinkGetCodec()];
  [channel sendMessage:@[ arg_oldRole ?: [NSNull null], arg_newRole ?: [NSNull null] ]
                 reply:^(id reply) {
                   completion(nil);
                 }];
}
- (void)onErrorCode:(NSNumber *)arg_code completion:(void (^)(FlutterError *_Nullable))completion {
  FlutterBasicMessageChannel *channel = [FlutterBasicMessageChannel
      messageChannelWithName:@"dev.flutter.pigeon.NERtcChannelEventSink.onError"
             binaryMessenger:self.binaryMessenger
                       codec:NEFLTNERtcChannelEventSinkGetCodec()];
  [channel sendMessage:@[ arg_code ?: [NSNull null] ]
                 reply:^(id reply) {
                   completion(nil);
                 }];
}
- (void)onWarningCode:(NSNumber *)arg_code
           completion:(void (^)(FlutterError *_Nullable))completion {
  FlutterBasicMessageChannel *channel = [FlutterBasicMessageChannel
      messageChannelWithName:@"dev.flutter.pigeon.NERtcChannelEventSink.onWarning"
             binaryMessenger:self.binaryMessenger
                       codec:NEFLTNERtcChannelEventSinkGetCodec()];
  [channel sendMessage:@[ arg_code ?: [NSNull null] ]
                 reply:^(id reply) {
                   completion(nil);
                 }];
}
- (void)onUserSubStreamVideoStartUid:(NSNumber *)arg_uid
                          maxProfile:(NSNumber *)arg_maxProfile
                          completion:(void (^)(FlutterError *_Nullable))completion {
  FlutterBasicMessageChannel *channel = [FlutterBasicMessageChannel
      messageChannelWithName:@"dev.flutter.pigeon.NERtcChannelEventSink.onUserSubStreamVideoStart"
             binaryMessenger:self.binaryMessenger
                       codec:NEFLTNERtcChannelEventSinkGetCodec()];
  [channel sendMessage:@[ arg_uid ?: [NSNull null], arg_maxProfile ?: [NSNull null] ]
                 reply:^(id reply) {
                   completion(nil);
                 }];
}
- (void)onUserSubStreamVideoStopUid:(NSNumber *)arg_uid
                         completion:(void (^)(FlutterError *_Nullable))completion {
  FlutterBasicMessageChannel *channel = [FlutterBasicMessageChannel
      messageChannelWithName:@"dev.flutter.pigeon.NERtcChannelEventSink.onUserSubStreamVideoStop"
             binaryMessenger:self.binaryMessenger
                       codec:NEFLTNERtcChannelEventSinkGetCodec()];
  [channel sendMessage:@[ arg_uid ?: [NSNull null] ]
                 reply:^(id reply) {
                   completion(nil);
                 }];
}
- (void)onAudioHasHowlingWithCompletion:(void (^)(FlutterError *_Nullable))completion {
  FlutterBasicMessageChannel *channel = [FlutterBasicMessageChannel
      messageChannelWithName:@"dev.flutter.pigeon.NERtcChannelEventSink.onAudioHasHowling"
             binaryMessenger:self.binaryMessenger
                       codec:NEFLTNERtcChannelEventSinkGetCodec()];
  [channel sendMessage:nil
                 reply:^(id reply) {
                   completion(nil);
                 }];
}
- (void)onRecvSEIMsgUserID:(NSNumber *)arg_userID
                    seiMsg:(NSString *)arg_seiMsg
                completion:(void (^)(FlutterError *_Nullable))completion {
  FlutterBasicMessageChannel *channel = [FlutterBasicMessageChannel
      messageChannelWithName:@"dev.flutter.pigeon.NERtcChannelEventSink.onRecvSEIMsg"
             binaryMessenger:self.binaryMessenger
                       codec:NEFLTNERtcChannelEventSinkGetCodec()];
  [channel sendMessage:@[ arg_userID ?: [NSNull null], arg_seiMsg ?: [NSNull null] ]
                 reply:^(id reply) {
                   completion(nil);
                 }];
}
- (void)onAudioRecordingCode:(NSNumber *)arg_code
                    filePath:(NSString *)arg_filePath
                  completion:(void (^)(FlutterError *_Nullable))completion {
  FlutterBasicMessageChannel *channel = [FlutterBasicMessageChannel
      messageChannelWithName:@"dev.flutter.pigeon.NERtcChannelEventSink.onAudioRecording"
             binaryMessenger:self.binaryMessenger
                       codec:NEFLTNERtcChannelEventSinkGetCodec()];
  [channel sendMessage:@[ arg_code ?: [NSNull null], arg_filePath ?: [NSNull null] ]
                 reply:^(id reply) {
                   completion(nil);
                 }];
}
- (void)onMediaRightChangeIsAudioBannedByServer:(NSNumber *)arg_isAudioBannedByServer
                          isVideoBannedByServer:(NSNumber *)arg_isVideoBannedByServer
                                     completion:(void (^)(FlutterError *_Nullable))completion {
  FlutterBasicMessageChannel *channel = [FlutterBasicMessageChannel
      messageChannelWithName:@"dev.flutter.pigeon.NERtcChannelEventSink.onMediaRightChange"
             binaryMessenger:self.binaryMessenger
                       codec:NEFLTNERtcChannelEventSinkGetCodec()];
  [channel sendMessage:@[
    arg_isAudioBannedByServer ?: [NSNull null], arg_isVideoBannedByServer ?: [NSNull null]
  ]
                 reply:^(id reply) {
                   completion(nil);
                 }];
}
- (void)onMediaRelayStatesChangeState:(NSNumber *)arg_state
                          channelName:(NSString *)arg_channelName
                           completion:(void (^)(FlutterError *_Nullable))completion {
  FlutterBasicMessageChannel *channel = [FlutterBasicMessageChannel
      messageChannelWithName:@"dev.flutter.pigeon.NERtcChannelEventSink.onMediaRelayStatesChange"
             binaryMessenger:self.binaryMessenger
                       codec:NEFLTNERtcChannelEventSinkGetCodec()];
  [channel sendMessage:@[ arg_state ?: [NSNull null], arg_channelName ?: [NSNull null] ]
                 reply:^(id reply) {
                   completion(nil);
                 }];
}
- (void)onMediaRelayReceiveEventEvent:(NSNumber *)arg_event
                                 code:(NSNumber *)arg_code
                          channelName:(NSString *)arg_channelName
                           completion:(void (^)(FlutterError *_Nullable))completion {
  FlutterBasicMessageChannel *channel = [FlutterBasicMessageChannel
      messageChannelWithName:@"dev.flutter.pigeon.NERtcChannelEventSink.onMediaRelayReceiveEvent"
             binaryMessenger:self.binaryMessenger
                       codec:NEFLTNERtcChannelEventSinkGetCodec()];
  [channel sendMessage:@[
    arg_event ?: [NSNull null], arg_code ?: [NSNull null], arg_channelName ?: [NSNull null]
  ]
                 reply:^(id reply) {
                   completion(nil);
                 }];
}
- (void)onLocalPublishFallbackToAudioOnlyIsFallback:(NSNumber *)arg_isFallback
                                         streamType:(NSNumber *)arg_streamType
                                         completion:(void (^)(FlutterError *_Nullable))completion {
  FlutterBasicMessageChannel *channel = [FlutterBasicMessageChannel
      messageChannelWithName:
          @"dev.flutter.pigeon.NERtcChannelEventSink.onLocalPublishFallbackToAudioOnly"
             binaryMessenger:self.binaryMessenger
                       codec:NEFLTNERtcChannelEventSinkGetCodec()];
  [channel sendMessage:@[ arg_isFallback ?: [NSNull null], arg_streamType ?: [NSNull null] ]
                 reply:^(id reply) {
                   completion(nil);
                 }];
}
- (void)onRemoteSubscribeFallbackToAudioOnlyUid:(NSNumber *)arg_uid
                                     isFallback:(NSNumber *)arg_isFallback
                                     streamType:(NSNumber *)arg_streamType
                                     completion:(void (^)(FlutterError *_Nullable))completion {
  FlutterBasicMessageChannel *channel = [FlutterBasicMessageChannel
      messageChannelWithName:
          @"dev.flutter.pigeon.NERtcChannelEventSink.onRemoteSubscribeFallbackToAudioOnly"
             binaryMessenger:self.binaryMessenger
                       codec:NEFLTNERtcChannelEventSinkGetCodec()];
  [channel sendMessage:@[
    arg_uid ?: [NSNull null], arg_isFallback ?: [NSNull null], arg_streamType ?: [NSNull null]
  ]
                 reply:^(id reply) {
                   completion(nil);
                 }];
}
- (void)onLocalVideoWatermarkStateVideoStreamType:(NSNumber *)arg_videoStreamType
                                            state:(NSNumber *)arg_state
                                       completion:(void (^)(FlutterError *_Nullable))completion {
  FlutterBasicMessageChannel *channel = [FlutterBasicMessageChannel
      messageChannelWithName:@"dev.flutter.pigeon.NERtcChannelEventSink.onLocalVideoWatermarkState"
             binaryMessenger:self.binaryMessenger
                       codec:NEFLTNERtcChannelEventSinkGetCodec()];
  [channel sendMessage:@[ arg_videoStreamType ?: [NSNull null], arg_state ?: [NSNull null] ]
                 reply:^(id reply) {
                   completion(nil);
                 }];
}
- (void)onLastmileQualityQuality:(NSNumber *)arg_quality
                      completion:(void (^)(FlutterError *_Nullable))completion {
  FlutterBasicMessageChannel *channel = [FlutterBasicMessageChannel
      messageChannelWithName:@"dev.flutter.pigeon.NERtcChannelEventSink.onLastmileQuality"
             binaryMessenger:self.binaryMessenger
                       codec:NEFLTNERtcChannelEventSinkGetCodec()];
  [channel sendMessage:@[ arg_quality ?: [NSNull null] ]
                 reply:^(id reply) {
                   completion(nil);
                 }];
}
- (void)onLastmileProbeResultResult:(NEFLTNERtcLastmileProbeResult *)arg_result
                         completion:(void (^)(FlutterError *_Nullable))completion {
  FlutterBasicMessageChannel *channel = [FlutterBasicMessageChannel
      messageChannelWithName:@"dev.flutter.pigeon.NERtcChannelEventSink.onLastmileProbeResult"
             binaryMessenger:self.binaryMessenger
                       codec:NEFLTNERtcChannelEventSinkGetCodec()];
  [channel sendMessage:@[ arg_result ?: [NSNull null] ]
                 reply:^(id reply) {
                   completion(nil);
                 }];
}
- (void)onTakeSnapshotResultCode:(NSNumber *)arg_code
                            path:(NSString *)arg_path
                      completion:(void (^)(FlutterError *_Nullable))completion {
  FlutterBasicMessageChannel *channel = [FlutterBasicMessageChannel
      messageChannelWithName:@"dev.flutter.pigeon.NERtcChannelEventSink.onTakeSnapshotResult"
             binaryMessenger:self.binaryMessenger
                       codec:NEFLTNERtcChannelEventSinkGetCodec()];
  [channel sendMessage:@[ arg_code ?: [NSNull null], arg_path ?: [NSNull null] ]
                 reply:^(id reply) {
                   completion(nil);
                 }];
}
@end

@interface NEFLTEngineApiCodecReader : FlutterStandardReader
@end
@implementation NEFLTEngineApiCodecReader
- (nullable id)readValueOfType:(UInt8)type {
  switch (type) {
    case 128:
      return [NEFLTAddOrUpdateLiveStreamTaskRequest fromList:[self readValue]];
    case 129:
      return [NEFLTAdjustUserPlaybackSignalVolumeRequest fromList:[self readValue]];
    case 130:
      return [NEFLTAudioRecordingConfigurationRequest fromList:[self readValue]];
    case 131:
      return [NEFLTAudioVolumeInfo fromList:[self readValue]];
    case 132:
      return [NEFLTCGPoint fromList:[self readValue]];
    case 133:
      return [NEFLTCreateEngineRequest fromList:[self readValue]];
    case 134:
      return [NEFLTDeleteLiveStreamTaskRequest fromList:[self readValue]];
    case 135:
      return [NEFLTEnableAudioVolumeIndicationRequest fromList:[self readValue]];
    case 136:
      return [NEFLTEnableEncryptionRequest fromList:[self readValue]];
    case 137:
      return [NEFLTEnableLocalVideoRequest fromList:[self readValue]];
    case 138:
      return [NEFLTEnableVirtualBackgroundRequest fromList:[self readValue]];
    case 139:
      return [NEFLTFirstVideoDataReceivedEvent fromList:[self readValue]];
    case 140:
      return [NEFLTFirstVideoFrameDecodedEvent fromList:[self readValue]];
    case 141:
      return [NEFLTJoinChannelOptions fromList:[self readValue]];
    case 142:
      return [NEFLTJoinChannelRequest fromList:[self readValue]];
    case 143:
      return [NEFLTNERtcLastmileProbeOneWayResult fromList:[self readValue]];
    case 144:
      return [NEFLTNERtcLastmileProbeResult fromList:[self readValue]];
    case 145:
      return [NEFLTNERtcUserJoinExtraInfo fromList:[self readValue]];
    case 146:
      return [NEFLTNERtcUserLeaveExtraInfo fromList:[self readValue]];
    case 147:
      return [NEFLTNERtcVersion fromList:[self readValue]];
    case 148:
      return [NEFLTPlayEffectRequest fromList:[self readValue]];
    case 149:
      return [NEFLTRemoteAudioVolumeIndicationEvent fromList:[self readValue]];
    case 150:
      return [NEFLTReportCustomEventRequest fromList:[self readValue]];
    case 151:
      return [NEFLTRtcServerAddresses fromList:[self readValue]];
    case 152:
      return [NEFLTSendSEIMsgRequest fromList:[self readValue]];
    case 153:
      return [NEFLTSetAudioProfileRequest fromList:[self readValue]];
    case 154:
      return [NEFLTSetAudioSubscribeOnlyByRequest fromList:[self readValue]];
    case 155:
      return [NEFLTSetCameraCaptureConfigRequest fromList:[self readValue]];
    case 156:
      return [NEFLTSetCameraPositionRequest fromList:[self readValue]];
    case 157:
      return [NEFLTSetLocalMediaPriorityRequest fromList:[self readValue]];
    case 158:
      return [NEFLTSetLocalVideoConfigRequest fromList:[self readValue]];
    case 159:
      return [NEFLTSetLocalVideoWatermarkConfigsRequest fromList:[self readValue]];
    case 160:
      return [NEFLTSetLocalVoiceEqualizationRequest fromList:[self readValue]];
    case 161:
      return [NEFLTSetLocalVoiceReverbParamRequest fromList:[self readValue]];
    case 162:
      return [NEFLTSetRemoteHighPriorityAudioStreamRequest fromList:[self readValue]];
    case 163:
      return [NEFLTSetVideoCorrectionConfigRequest fromList:[self readValue]];
    case 164:
      return [NEFLTStartAudioMixingRequest fromList:[self readValue]];
    case 165:
      return [NEFLTStartAudioRecordingRequest fromList:[self readValue]];
    case 166:
      return [NEFLTStartLastmileProbeTestRequest fromList:[self readValue]];
    case 167:
      return [NEFLTStartOrUpdateChannelMediaRelayRequest fromList:[self readValue]];
    case 168:
      return [NEFLTStartScreenCaptureRequest fromList:[self readValue]];
    case 169:
      return [NEFLTStartorStopVideoPreviewRequest fromList:[self readValue]];
    case 170:
      return [NEFLTSubscribeRemoteAudioRequest fromList:[self readValue]];
    case 171:
      return [NEFLTSubscribeRemoteSubStreamAudioRequest fromList:[self readValue]];
    case 172:
      return [NEFLTSubscribeRemoteSubStreamVideoRequest fromList:[self readValue]];
    case 173:
      return [NEFLTSubscribeRemoteVideoStreamRequest fromList:[self readValue]];
    case 174:
      return [NEFLTSwitchChannelRequest fromList:[self readValue]];
    case 175:
      return [NEFLTUserJoinedEvent fromList:[self readValue]];
    case 176:
      return [NEFLTUserLeaveEvent fromList:[self readValue]];
    case 177:
      return [NEFLTUserVideoMuteEvent fromList:[self readValue]];
    case 178:
      return [NEFLTVideoWatermarkConfig fromList:[self readValue]];
    case 179:
      return [NEFLTVideoWatermarkImageConfig fromList:[self readValue]];
    case 180:
      return [NEFLTVideoWatermarkTextConfig fromList:[self readValue]];
    case 181:
      return [NEFLTVideoWatermarkTimestampConfig fromList:[self readValue]];
    case 182:
      return [NEFLTVirtualBackgroundSourceEnabledEvent fromList:[self readValue]];
    default:
      return [super readValueOfType:type];
  }
}
@end

@interface NEFLTEngineApiCodecWriter : FlutterStandardWriter
@end
@implementation NEFLTEngineApiCodecWriter
- (void)writeValue:(id)value {
  if ([value isKindOfClass:[NEFLTAddOrUpdateLiveStreamTaskRequest class]]) {
    [self writeByte:128];
    [self writeValue:[value toList]];
  } else if ([value isKindOfClass:[NEFLTAdjustUserPlaybackSignalVolumeRequest class]]) {
    [self writeByte:129];
    [self writeValue:[value toList]];
  } else if ([value isKindOfClass:[NEFLTAudioRecordingConfigurationRequest class]]) {
    [self writeByte:130];
    [self writeValue:[value toList]];
  } else if ([value isKindOfClass:[NEFLTAudioVolumeInfo class]]) {
    [self writeByte:131];
    [self writeValue:[value toList]];
  } else if ([value isKindOfClass:[NEFLTCGPoint class]]) {
    [self writeByte:132];
    [self writeValue:[value toList]];
  } else if ([value isKindOfClass:[NEFLTCreateEngineRequest class]]) {
    [self writeByte:133];
    [self writeValue:[value toList]];
  } else if ([value isKindOfClass:[NEFLTDeleteLiveStreamTaskRequest class]]) {
    [self writeByte:134];
    [self writeValue:[value toList]];
  } else if ([value isKindOfClass:[NEFLTEnableAudioVolumeIndicationRequest class]]) {
    [self writeByte:135];
    [self writeValue:[value toList]];
  } else if ([value isKindOfClass:[NEFLTEnableEncryptionRequest class]]) {
    [self writeByte:136];
    [self writeValue:[value toList]];
  } else if ([value isKindOfClass:[NEFLTEnableLocalVideoRequest class]]) {
    [self writeByte:137];
    [self writeValue:[value toList]];
  } else if ([value isKindOfClass:[NEFLTEnableVirtualBackgroundRequest class]]) {
    [self writeByte:138];
    [self writeValue:[value toList]];
  } else if ([value isKindOfClass:[NEFLTFirstVideoDataReceivedEvent class]]) {
    [self writeByte:139];
    [self writeValue:[value toList]];
  } else if ([value isKindOfClass:[NEFLTFirstVideoFrameDecodedEvent class]]) {
    [self writeByte:140];
    [self writeValue:[value toList]];
  } else if ([value isKindOfClass:[NEFLTJoinChannelOptions class]]) {
    [self writeByte:141];
    [self writeValue:[value toList]];
  } else if ([value isKindOfClass:[NEFLTJoinChannelRequest class]]) {
    [self writeByte:142];
    [self writeValue:[value toList]];
  } else if ([value isKindOfClass:[NEFLTNERtcLastmileProbeOneWayResult class]]) {
    [self writeByte:143];
    [self writeValue:[value toList]];
  } else if ([value isKindOfClass:[NEFLTNERtcLastmileProbeResult class]]) {
    [self writeByte:144];
    [self writeValue:[value toList]];
  } else if ([value isKindOfClass:[NEFLTNERtcUserJoinExtraInfo class]]) {
    [self writeByte:145];
    [self writeValue:[value toList]];
  } else if ([value isKindOfClass:[NEFLTNERtcUserLeaveExtraInfo class]]) {
    [self writeByte:146];
    [self writeValue:[value toList]];
  } else if ([value isKindOfClass:[NEFLTNERtcVersion class]]) {
    [self writeByte:147];
    [self writeValue:[value toList]];
  } else if ([value isKindOfClass:[NEFLTPlayEffectRequest class]]) {
    [self writeByte:148];
    [self writeValue:[value toList]];
  } else if ([value isKindOfClass:[NEFLTRemoteAudioVolumeIndicationEvent class]]) {
    [self writeByte:149];
    [self writeValue:[value toList]];
  } else if ([value isKindOfClass:[NEFLTReportCustomEventRequest class]]) {
    [self writeByte:150];
    [self writeValue:[value toList]];
  } else if ([value isKindOfClass:[NEFLTRtcServerAddresses class]]) {
    [self writeByte:151];
    [self writeValue:[value toList]];
  } else if ([value isKindOfClass:[NEFLTSendSEIMsgRequest class]]) {
    [self writeByte:152];
    [self writeValue:[value toList]];
  } else if ([value isKindOfClass:[NEFLTSetAudioProfileRequest class]]) {
    [self writeByte:153];
    [self writeValue:[value toList]];
  } else if ([value isKindOfClass:[NEFLTSetAudioSubscribeOnlyByRequest class]]) {
    [self writeByte:154];
    [self writeValue:[value toList]];
  } else if ([value isKindOfClass:[NEFLTSetCameraCaptureConfigRequest class]]) {
    [self writeByte:155];
    [self writeValue:[value toList]];
  } else if ([value isKindOfClass:[NEFLTSetCameraPositionRequest class]]) {
    [self writeByte:156];
    [self writeValue:[value toList]];
  } else if ([value isKindOfClass:[NEFLTSetLocalMediaPriorityRequest class]]) {
    [self writeByte:157];
    [self writeValue:[value toList]];
  } else if ([value isKindOfClass:[NEFLTSetLocalVideoConfigRequest class]]) {
    [self writeByte:158];
    [self writeValue:[value toList]];
  } else if ([value isKindOfClass:[NEFLTSetLocalVideoWatermarkConfigsRequest class]]) {
    [self writeByte:159];
    [self writeValue:[value toList]];
  } else if ([value isKindOfClass:[NEFLTSetLocalVoiceEqualizationRequest class]]) {
    [self writeByte:160];
    [self writeValue:[value toList]];
  } else if ([value isKindOfClass:[NEFLTSetLocalVoiceReverbParamRequest class]]) {
    [self writeByte:161];
    [self writeValue:[value toList]];
  } else if ([value isKindOfClass:[NEFLTSetRemoteHighPriorityAudioStreamRequest class]]) {
    [self writeByte:162];
    [self writeValue:[value toList]];
  } else if ([value isKindOfClass:[NEFLTSetVideoCorrectionConfigRequest class]]) {
    [self writeByte:163];
    [self writeValue:[value toList]];
  } else if ([value isKindOfClass:[NEFLTStartAudioMixingRequest class]]) {
    [self writeByte:164];
    [self writeValue:[value toList]];
  } else if ([value isKindOfClass:[NEFLTStartAudioRecordingRequest class]]) {
    [self writeByte:165];
    [self writeValue:[value toList]];
  } else if ([value isKindOfClass:[NEFLTStartLastmileProbeTestRequest class]]) {
    [self writeByte:166];
    [self writeValue:[value toList]];
  } else if ([value isKindOfClass:[NEFLTStartOrUpdateChannelMediaRelayRequest class]]) {
    [self writeByte:167];
    [self writeValue:[value toList]];
  } else if ([value isKindOfClass:[NEFLTStartScreenCaptureRequest class]]) {
    [self writeByte:168];
    [self writeValue:[value toList]];
  } else if ([value isKindOfClass:[NEFLTStartorStopVideoPreviewRequest class]]) {
    [self writeByte:169];
    [self writeValue:[value toList]];
  } else if ([value isKindOfClass:[NEFLTSubscribeRemoteAudioRequest class]]) {
    [self writeByte:170];
    [self writeValue:[value toList]];
  } else if ([value isKindOfClass:[NEFLTSubscribeRemoteSubStreamAudioRequest class]]) {
    [self writeByte:171];
    [self writeValue:[value toList]];
  } else if ([value isKindOfClass:[NEFLTSubscribeRemoteSubStreamVideoRequest class]]) {
    [self writeByte:172];
    [self writeValue:[value toList]];
  } else if ([value isKindOfClass:[NEFLTSubscribeRemoteVideoStreamRequest class]]) {
    [self writeByte:173];
    [self writeValue:[value toList]];
  } else if ([value isKindOfClass:[NEFLTSwitchChannelRequest class]]) {
    [self writeByte:174];
    [self writeValue:[value toList]];
  } else if ([value isKindOfClass:[NEFLTUserJoinedEvent class]]) {
    [self writeByte:175];
    [self writeValue:[value toList]];
  } else if ([value isKindOfClass:[NEFLTUserLeaveEvent class]]) {
    [self writeByte:176];
    [self writeValue:[value toList]];
  } else if ([value isKindOfClass:[NEFLTUserVideoMuteEvent class]]) {
    [self writeByte:177];
    [self writeValue:[value toList]];
  } else if ([value isKindOfClass:[NEFLTVideoWatermarkConfig class]]) {
    [self writeByte:178];
    [self writeValue:[value toList]];
  } else if ([value isKindOfClass:[NEFLTVideoWatermarkImageConfig class]]) {
    [self writeByte:179];
    [self writeValue:[value toList]];
  } else if ([value isKindOfClass:[NEFLTVideoWatermarkTextConfig class]]) {
    [self writeByte:180];
    [self writeValue:[value toList]];
  } else if ([value isKindOfClass:[NEFLTVideoWatermarkTimestampConfig class]]) {
    [self writeByte:181];
    [self writeValue:[value toList]];
  } else if ([value isKindOfClass:[NEFLTVirtualBackgroundSourceEnabledEvent class]]) {
    [self writeByte:182];
    [self writeValue:[value toList]];
  } else {
    [super writeValue:value];
  }
}
@end

@interface NEFLTEngineApiCodecReaderWriter : FlutterStandardReaderWriter
@end
@implementation NEFLTEngineApiCodecReaderWriter
- (FlutterStandardWriter *)writerWithData:(NSMutableData *)data {
  return [[NEFLTEngineApiCodecWriter alloc] initWithData:data];
}
- (FlutterStandardReader *)readerWithData:(NSData *)data {
  return [[NEFLTEngineApiCodecReader alloc] initWithData:data];
}
@end

NSObject<FlutterMessageCodec> *NEFLTEngineApiGetCodec() {
  static FlutterStandardMessageCodec *sSharedObject = nil;
  static dispatch_once_t sPred = 0;
  dispatch_once(&sPred, ^{
    NEFLTEngineApiCodecReaderWriter *readerWriter = [[NEFLTEngineApiCodecReaderWriter alloc] init];
    sSharedObject = [FlutterStandardMessageCodec codecWithReaderWriter:readerWriter];
  });
  return sSharedObject;
}

void NEFLTEngineApiSetup(id<FlutterBinaryMessenger> binaryMessenger,
                         NSObject<NEFLTEngineApi> *api) {
  {
    FlutterBasicMessageChannel *channel =
        [[FlutterBasicMessageChannel alloc] initWithName:@"dev.flutter.pigeon.EngineApi.create"
                                         binaryMessenger:binaryMessenger
                                                   codec:NEFLTEngineApiGetCodec()];
    if (api) {
      NSCAssert([api respondsToSelector:@selector(createRequest:error:)],
                @"NEFLTEngineApi api (%@) doesn't respond to @selector(createRequest:error:)", api);
      [channel setMessageHandler:^(id _Nullable message, FlutterReply callback) {
        NSArray *args = message;
        NEFLTCreateEngineRequest *arg_request = GetNullableObjectAtIndex(args, 0);
        FlutterError *error;
        NSNumber *output = [api createRequest:arg_request error:&error];
        callback(wrapResult(output, error));
      }];
    } else {
      [channel setMessageHandler:nil];
    }
  }
  {
    FlutterBasicMessageChannel *channel =
        [[FlutterBasicMessageChannel alloc] initWithName:@"dev.flutter.pigeon.EngineApi.version"
                                         binaryMessenger:binaryMessenger
                                                   codec:NEFLTEngineApiGetCodec()];
    if (api) {
      NSCAssert([api respondsToSelector:@selector(versionWithError:)],
                @"NEFLTEngineApi api (%@) doesn't respond to @selector(versionWithError:)", api);
      [channel setMessageHandler:^(id _Nullable message, FlutterReply callback) {
        FlutterError *error;
        NEFLTNERtcVersion *output = [api versionWithError:&error];
        callback(wrapResult(output, error));
      }];
    } else {
      [channel setMessageHandler:nil];
    }
  }
  {
    FlutterBasicMessageChannel *channel = [[FlutterBasicMessageChannel alloc]
           initWithName:@"dev.flutter.pigeon.EngineApi.checkPermission"
        binaryMessenger:binaryMessenger
                  codec:NEFLTEngineApiGetCodec()];
    if (api) {
      NSCAssert([api respondsToSelector:@selector(checkPermissionWithError:)],
                @"NEFLTEngineApi api (%@) doesn't respond to @selector(checkPermissionWithError:)",
                api);
      [channel setMessageHandler:^(id _Nullable message, FlutterReply callback) {
        FlutterError *error;
        NSArray<NSString *> *output = [api checkPermissionWithError:&error];
        callback(wrapResult(output, error));
      }];
    } else {
      [channel setMessageHandler:nil];
    }
  }
  {
    FlutterBasicMessageChannel *channel = [[FlutterBasicMessageChannel alloc]
           initWithName:@"dev.flutter.pigeon.EngineApi.setParameters"
        binaryMessenger:binaryMessenger
                  codec:NEFLTEngineApiGetCodec()];
    if (api) {
      NSCAssert([api respondsToSelector:@selector(setParametersParams:error:)],
                @"NEFLTEngineApi api (%@) doesn't respond to @selector(setParametersParams:error:)",
                api);
      [channel setMessageHandler:^(id _Nullable message, FlutterReply callback) {
        NSArray *args = message;
        NSDictionary<NSString *, id> *arg_params = GetNullableObjectAtIndex(args, 0);
        FlutterError *error;
        NSNumber *output = [api setParametersParams:arg_params error:&error];
        callback(wrapResult(output, error));
      }];
    } else {
      [channel setMessageHandler:nil];
    }
  }
  {
    FlutterBasicMessageChannel *channel =
        [[FlutterBasicMessageChannel alloc] initWithName:@"dev.flutter.pigeon.EngineApi.release"
                                         binaryMessenger:binaryMessenger
                                                   codec:NEFLTEngineApiGetCodec()];
    if (api) {
      NSCAssert([api respondsToSelector:@selector(releaseWithCompletion:)],
                @"NEFLTEngineApi api (%@) doesn't respond to @selector(releaseWithCompletion:)",
                api);
      [channel setMessageHandler:^(id _Nullable message, FlutterReply callback) {
        [api releaseWithCompletion:^(NSNumber *_Nullable output, FlutterError *_Nullable error) {
          callback(wrapResult(output, error));
        }];
      }];
    } else {
      [channel setMessageHandler:nil];
    }
  }
  {
    FlutterBasicMessageChannel *channel = [[FlutterBasicMessageChannel alloc]
           initWithName:@"dev.flutter.pigeon.EngineApi.setStatsEventCallback"
        binaryMessenger:binaryMessenger
                  codec:NEFLTEngineApiGetCodec()];
    if (api) {
      NSCAssert(
          [api respondsToSelector:@selector(setStatsEventCallbackWithError:)],
          @"NEFLTEngineApi api (%@) doesn't respond to @selector(setStatsEventCallbackWithError:)",
          api);
      [channel setMessageHandler:^(id _Nullable message, FlutterReply callback) {
        FlutterError *error;
        NSNumber *output = [api setStatsEventCallbackWithError:&error];
        callback(wrapResult(output, error));
      }];
    } else {
      [channel setMessageHandler:nil];
    }
  }
  {
    FlutterBasicMessageChannel *channel = [[FlutterBasicMessageChannel alloc]
           initWithName:@"dev.flutter.pigeon.EngineApi.clearStatsEventCallback"
        binaryMessenger:binaryMessenger
                  codec:NEFLTEngineApiGetCodec()];
    if (api) {
      NSCAssert([api respondsToSelector:@selector(clearStatsEventCallbackWithError:)],
                @"NEFLTEngineApi api (%@) doesn't respond to "
                @"@selector(clearStatsEventCallbackWithError:)",
                api);
      [channel setMessageHandler:^(id _Nullable message, FlutterReply callback) {
        FlutterError *error;
        NSNumber *output = [api clearStatsEventCallbackWithError:&error];
        callback(wrapResult(output, error));
      }];
    } else {
      [channel setMessageHandler:nil];
    }
  }
  {
    FlutterBasicMessageChannel *channel = [[FlutterBasicMessageChannel alloc]
           initWithName:@"dev.flutter.pigeon.EngineApi.setChannelProfile"
        binaryMessenger:binaryMessenger
                  codec:NEFLTEngineApiGetCodec()];
    if (api) {
      NSCAssert([api respondsToSelector:@selector(setChannelProfileChannelProfile:error:)],
                @"NEFLTEngineApi api (%@) doesn't respond to "
                @"@selector(setChannelProfileChannelProfile:error:)",
                api);
      [channel setMessageHandler:^(id _Nullable message, FlutterReply callback) {
        NSArray *args = message;
        NSNumber *arg_channelProfile = GetNullableObjectAtIndex(args, 0);
        FlutterError *error;
        NSNumber *output = [api setChannelProfileChannelProfile:arg_channelProfile error:&error];
        callback(wrapResult(output, error));
      }];
    } else {
      [channel setMessageHandler:nil];
    }
  }
  {
    FlutterBasicMessageChannel *channel =
        [[FlutterBasicMessageChannel alloc] initWithName:@"dev.flutter.pigeon.EngineApi.joinChannel"
                                         binaryMessenger:binaryMessenger
                                                   codec:NEFLTEngineApiGetCodec()];
    if (api) {
      NSCAssert([api respondsToSelector:@selector(joinChannelRequest:error:)],
                @"NEFLTEngineApi api (%@) doesn't respond to @selector(joinChannelRequest:error:)",
                api);
      [channel setMessageHandler:^(id _Nullable message, FlutterReply callback) {
        NSArray *args = message;
        NEFLTJoinChannelRequest *arg_request = GetNullableObjectAtIndex(args, 0);
        FlutterError *error;
        NSNumber *output = [api joinChannelRequest:arg_request error:&error];
        callback(wrapResult(output, error));
      }];
    } else {
      [channel setMessageHandler:nil];
    }
  }
  {
    FlutterBasicMessageChannel *channel = [[FlutterBasicMessageChannel alloc]
           initWithName:@"dev.flutter.pigeon.EngineApi.leaveChannel"
        binaryMessenger:binaryMessenger
                  codec:NEFLTEngineApiGetCodec()];
    if (api) {
      NSCAssert([api respondsToSelector:@selector(leaveChannelWithError:)],
                @"NEFLTEngineApi api (%@) doesn't respond to @selector(leaveChannelWithError:)",
                api);
      [channel setMessageHandler:^(id _Nullable message, FlutterReply callback) {
        FlutterError *error;
        NSNumber *output = [api leaveChannelWithError:&error];
        callback(wrapResult(output, error));
      }];
    } else {
      [channel setMessageHandler:nil];
    }
  }
  {
    FlutterBasicMessageChannel *channel = [[FlutterBasicMessageChannel alloc]
           initWithName:@"dev.flutter.pigeon.EngineApi.updatePermissionKey"
        binaryMessenger:binaryMessenger
                  codec:NEFLTEngineApiGetCodec()];
    if (api) {
      NSCAssert(
          [api respondsToSelector:@selector(updatePermissionKeyKey:error:)],
          @"NEFLTEngineApi api (%@) doesn't respond to @selector(updatePermissionKeyKey:error:)",
          api);
      [channel setMessageHandler:^(id _Nullable message, FlutterReply callback) {
        NSArray *args = message;
        NSString *arg_key = GetNullableObjectAtIndex(args, 0);
        FlutterError *error;
        NSNumber *output = [api updatePermissionKeyKey:arg_key error:&error];
        callback(wrapResult(output, error));
      }];
    } else {
      [channel setMessageHandler:nil];
    }
  }
  {
    FlutterBasicMessageChannel *channel = [[FlutterBasicMessageChannel alloc]
           initWithName:@"dev.flutter.pigeon.EngineApi.enableLocalAudio"
        binaryMessenger:binaryMessenger
                  codec:NEFLTEngineApiGetCodec()];
    if (api) {
      NSCAssert(
          [api respondsToSelector:@selector(enableLocalAudioEnable:error:)],
          @"NEFLTEngineApi api (%@) doesn't respond to @selector(enableLocalAudioEnable:error:)",
          api);
      [channel setMessageHandler:^(id _Nullable message, FlutterReply callback) {
        NSArray *args = message;
        NSNumber *arg_enable = GetNullableObjectAtIndex(args, 0);
        FlutterError *error;
        NSNumber *output = [api enableLocalAudioEnable:arg_enable error:&error];
        callback(wrapResult(output, error));
      }];
    } else {
      [channel setMessageHandler:nil];
    }
  }
  {
    FlutterBasicMessageChannel *channel = [[FlutterBasicMessageChannel alloc]
           initWithName:@"dev.flutter.pigeon.EngineApi.subscribeRemoteAudio"
        binaryMessenger:binaryMessenger
                  codec:NEFLTEngineApiGetCodec()];
    if (api) {
      NSCAssert([api respondsToSelector:@selector(subscribeRemoteAudioRequest:error:)],
                @"NEFLTEngineApi api (%@) doesn't respond to "
                @"@selector(subscribeRemoteAudioRequest:error:)",
                api);
      [channel setMessageHandler:^(id _Nullable message, FlutterReply callback) {
        NSArray *args = message;
        NEFLTSubscribeRemoteAudioRequest *arg_request = GetNullableObjectAtIndex(args, 0);
        FlutterError *error;
        NSNumber *output = [api subscribeRemoteAudioRequest:arg_request error:&error];
        callback(wrapResult(output, error));
      }];
    } else {
      [channel setMessageHandler:nil];
    }
  }
  {
    FlutterBasicMessageChannel *channel = [[FlutterBasicMessageChannel alloc]
           initWithName:@"dev.flutter.pigeon.EngineApi.subscribeAllRemoteAudio"
        binaryMessenger:binaryMessenger
                  codec:NEFLTEngineApiGetCodec()];
    if (api) {
      NSCAssert([api respondsToSelector:@selector(subscribeAllRemoteAudioSubscribe:error:)],
                @"NEFLTEngineApi api (%@) doesn't respond to "
                @"@selector(subscribeAllRemoteAudioSubscribe:error:)",
                api);
      [channel setMessageHandler:^(id _Nullable message, FlutterReply callback) {
        NSArray *args = message;
        NSNumber *arg_subscribe = GetNullableObjectAtIndex(args, 0);
        FlutterError *error;
        NSNumber *output = [api subscribeAllRemoteAudioSubscribe:arg_subscribe error:&error];
        callback(wrapResult(output, error));
      }];
    } else {
      [channel setMessageHandler:nil];
    }
  }
  {
    FlutterBasicMessageChannel *channel = [[FlutterBasicMessageChannel alloc]
           initWithName:@"dev.flutter.pigeon.EngineApi.setAudioProfile"
        binaryMessenger:binaryMessenger
                  codec:NEFLTEngineApiGetCodec()];
    if (api) {
      NSCAssert(
          [api respondsToSelector:@selector(setAudioProfileRequest:error:)],
          @"NEFLTEngineApi api (%@) doesn't respond to @selector(setAudioProfileRequest:error:)",
          api);
      [channel setMessageHandler:^(id _Nullable message, FlutterReply callback) {
        NSArray *args = message;
        NEFLTSetAudioProfileRequest *arg_request = GetNullableObjectAtIndex(args, 0);
        FlutterError *error;
        NSNumber *output = [api setAudioProfileRequest:arg_request error:&error];
        callback(wrapResult(output, error));
      }];
    } else {
      [channel setMessageHandler:nil];
    }
  }
  {
    FlutterBasicMessageChannel *channel = [[FlutterBasicMessageChannel alloc]
           initWithName:@"dev.flutter.pigeon.EngineApi.enableDualStreamMode"
        binaryMessenger:binaryMessenger
                  codec:NEFLTEngineApiGetCodec()];
    if (api) {
      NSCAssert([api respondsToSelector:@selector(enableDualStreamModeEnable:error:)],
                @"NEFLTEngineApi api (%@) doesn't respond to "
                @"@selector(enableDualStreamModeEnable:error:)",
                api);
      [channel setMessageHandler:^(id _Nullable message, FlutterReply callback) {
        NSArray *args = message;
        NSNumber *arg_enable = GetNullableObjectAtIndex(args, 0);
        FlutterError *error;
        NSNumber *output = [api enableDualStreamModeEnable:arg_enable error:&error];
        callback(wrapResult(output, error));
      }];
    } else {
      [channel setMessageHandler:nil];
    }
  }
  {
    FlutterBasicMessageChannel *channel = [[FlutterBasicMessageChannel alloc]
           initWithName:@"dev.flutter.pigeon.EngineApi.setLocalVideoConfig"
        binaryMessenger:binaryMessenger
                  codec:NEFLTEngineApiGetCodec()];
    if (api) {
      NSCAssert([api respondsToSelector:@selector(setLocalVideoConfigRequest:error:)],
                @"NEFLTEngineApi api (%@) doesn't respond to "
                @"@selector(setLocalVideoConfigRequest:error:)",
                api);
      [channel setMessageHandler:^(id _Nullable message, FlutterReply callback) {
        NSArray *args = message;
        NEFLTSetLocalVideoConfigRequest *arg_request = GetNullableObjectAtIndex(args, 0);
        FlutterError *error;
        NSNumber *output = [api setLocalVideoConfigRequest:arg_request error:&error];
        callback(wrapResult(output, error));
      }];
    } else {
      [channel setMessageHandler:nil];
    }
  }
  {
    FlutterBasicMessageChannel *channel = [[FlutterBasicMessageChannel alloc]
           initWithName:@"dev.flutter.pigeon.EngineApi.setCameraCaptureConfig"
        binaryMessenger:binaryMessenger
                  codec:NEFLTEngineApiGetCodec()];
    if (api) {
      NSCAssert([api respondsToSelector:@selector(setCameraCaptureConfigRequest:error:)],
                @"NEFLTEngineApi api (%@) doesn't respond to "
                @"@selector(setCameraCaptureConfigRequest:error:)",
                api);
      [channel setMessageHandler:^(id _Nullable message, FlutterReply callback) {
        NSArray *args = message;
        NEFLTSetCameraCaptureConfigRequest *arg_request = GetNullableObjectAtIndex(args, 0);
        FlutterError *error;
        NSNumber *output = [api setCameraCaptureConfigRequest:arg_request error:&error];
        callback(wrapResult(output, error));
      }];
    } else {
      [channel setMessageHandler:nil];
    }
  }
  {
    FlutterBasicMessageChannel *channel = [[FlutterBasicMessageChannel alloc]
           initWithName:@"dev.flutter.pigeon.EngineApi.startVideoPreview"
        binaryMessenger:binaryMessenger
                  codec:NEFLTEngineApiGetCodec()];
    if (api) {
      NSCAssert(
          [api respondsToSelector:@selector(startVideoPreviewRequest:error:)],
          @"NEFLTEngineApi api (%@) doesn't respond to @selector(startVideoPreviewRequest:error:)",
          api);
      [channel setMessageHandler:^(id _Nullable message, FlutterReply callback) {
        NSArray *args = message;
        NEFLTStartorStopVideoPreviewRequest *arg_request = GetNullableObjectAtIndex(args, 0);
        FlutterError *error;
        NSNumber *output = [api startVideoPreviewRequest:arg_request error:&error];
        callback(wrapResult(output, error));
      }];
    } else {
      [channel setMessageHandler:nil];
    }
  }
  {
    FlutterBasicMessageChannel *channel = [[FlutterBasicMessageChannel alloc]
           initWithName:@"dev.flutter.pigeon.EngineApi.stopVideoPreview"
        binaryMessenger:binaryMessenger
                  codec:NEFLTEngineApiGetCodec()];
    if (api) {
      NSCAssert(
          [api respondsToSelector:@selector(stopVideoPreviewRequest:error:)],
          @"NEFLTEngineApi api (%@) doesn't respond to @selector(stopVideoPreviewRequest:error:)",
          api);
      [channel setMessageHandler:^(id _Nullable message, FlutterReply callback) {
        NSArray *args = message;
        NEFLTStartorStopVideoPreviewRequest *arg_request = GetNullableObjectAtIndex(args, 0);
        FlutterError *error;
        NSNumber *output = [api stopVideoPreviewRequest:arg_request error:&error];
        callback(wrapResult(output, error));
      }];
    } else {
      [channel setMessageHandler:nil];
    }
  }
  {
    FlutterBasicMessageChannel *channel = [[FlutterBasicMessageChannel alloc]
           initWithName:@"dev.flutter.pigeon.EngineApi.enableLocalVideo"
        binaryMessenger:binaryMessenger
                  codec:NEFLTEngineApiGetCodec()];
    if (api) {
      NSCAssert(
          [api respondsToSelector:@selector(enableLocalVideoRequest:error:)],
          @"NEFLTEngineApi api (%@) doesn't respond to @selector(enableLocalVideoRequest:error:)",
          api);
      [channel setMessageHandler:^(id _Nullable message, FlutterReply callback) {
        NSArray *args = message;
        NEFLTEnableLocalVideoRequest *arg_request = GetNullableObjectAtIndex(args, 0);
        FlutterError *error;
        NSNumber *output = [api enableLocalVideoRequest:arg_request error:&error];
        callback(wrapResult(output, error));
      }];
    } else {
      [channel setMessageHandler:nil];
    }
  }
  {
    FlutterBasicMessageChannel *channel = [[FlutterBasicMessageChannel alloc]
           initWithName:@"dev.flutter.pigeon.EngineApi.enableLocalSubStreamAudio"
        binaryMessenger:binaryMessenger
                  codec:NEFLTEngineApiGetCodec()];
    if (api) {
      NSCAssert([api respondsToSelector:@selector(enableLocalSubStreamAudioEnable:error:)],
                @"NEFLTEngineApi api (%@) doesn't respond to "
                @"@selector(enableLocalSubStreamAudioEnable:error:)",
                api);
      [channel setMessageHandler:^(id _Nullable message, FlutterReply callback) {
        NSArray *args = message;
        NSNumber *arg_enable = GetNullableObjectAtIndex(args, 0);
        FlutterError *error;
        NSNumber *output = [api enableLocalSubStreamAudioEnable:arg_enable error:&error];
        callback(wrapResult(output, error));
      }];
    } else {
      [channel setMessageHandler:nil];
    }
  }
  {
    FlutterBasicMessageChannel *channel = [[FlutterBasicMessageChannel alloc]
           initWithName:@"dev.flutter.pigeon.EngineApi.subscribeRemoteSubStreamAudio"
        binaryMessenger:binaryMessenger
                  codec:NEFLTEngineApiGetCodec()];
    if (api) {
      NSCAssert([api respondsToSelector:@selector(subscribeRemoteSubStreamAudioRequest:error:)],
                @"NEFLTEngineApi api (%@) doesn't respond to "
                @"@selector(subscribeRemoteSubStreamAudioRequest:error:)",
                api);
      [channel setMessageHandler:^(id _Nullable message, FlutterReply callback) {
        NSArray *args = message;
        NEFLTSubscribeRemoteSubStreamAudioRequest *arg_request = GetNullableObjectAtIndex(args, 0);
        FlutterError *error;
        NSNumber *output = [api subscribeRemoteSubStreamAudioRequest:arg_request error:&error];
        callback(wrapResult(output, error));
      }];
    } else {
      [channel setMessageHandler:nil];
    }
  }
  {
    FlutterBasicMessageChannel *channel = [[FlutterBasicMessageChannel alloc]
           initWithName:@"dev.flutter.pigeon.EngineApi.muteLocalSubStreamAudio"
        binaryMessenger:binaryMessenger
                  codec:NEFLTEngineApiGetCodec()];
    if (api) {
      NSCAssert([api respondsToSelector:@selector(muteLocalSubStreamAudioMuted:error:)],
                @"NEFLTEngineApi api (%@) doesn't respond to "
                @"@selector(muteLocalSubStreamAudioMuted:error:)",
                api);
      [channel setMessageHandler:^(id _Nullable message, FlutterReply callback) {
        NSArray *args = message;
        NSNumber *arg_muted = GetNullableObjectAtIndex(args, 0);
        FlutterError *error;
        NSNumber *output = [api muteLocalSubStreamAudioMuted:arg_muted error:&error];
        callback(wrapResult(output, error));
      }];
    } else {
      [channel setMessageHandler:nil];
    }
  }
  {
    FlutterBasicMessageChannel *channel = [[FlutterBasicMessageChannel alloc]
           initWithName:@"dev.flutter.pigeon.EngineApi.setAudioSubscribeOnlyBy"
        binaryMessenger:binaryMessenger
                  codec:NEFLTEngineApiGetCodec()];
    if (api) {
      NSCAssert([api respondsToSelector:@selector(setAudioSubscribeOnlyByRequest:error:)],
                @"NEFLTEngineApi api (%@) doesn't respond to "
                @"@selector(setAudioSubscribeOnlyByRequest:error:)",
                api);
      [channel setMessageHandler:^(id _Nullable message, FlutterReply callback) {
        NSArray *args = message;
        NEFLTSetAudioSubscribeOnlyByRequest *arg_request = GetNullableObjectAtIndex(args, 0);
        FlutterError *error;
        NSNumber *output = [api setAudioSubscribeOnlyByRequest:arg_request error:&error];
        callback(wrapResult(output, error));
      }];
    } else {
      [channel setMessageHandler:nil];
    }
  }
  {
    FlutterBasicMessageChannel *channel = [[FlutterBasicMessageChannel alloc]
           initWithName:@"dev.flutter.pigeon.EngineApi.startScreenCapture"
        binaryMessenger:binaryMessenger
                  codec:NEFLTEngineApiGetCodec()];
    if (api) {
      NSCAssert([api respondsToSelector:@selector(startScreenCaptureRequest:completion:)],
                @"NEFLTEngineApi api (%@) doesn't respond to "
                @"@selector(startScreenCaptureRequest:completion:)",
                api);
      [channel setMessageHandler:^(id _Nullable message, FlutterReply callback) {
        NSArray *args = message;
        NEFLTStartScreenCaptureRequest *arg_request = GetNullableObjectAtIndex(args, 0);
        [api
            startScreenCaptureRequest:arg_request
                           completion:^(NSNumber *_Nullable output, FlutterError *_Nullable error) {
                             callback(wrapResult(output, error));
                           }];
      }];
    } else {
      [channel setMessageHandler:nil];
    }
  }
  {
    FlutterBasicMessageChannel *channel = [[FlutterBasicMessageChannel alloc]
           initWithName:@"dev.flutter.pigeon.EngineApi.stopScreenCapture"
        binaryMessenger:binaryMessenger
                  codec:NEFLTEngineApiGetCodec()];
    if (api) {
      NSCAssert(
          [api respondsToSelector:@selector(stopScreenCaptureWithError:)],
          @"NEFLTEngineApi api (%@) doesn't respond to @selector(stopScreenCaptureWithError:)",
          api);
      [channel setMessageHandler:^(id _Nullable message, FlutterReply callback) {
        FlutterError *error;
        NSNumber *output = [api stopScreenCaptureWithError:&error];
        callback(wrapResult(output, error));
      }];
    } else {
      [channel setMessageHandler:nil];
    }
  }
  {
    FlutterBasicMessageChannel *channel = [[FlutterBasicMessageChannel alloc]
           initWithName:@"dev.flutter.pigeon.EngineApi.enableLoopbackRecording"
        binaryMessenger:binaryMessenger
                  codec:NEFLTEngineApiGetCodec()];
    if (api) {
      NSCAssert([api respondsToSelector:@selector(enableLoopbackRecordingEnable:error:)],
                @"NEFLTEngineApi api (%@) doesn't respond to "
                @"@selector(enableLoopbackRecordingEnable:error:)",
                api);
      [channel setMessageHandler:^(id _Nullable message, FlutterReply callback) {
        NSArray *args = message;
        NSNumber *arg_enable = GetNullableObjectAtIndex(args, 0);
        FlutterError *error;
        NSNumber *output = [api enableLoopbackRecordingEnable:arg_enable error:&error];
        callback(wrapResult(output, error));
      }];
    } else {
      [channel setMessageHandler:nil];
    }
  }
  {
    FlutterBasicMessageChannel *channel = [[FlutterBasicMessageChannel alloc]
           initWithName:@"dev.flutter.pigeon.EngineApi.subscribeRemoteVideoStream"
        binaryMessenger:binaryMessenger
                  codec:NEFLTEngineApiGetCodec()];
    if (api) {
      NSCAssert([api respondsToSelector:@selector(subscribeRemoteVideoStreamRequest:error:)],
                @"NEFLTEngineApi api (%@) doesn't respond to "
                @"@selector(subscribeRemoteVideoStreamRequest:error:)",
                api);
      [channel setMessageHandler:^(id _Nullable message, FlutterReply callback) {
        NSArray *args = message;
        NEFLTSubscribeRemoteVideoStreamRequest *arg_request = GetNullableObjectAtIndex(args, 0);
        FlutterError *error;
        NSNumber *output = [api subscribeRemoteVideoStreamRequest:arg_request error:&error];
        callback(wrapResult(output, error));
      }];
    } else {
      [channel setMessageHandler:nil];
    }
  }
  {
    FlutterBasicMessageChannel *channel = [[FlutterBasicMessageChannel alloc]
           initWithName:@"dev.flutter.pigeon.EngineApi.subscribeRemoteSubStreamVideo"
        binaryMessenger:binaryMessenger
                  codec:NEFLTEngineApiGetCodec()];
    if (api) {
      NSCAssert([api respondsToSelector:@selector(subscribeRemoteSubStreamVideoRequest:error:)],
                @"NEFLTEngineApi api (%@) doesn't respond to "
                @"@selector(subscribeRemoteSubStreamVideoRequest:error:)",
                api);
      [channel setMessageHandler:^(id _Nullable message, FlutterReply callback) {
        NSArray *args = message;
        NEFLTSubscribeRemoteSubStreamVideoRequest *arg_request = GetNullableObjectAtIndex(args, 0);
        FlutterError *error;
        NSNumber *output = [api subscribeRemoteSubStreamVideoRequest:arg_request error:&error];
        callback(wrapResult(output, error));
      }];
    } else {
      [channel setMessageHandler:nil];
    }
  }
  {
    FlutterBasicMessageChannel *channel = [[FlutterBasicMessageChannel alloc]
           initWithName:@"dev.flutter.pigeon.EngineApi.muteLocalAudioStream"
        binaryMessenger:binaryMessenger
                  codec:NEFLTEngineApiGetCodec()];
    if (api) {
      NSCAssert(
          [api respondsToSelector:@selector(muteLocalAudioStreamMute:error:)],
          @"NEFLTEngineApi api (%@) doesn't respond to @selector(muteLocalAudioStreamMute:error:)",
          api);
      [channel setMessageHandler:^(id _Nullable message, FlutterReply callback) {
        NSArray *args = message;
        NSNumber *arg_mute = GetNullableObjectAtIndex(args, 0);
        FlutterError *error;
        NSNumber *output = [api muteLocalAudioStreamMute:arg_mute error:&error];
        callback(wrapResult(output, error));
      }];
    } else {
      [channel setMessageHandler:nil];
    }
  }
  {
    FlutterBasicMessageChannel *channel = [[FlutterBasicMessageChannel alloc]
           initWithName:@"dev.flutter.pigeon.EngineApi.muteLocalVideoStream"
        binaryMessenger:binaryMessenger
                  codec:NEFLTEngineApiGetCodec()];
    if (api) {
      NSCAssert([api respondsToSelector:@selector(muteLocalVideoStreamMute:streamType:error:)],
                @"NEFLTEngineApi api (%@) doesn't respond to "
                @"@selector(muteLocalVideoStreamMute:streamType:error:)",
                api);
      [channel setMessageHandler:^(id _Nullable message, FlutterReply callback) {
        NSArray *args = message;
        NSNumber *arg_mute = GetNullableObjectAtIndex(args, 0);
        NSNumber *arg_streamType = GetNullableObjectAtIndex(args, 1);
        FlutterError *error;
        NSNumber *output = [api muteLocalVideoStreamMute:arg_mute
                                              streamType:arg_streamType
                                                   error:&error];
        callback(wrapResult(output, error));
      }];
    } else {
      [channel setMessageHandler:nil];
    }
  }
  {
    FlutterBasicMessageChannel *channel = [[FlutterBasicMessageChannel alloc]
           initWithName:@"dev.flutter.pigeon.EngineApi.startAudioDump"
        binaryMessenger:binaryMessenger
                  codec:NEFLTEngineApiGetCodec()];
    if (api) {
      NSCAssert([api respondsToSelector:@selector(startAudioDumpWithError:)],
                @"NEFLTEngineApi api (%@) doesn't respond to @selector(startAudioDumpWithError:)",
                api);
      [channel setMessageHandler:^(id _Nullable message, FlutterReply callback) {
        FlutterError *error;
        NSNumber *output = [api startAudioDumpWithError:&error];
        callback(wrapResult(output, error));
      }];
    } else {
      [channel setMessageHandler:nil];
    }
  }
  {
    FlutterBasicMessageChannel *channel = [[FlutterBasicMessageChannel alloc]
           initWithName:@"dev.flutter.pigeon.EngineApi.startAudioDumpWithType"
        binaryMessenger:binaryMessenger
                  codec:NEFLTEngineApiGetCodec()];
    if (api) {
      NSCAssert([api respondsToSelector:@selector(startAudioDumpWithTypeDumpType:error:)],
                @"NEFLTEngineApi api (%@) doesn't respond to "
                @"@selector(startAudioDumpWithTypeDumpType:error:)",
                api);
      [channel setMessageHandler:^(id _Nullable message, FlutterReply callback) {
        NSArray *args = message;
        NSNumber *arg_dumpType = GetNullableObjectAtIndex(args, 0);
        FlutterError *error;
        NSNumber *output = [api startAudioDumpWithTypeDumpType:arg_dumpType error:&error];
        callback(wrapResult(output, error));
      }];
    } else {
      [channel setMessageHandler:nil];
    }
  }
  {
    FlutterBasicMessageChannel *channel = [[FlutterBasicMessageChannel alloc]
           initWithName:@"dev.flutter.pigeon.EngineApi.stopAudioDump"
        binaryMessenger:binaryMessenger
                  codec:NEFLTEngineApiGetCodec()];
    if (api) {
      NSCAssert([api respondsToSelector:@selector(stopAudioDumpWithError:)],
                @"NEFLTEngineApi api (%@) doesn't respond to @selector(stopAudioDumpWithError:)",
                api);
      [channel setMessageHandler:^(id _Nullable message, FlutterReply callback) {
        FlutterError *error;
        NSNumber *output = [api stopAudioDumpWithError:&error];
        callback(wrapResult(output, error));
      }];
    } else {
      [channel setMessageHandler:nil];
    }
  }
  {
    FlutterBasicMessageChannel *channel = [[FlutterBasicMessageChannel alloc]
           initWithName:@"dev.flutter.pigeon.EngineApi.enableAudioVolumeIndication"
        binaryMessenger:binaryMessenger
                  codec:NEFLTEngineApiGetCodec()];
    if (api) {
      NSCAssert([api respondsToSelector:@selector(enableAudioVolumeIndicationRequest:error:)],
                @"NEFLTEngineApi api (%@) doesn't respond to "
                @"@selector(enableAudioVolumeIndicationRequest:error:)",
                api);
      [channel setMessageHandler:^(id _Nullable message, FlutterReply callback) {
        NSArray *args = message;
        NEFLTEnableAudioVolumeIndicationRequest *arg_request = GetNullableObjectAtIndex(args, 0);
        FlutterError *error;
        NSNumber *output = [api enableAudioVolumeIndicationRequest:arg_request error:&error];
        callback(wrapResult(output, error));
      }];
    } else {
      [channel setMessageHandler:nil];
    }
  }
  {
    FlutterBasicMessageChannel *channel = [[FlutterBasicMessageChannel alloc]
           initWithName:@"dev.flutter.pigeon.EngineApi.adjustRecordingSignalVolume"
        binaryMessenger:binaryMessenger
                  codec:NEFLTEngineApiGetCodec()];
    if (api) {
      NSCAssert([api respondsToSelector:@selector(adjustRecordingSignalVolumeVolume:error:)],
                @"NEFLTEngineApi api (%@) doesn't respond to "
                @"@selector(adjustRecordingSignalVolumeVolume:error:)",
                api);
      [channel setMessageHandler:^(id _Nullable message, FlutterReply callback) {
        NSArray *args = message;
        NSNumber *arg_volume = GetNullableObjectAtIndex(args, 0);
        FlutterError *error;
        NSNumber *output = [api adjustRecordingSignalVolumeVolume:arg_volume error:&error];
        callback(wrapResult(output, error));
      }];
    } else {
      [channel setMessageHandler:nil];
    }
  }
  {
    FlutterBasicMessageChannel *channel = [[FlutterBasicMessageChannel alloc]
           initWithName:@"dev.flutter.pigeon.EngineApi.adjustPlaybackSignalVolume"
        binaryMessenger:binaryMessenger
                  codec:NEFLTEngineApiGetCodec()];
    if (api) {
      NSCAssert([api respondsToSelector:@selector(adjustPlaybackSignalVolumeVolume:error:)],
                @"NEFLTEngineApi api (%@) doesn't respond to "
                @"@selector(adjustPlaybackSignalVolumeVolume:error:)",
                api);
      [channel setMessageHandler:^(id _Nullable message, FlutterReply callback) {
        NSArray *args = message;
        NSNumber *arg_volume = GetNullableObjectAtIndex(args, 0);
        FlutterError *error;
        NSNumber *output = [api adjustPlaybackSignalVolumeVolume:arg_volume error:&error];
        callback(wrapResult(output, error));
      }];
    } else {
      [channel setMessageHandler:nil];
    }
  }
  {
    FlutterBasicMessageChannel *channel = [[FlutterBasicMessageChannel alloc]
           initWithName:@"dev.flutter.pigeon.EngineApi.adjustLoopBackRecordingSignalVolume"
        binaryMessenger:binaryMessenger
                  codec:NEFLTEngineApiGetCodec()];
    if (api) {
      NSCAssert([api respondsToSelector:@selector(adjustLoopBackRecordingSignalVolumeVolume:
                                                                                      error:)],
                @"NEFLTEngineApi api (%@) doesn't respond to "
                @"@selector(adjustLoopBackRecordingSignalVolumeVolume:error:)",
                api);
      [channel setMessageHandler:^(id _Nullable message, FlutterReply callback) {
        NSArray *args = message;
        NSNumber *arg_volume = GetNullableObjectAtIndex(args, 0);
        FlutterError *error;
        NSNumber *output = [api adjustLoopBackRecordingSignalVolumeVolume:arg_volume error:&error];
        callback(wrapResult(output, error));
      }];
    } else {
      [channel setMessageHandler:nil];
    }
  }
  {
    FlutterBasicMessageChannel *channel = [[FlutterBasicMessageChannel alloc]
           initWithName:@"dev.flutter.pigeon.EngineApi.addLiveStreamTask"
        binaryMessenger:binaryMessenger
                  codec:NEFLTEngineApiGetCodec()];
    if (api) {
      NSCAssert(
          [api respondsToSelector:@selector(addLiveStreamTaskRequest:error:)],
          @"NEFLTEngineApi api (%@) doesn't respond to @selector(addLiveStreamTaskRequest:error:)",
          api);
      [channel setMessageHandler:^(id _Nullable message, FlutterReply callback) {
        NSArray *args = message;
        NEFLTAddOrUpdateLiveStreamTaskRequest *arg_request = GetNullableObjectAtIndex(args, 0);
        FlutterError *error;
        NSNumber *output = [api addLiveStreamTaskRequest:arg_request error:&error];
        callback(wrapResult(output, error));
      }];
    } else {
      [channel setMessageHandler:nil];
    }
  }
  {
    FlutterBasicMessageChannel *channel = [[FlutterBasicMessageChannel alloc]
           initWithName:@"dev.flutter.pigeon.EngineApi.updateLiveStreamTask"
        binaryMessenger:binaryMessenger
                  codec:NEFLTEngineApiGetCodec()];
    if (api) {
      NSCAssert([api respondsToSelector:@selector(updateLiveStreamTaskRequest:error:)],
                @"NEFLTEngineApi api (%@) doesn't respond to "
                @"@selector(updateLiveStreamTaskRequest:error:)",
                api);
      [channel setMessageHandler:^(id _Nullable message, FlutterReply callback) {
        NSArray *args = message;
        NEFLTAddOrUpdateLiveStreamTaskRequest *arg_request = GetNullableObjectAtIndex(args, 0);
        FlutterError *error;
        NSNumber *output = [api updateLiveStreamTaskRequest:arg_request error:&error];
        callback(wrapResult(output, error));
      }];
    } else {
      [channel setMessageHandler:nil];
    }
  }
  {
    FlutterBasicMessageChannel *channel = [[FlutterBasicMessageChannel alloc]
           initWithName:@"dev.flutter.pigeon.EngineApi.removeLiveStreamTask"
        binaryMessenger:binaryMessenger
                  codec:NEFLTEngineApiGetCodec()];
    if (api) {
      NSCAssert([api respondsToSelector:@selector(removeLiveStreamTaskRequest:error:)],
                @"NEFLTEngineApi api (%@) doesn't respond to "
                @"@selector(removeLiveStreamTaskRequest:error:)",
                api);
      [channel setMessageHandler:^(id _Nullable message, FlutterReply callback) {
        NSArray *args = message;
        NEFLTDeleteLiveStreamTaskRequest *arg_request = GetNullableObjectAtIndex(args, 0);
        FlutterError *error;
        NSNumber *output = [api removeLiveStreamTaskRequest:arg_request error:&error];
        callback(wrapResult(output, error));
      }];
    } else {
      [channel setMessageHandler:nil];
    }
  }
  {
    FlutterBasicMessageChannel *channel = [[FlutterBasicMessageChannel alloc]
           initWithName:@"dev.flutter.pigeon.EngineApi.setClientRole"
        binaryMessenger:binaryMessenger
                  codec:NEFLTEngineApiGetCodec()];
    if (api) {
      NSCAssert([api respondsToSelector:@selector(setClientRoleRole:error:)],
                @"NEFLTEngineApi api (%@) doesn't respond to @selector(setClientRoleRole:error:)",
                api);
      [channel setMessageHandler:^(id _Nullable message, FlutterReply callback) {
        NSArray *args = message;
        NSNumber *arg_role = GetNullableObjectAtIndex(args, 0);
        FlutterError *error;
        NSNumber *output = [api setClientRoleRole:arg_role error:&error];
        callback(wrapResult(output, error));
      }];
    } else {
      [channel setMessageHandler:nil];
    }
  }
  {
    FlutterBasicMessageChannel *channel = [[FlutterBasicMessageChannel alloc]
           initWithName:@"dev.flutter.pigeon.EngineApi.getConnectionState"
        binaryMessenger:binaryMessenger
                  codec:NEFLTEngineApiGetCodec()];
    if (api) {
      NSCAssert(
          [api respondsToSelector:@selector(getConnectionStateWithError:)],
          @"NEFLTEngineApi api (%@) doesn't respond to @selector(getConnectionStateWithError:)",
          api);
      [channel setMessageHandler:^(id _Nullable message, FlutterReply callback) {
        FlutterError *error;
        NSNumber *output = [api getConnectionStateWithError:&error];
        callback(wrapResult(output, error));
      }];
    } else {
      [channel setMessageHandler:nil];
    }
  }
  {
    FlutterBasicMessageChannel *channel = [[FlutterBasicMessageChannel alloc]
           initWithName:@"dev.flutter.pigeon.EngineApi.uploadSdkInfo"
        binaryMessenger:binaryMessenger
                  codec:NEFLTEngineApiGetCodec()];
    if (api) {
      NSCAssert([api respondsToSelector:@selector(uploadSdkInfoWithError:)],
                @"NEFLTEngineApi api (%@) doesn't respond to @selector(uploadSdkInfoWithError:)",
                api);
      [channel setMessageHandler:^(id _Nullable message, FlutterReply callback) {
        FlutterError *error;
        NSNumber *output = [api uploadSdkInfoWithError:&error];
        callback(wrapResult(output, error));
      }];
    } else {
      [channel setMessageHandler:nil];
    }
  }
  {
    FlutterBasicMessageChannel *channel =
        [[FlutterBasicMessageChannel alloc] initWithName:@"dev.flutter.pigeon.EngineApi.sendSEIMsg"
                                         binaryMessenger:binaryMessenger
                                                   codec:NEFLTEngineApiGetCodec()];
    if (api) {
      NSCAssert([api respondsToSelector:@selector(sendSEIMsgRequest:error:)],
                @"NEFLTEngineApi api (%@) doesn't respond to @selector(sendSEIMsgRequest:error:)",
                api);
      [channel setMessageHandler:^(id _Nullable message, FlutterReply callback) {
        NSArray *args = message;
        NEFLTSendSEIMsgRequest *arg_request = GetNullableObjectAtIndex(args, 0);
        FlutterError *error;
        NSNumber *output = [api sendSEIMsgRequest:arg_request error:&error];
        callback(wrapResult(output, error));
      }];
    } else {
      [channel setMessageHandler:nil];
    }
  }
  {
    FlutterBasicMessageChannel *channel = [[FlutterBasicMessageChannel alloc]
           initWithName:@"dev.flutter.pigeon.EngineApi.setLocalVoiceReverbParam"
        binaryMessenger:binaryMessenger
                  codec:NEFLTEngineApiGetCodec()];
    if (api) {
      NSCAssert([api respondsToSelector:@selector(setLocalVoiceReverbParamRequest:error:)],
                @"NEFLTEngineApi api (%@) doesn't respond to "
                @"@selector(setLocalVoiceReverbParamRequest:error:)",
                api);
      [channel setMessageHandler:^(id _Nullable message, FlutterReply callback) {
        NSArray *args = message;
        NEFLTSetLocalVoiceReverbParamRequest *arg_request = GetNullableObjectAtIndex(args, 0);
        FlutterError *error;
        NSNumber *output = [api setLocalVoiceReverbParamRequest:arg_request error:&error];
        callback(wrapResult(output, error));
      }];
    } else {
      [channel setMessageHandler:nil];
    }
  }
  {
    FlutterBasicMessageChannel *channel = [[FlutterBasicMessageChannel alloc]
           initWithName:@"dev.flutter.pigeon.EngineApi.setAudioEffectPreset"
        binaryMessenger:binaryMessenger
                  codec:NEFLTEngineApiGetCodec()];
    if (api) {
      NSCAssert([api respondsToSelector:@selector(setAudioEffectPresetPreset:error:)],
                @"NEFLTEngineApi api (%@) doesn't respond to "
                @"@selector(setAudioEffectPresetPreset:error:)",
                api);
      [channel setMessageHandler:^(id _Nullable message, FlutterReply callback) {
        NSArray *args = message;
        NSNumber *arg_preset = GetNullableObjectAtIndex(args, 0);
        FlutterError *error;
        NSNumber *output = [api setAudioEffectPresetPreset:arg_preset error:&error];
        callback(wrapResult(output, error));
      }];
    } else {
      [channel setMessageHandler:nil];
    }
  }
  {
    FlutterBasicMessageChannel *channel = [[FlutterBasicMessageChannel alloc]
           initWithName:@"dev.flutter.pigeon.EngineApi.setVoiceBeautifierPreset"
        binaryMessenger:binaryMessenger
                  codec:NEFLTEngineApiGetCodec()];
    if (api) {
      NSCAssert([api respondsToSelector:@selector(setVoiceBeautifierPresetPreset:error:)],
                @"NEFLTEngineApi api (%@) doesn't respond to "
                @"@selector(setVoiceBeautifierPresetPreset:error:)",
                api);
      [channel setMessageHandler:^(id _Nullable message, FlutterReply callback) {
        NSArray *args = message;
        NSNumber *arg_preset = GetNullableObjectAtIndex(args, 0);
        FlutterError *error;
        NSNumber *output = [api setVoiceBeautifierPresetPreset:arg_preset error:&error];
        callback(wrapResult(output, error));
      }];
    } else {
      [channel setMessageHandler:nil];
    }
  }
  {
    FlutterBasicMessageChannel *channel = [[FlutterBasicMessageChannel alloc]
           initWithName:@"dev.flutter.pigeon.EngineApi.setLocalVoicePitch"
        binaryMessenger:binaryMessenger
                  codec:NEFLTEngineApiGetCodec()];
    if (api) {
      NSCAssert(
          [api respondsToSelector:@selector(setLocalVoicePitchPitch:error:)],
          @"NEFLTEngineApi api (%@) doesn't respond to @selector(setLocalVoicePitchPitch:error:)",
          api);
      [channel setMessageHandler:^(id _Nullable message, FlutterReply callback) {
        NSArray *args = message;
        NSNumber *arg_pitch = GetNullableObjectAtIndex(args, 0);
        FlutterError *error;
        NSNumber *output = [api setLocalVoicePitchPitch:arg_pitch error:&error];
        callback(wrapResult(output, error));
      }];
    } else {
      [channel setMessageHandler:nil];
    }
  }
  {
    FlutterBasicMessageChannel *channel = [[FlutterBasicMessageChannel alloc]
           initWithName:@"dev.flutter.pigeon.EngineApi.setLocalVoiceEqualization"
        binaryMessenger:binaryMessenger
                  codec:NEFLTEngineApiGetCodec()];
    if (api) {
      NSCAssert([api respondsToSelector:@selector(setLocalVoiceEqualizationRequest:error:)],
                @"NEFLTEngineApi api (%@) doesn't respond to "
                @"@selector(setLocalVoiceEqualizationRequest:error:)",
                api);
      [channel setMessageHandler:^(id _Nullable message, FlutterReply callback) {
        NSArray *args = message;
        NEFLTSetLocalVoiceEqualizationRequest *arg_request = GetNullableObjectAtIndex(args, 0);
        FlutterError *error;
        NSNumber *output = [api setLocalVoiceEqualizationRequest:arg_request error:&error];
        callback(wrapResult(output, error));
      }];
    } else {
      [channel setMessageHandler:nil];
    }
  }
  {
    FlutterBasicMessageChannel *channel = [[FlutterBasicMessageChannel alloc]
           initWithName:@"dev.flutter.pigeon.EngineApi.switchChannel"
        binaryMessenger:binaryMessenger
                  codec:NEFLTEngineApiGetCodec()];
    if (api) {
      NSCAssert(
          [api respondsToSelector:@selector(switchChannelRequest:error:)],
          @"NEFLTEngineApi api (%@) doesn't respond to @selector(switchChannelRequest:error:)",
          api);
      [channel setMessageHandler:^(id _Nullable message, FlutterReply callback) {
        NSArray *args = message;
        NEFLTSwitchChannelRequest *arg_request = GetNullableObjectAtIndex(args, 0);
        FlutterError *error;
        NSNumber *output = [api switchChannelRequest:arg_request error:&error];
        callback(wrapResult(output, error));
      }];
    } else {
      [channel setMessageHandler:nil];
    }
  }
  {
    FlutterBasicMessageChannel *channel = [[FlutterBasicMessageChannel alloc]
           initWithName:@"dev.flutter.pigeon.EngineApi.startAudioRecording"
        binaryMessenger:binaryMessenger
                  codec:NEFLTEngineApiGetCodec()];
    if (api) {
      NSCAssert([api respondsToSelector:@selector(startAudioRecordingRequest:error:)],
                @"NEFLTEngineApi api (%@) doesn't respond to "
                @"@selector(startAudioRecordingRequest:error:)",
                api);
      [channel setMessageHandler:^(id _Nullable message, FlutterReply callback) {
        NSArray *args = message;
        NEFLTStartAudioRecordingRequest *arg_request = GetNullableObjectAtIndex(args, 0);
        FlutterError *error;
        NSNumber *output = [api startAudioRecordingRequest:arg_request error:&error];
        callback(wrapResult(output, error));
      }];
    } else {
      [channel setMessageHandler:nil];
    }
  }
  {
    FlutterBasicMessageChannel *channel = [[FlutterBasicMessageChannel alloc]
           initWithName:@"dev.flutter.pigeon.EngineApi.startAudioRecordingWithConfig"
        binaryMessenger:binaryMessenger
                  codec:NEFLTEngineApiGetCodec()];
    if (api) {
      NSCAssert([api respondsToSelector:@selector(startAudioRecordingWithConfigRequest:error:)],
                @"NEFLTEngineApi api (%@) doesn't respond to "
                @"@selector(startAudioRecordingWithConfigRequest:error:)",
                api);
      [channel setMessageHandler:^(id _Nullable message, FlutterReply callback) {
        NSArray *args = message;
        NEFLTAudioRecordingConfigurationRequest *arg_request = GetNullableObjectAtIndex(args, 0);
        FlutterError *error;
        NSNumber *output = [api startAudioRecordingWithConfigRequest:arg_request error:&error];
        callback(wrapResult(output, error));
      }];
    } else {
      [channel setMessageHandler:nil];
    }
  }
  {
    FlutterBasicMessageChannel *channel = [[FlutterBasicMessageChannel alloc]
           initWithName:@"dev.flutter.pigeon.EngineApi.stopAudioRecording"
        binaryMessenger:binaryMessenger
                  codec:NEFLTEngineApiGetCodec()];
    if (api) {
      NSCAssert(
          [api respondsToSelector:@selector(stopAudioRecordingWithError:)],
          @"NEFLTEngineApi api (%@) doesn't respond to @selector(stopAudioRecordingWithError:)",
          api);
      [channel setMessageHandler:^(id _Nullable message, FlutterReply callback) {
        FlutterError *error;
        NSNumber *output = [api stopAudioRecordingWithError:&error];
        callback(wrapResult(output, error));
      }];
    } else {
      [channel setMessageHandler:nil];
    }
  }
  {
    FlutterBasicMessageChannel *channel = [[FlutterBasicMessageChannel alloc]
           initWithName:@"dev.flutter.pigeon.EngineApi.setLocalMediaPriority"
        binaryMessenger:binaryMessenger
                  codec:NEFLTEngineApiGetCodec()];
    if (api) {
      NSCAssert([api respondsToSelector:@selector(setLocalMediaPriorityRequest:error:)],
                @"NEFLTEngineApi api (%@) doesn't respond to "
                @"@selector(setLocalMediaPriorityRequest:error:)",
                api);
      [channel setMessageHandler:^(id _Nullable message, FlutterReply callback) {
        NSArray *args = message;
        NEFLTSetLocalMediaPriorityRequest *arg_request = GetNullableObjectAtIndex(args, 0);
        FlutterError *error;
        NSNumber *output = [api setLocalMediaPriorityRequest:arg_request error:&error];
        callback(wrapResult(output, error));
      }];
    } else {
      [channel setMessageHandler:nil];
    }
  }
  {
    FlutterBasicMessageChannel *channel = [[FlutterBasicMessageChannel alloc]
           initWithName:@"dev.flutter.pigeon.EngineApi.enableMediaPub"
        binaryMessenger:binaryMessenger
                  codec:NEFLTEngineApiGetCodec()];
    if (api) {
      NSCAssert([api respondsToSelector:@selector(enableMediaPubMediaType:enable:error:)],
                @"NEFLTEngineApi api (%@) doesn't respond to "
                @"@selector(enableMediaPubMediaType:enable:error:)",
                api);
      [channel setMessageHandler:^(id _Nullable message, FlutterReply callback) {
        NSArray *args = message;
        NSNumber *arg_mediaType = GetNullableObjectAtIndex(args, 0);
        NSNumber *arg_enable = GetNullableObjectAtIndex(args, 1);
        FlutterError *error;
        NSNumber *output = [api enableMediaPubMediaType:arg_mediaType
                                                 enable:arg_enable
                                                  error:&error];
        callback(wrapResult(output, error));
      }];
    } else {
      [channel setMessageHandler:nil];
    }
  }
  {
    FlutterBasicMessageChannel *channel = [[FlutterBasicMessageChannel alloc]
           initWithName:@"dev.flutter.pigeon.EngineApi.startChannelMediaRelay"
        binaryMessenger:binaryMessenger
                  codec:NEFLTEngineApiGetCodec()];
    if (api) {
      NSCAssert([api respondsToSelector:@selector(startChannelMediaRelayRequest:error:)],
                @"NEFLTEngineApi api (%@) doesn't respond to "
                @"@selector(startChannelMediaRelayRequest:error:)",
                api);
      [channel setMessageHandler:^(id _Nullable message, FlutterReply callback) {
        NSArray *args = message;
        NEFLTStartOrUpdateChannelMediaRelayRequest *arg_request = GetNullableObjectAtIndex(args, 0);
        FlutterError *error;
        NSNumber *output = [api startChannelMediaRelayRequest:arg_request error:&error];
        callback(wrapResult(output, error));
      }];
    } else {
      [channel setMessageHandler:nil];
    }
  }
  {
    FlutterBasicMessageChannel *channel = [[FlutterBasicMessageChannel alloc]
           initWithName:@"dev.flutter.pigeon.EngineApi.updateChannelMediaRelay"
        binaryMessenger:binaryMessenger
                  codec:NEFLTEngineApiGetCodec()];
    if (api) {
      NSCAssert([api respondsToSelector:@selector(updateChannelMediaRelayRequest:error:)],
                @"NEFLTEngineApi api (%@) doesn't respond to "
                @"@selector(updateChannelMediaRelayRequest:error:)",
                api);
      [channel setMessageHandler:^(id _Nullable message, FlutterReply callback) {
        NSArray *args = message;
        NEFLTStartOrUpdateChannelMediaRelayRequest *arg_request = GetNullableObjectAtIndex(args, 0);
        FlutterError *error;
        NSNumber *output = [api updateChannelMediaRelayRequest:arg_request error:&error];
        callback(wrapResult(output, error));
      }];
    } else {
      [channel setMessageHandler:nil];
    }
  }
  {
    FlutterBasicMessageChannel *channel = [[FlutterBasicMessageChannel alloc]
           initWithName:@"dev.flutter.pigeon.EngineApi.stopChannelMediaRelay"
        binaryMessenger:binaryMessenger
                  codec:NEFLTEngineApiGetCodec()];
    if (api) {
      NSCAssert(
          [api respondsToSelector:@selector(stopChannelMediaRelayWithError:)],
          @"NEFLTEngineApi api (%@) doesn't respond to @selector(stopChannelMediaRelayWithError:)",
          api);
      [channel setMessageHandler:^(id _Nullable message, FlutterReply callback) {
        FlutterError *error;
        NSNumber *output = [api stopChannelMediaRelayWithError:&error];
        callback(wrapResult(output, error));
      }];
    } else {
      [channel setMessageHandler:nil];
    }
  }
  {
    FlutterBasicMessageChannel *channel = [[FlutterBasicMessageChannel alloc]
           initWithName:@"dev.flutter.pigeon.EngineApi.adjustUserPlaybackSignalVolume"
        binaryMessenger:binaryMessenger
                  codec:NEFLTEngineApiGetCodec()];
    if (api) {
      NSCAssert([api respondsToSelector:@selector(adjustUserPlaybackSignalVolumeRequest:error:)],
                @"NEFLTEngineApi api (%@) doesn't respond to "
                @"@selector(adjustUserPlaybackSignalVolumeRequest:error:)",
                api);
      [channel setMessageHandler:^(id _Nullable message, FlutterReply callback) {
        NSArray *args = message;
        NEFLTAdjustUserPlaybackSignalVolumeRequest *arg_request = GetNullableObjectAtIndex(args, 0);
        FlutterError *error;
        NSNumber *output = [api adjustUserPlaybackSignalVolumeRequest:arg_request error:&error];
        callback(wrapResult(output, error));
      }];
    } else {
      [channel setMessageHandler:nil];
    }
  }
  {
    FlutterBasicMessageChannel *channel = [[FlutterBasicMessageChannel alloc]
           initWithName:@"dev.flutter.pigeon.EngineApi.setLocalPublishFallbackOption"
        binaryMessenger:binaryMessenger
                  codec:NEFLTEngineApiGetCodec()];
    if (api) {
      NSCAssert([api respondsToSelector:@selector(setLocalPublishFallbackOptionOption:error:)],
                @"NEFLTEngineApi api (%@) doesn't respond to "
                @"@selector(setLocalPublishFallbackOptionOption:error:)",
                api);
      [channel setMessageHandler:^(id _Nullable message, FlutterReply callback) {
        NSArray *args = message;
        NSNumber *arg_option = GetNullableObjectAtIndex(args, 0);
        FlutterError *error;
        NSNumber *output = [api setLocalPublishFallbackOptionOption:arg_option error:&error];
        callback(wrapResult(output, error));
      }];
    } else {
      [channel setMessageHandler:nil];
    }
  }
  {
    FlutterBasicMessageChannel *channel = [[FlutterBasicMessageChannel alloc]
           initWithName:@"dev.flutter.pigeon.EngineApi.setRemoteSubscribeFallbackOption"
        binaryMessenger:binaryMessenger
                  codec:NEFLTEngineApiGetCodec()];
    if (api) {
      NSCAssert([api respondsToSelector:@selector(setRemoteSubscribeFallbackOptionOption:error:)],
                @"NEFLTEngineApi api (%@) doesn't respond to "
                @"@selector(setRemoteSubscribeFallbackOptionOption:error:)",
                api);
      [channel setMessageHandler:^(id _Nullable message, FlutterReply callback) {
        NSArray *args = message;
        NSNumber *arg_option = GetNullableObjectAtIndex(args, 0);
        FlutterError *error;
        NSNumber *output = [api setRemoteSubscribeFallbackOptionOption:arg_option error:&error];
        callback(wrapResult(output, error));
      }];
    } else {
      [channel setMessageHandler:nil];
    }
  }
  {
    FlutterBasicMessageChannel *channel = [[FlutterBasicMessageChannel alloc]
           initWithName:@"dev.flutter.pigeon.EngineApi.enableSuperResolution"
        binaryMessenger:binaryMessenger
                  codec:NEFLTEngineApiGetCodec()];
    if (api) {
      NSCAssert([api respondsToSelector:@selector(enableSuperResolutionEnable:error:)],
                @"NEFLTEngineApi api (%@) doesn't respond to "
                @"@selector(enableSuperResolutionEnable:error:)",
                api);
      [channel setMessageHandler:^(id _Nullable message, FlutterReply callback) {
        NSArray *args = message;
        NSNumber *arg_enable = GetNullableObjectAtIndex(args, 0);
        FlutterError *error;
        NSNumber *output = [api enableSuperResolutionEnable:arg_enable error:&error];
        callback(wrapResult(output, error));
      }];
    } else {
      [channel setMessageHandler:nil];
    }
  }
  {
    FlutterBasicMessageChannel *channel = [[FlutterBasicMessageChannel alloc]
           initWithName:@"dev.flutter.pigeon.EngineApi.enableEncryption"
        binaryMessenger:binaryMessenger
                  codec:NEFLTEngineApiGetCodec()];
    if (api) {
      NSCAssert(
          [api respondsToSelector:@selector(enableEncryptionRequest:error:)],
          @"NEFLTEngineApi api (%@) doesn't respond to @selector(enableEncryptionRequest:error:)",
          api);
      [channel setMessageHandler:^(id _Nullable message, FlutterReply callback) {
        NSArray *args = message;
        NEFLTEnableEncryptionRequest *arg_request = GetNullableObjectAtIndex(args, 0);
        FlutterError *error;
        NSNumber *output = [api enableEncryptionRequest:arg_request error:&error];
        callback(wrapResult(output, error));
      }];
    } else {
      [channel setMessageHandler:nil];
    }
  }
  {
    FlutterBasicMessageChannel *channel = [[FlutterBasicMessageChannel alloc]
           initWithName:@"dev.flutter.pigeon.EngineApi.setAudioSessionOperationRestriction"
        binaryMessenger:binaryMessenger
                  codec:NEFLTEngineApiGetCodec()];
    if (api) {
      NSCAssert([api respondsToSelector:@selector(setAudioSessionOperationRestrictionOption:
                                                                                      error:)],
                @"NEFLTEngineApi api (%@) doesn't respond to "
                @"@selector(setAudioSessionOperationRestrictionOption:error:)",
                api);
      [channel setMessageHandler:^(id _Nullable message, FlutterReply callback) {
        NSArray *args = message;
        NSNumber *arg_option = GetNullableObjectAtIndex(args, 0);
        FlutterError *error;
        NSNumber *output = [api setAudioSessionOperationRestrictionOption:arg_option error:&error];
        callback(wrapResult(output, error));
      }];
    } else {
      [channel setMessageHandler:nil];
    }
  }
  {
    FlutterBasicMessageChannel *channel = [[FlutterBasicMessageChannel alloc]
           initWithName:@"dev.flutter.pigeon.EngineApi.enableVideoCorrection"
        binaryMessenger:binaryMessenger
                  codec:NEFLTEngineApiGetCodec()];
    if (api) {
      NSCAssert([api respondsToSelector:@selector(enableVideoCorrectionEnable:error:)],
                @"NEFLTEngineApi api (%@) doesn't respond to "
                @"@selector(enableVideoCorrectionEnable:error:)",
                api);
      [channel setMessageHandler:^(id _Nullable message, FlutterReply callback) {
        NSArray *args = message;
        NSNumber *arg_enable = GetNullableObjectAtIndex(args, 0);
        FlutterError *error;
        NSNumber *output = [api enableVideoCorrectionEnable:arg_enable error:&error];
        callback(wrapResult(output, error));
      }];
    } else {
      [channel setMessageHandler:nil];
    }
  }
  {
    FlutterBasicMessageChannel *channel = [[FlutterBasicMessageChannel alloc]
           initWithName:@"dev.flutter.pigeon.EngineApi.reportCustomEvent"
        binaryMessenger:binaryMessenger
                  codec:NEFLTEngineApiGetCodec()];
    if (api) {
      NSCAssert(
          [api respondsToSelector:@selector(reportCustomEventRequest:error:)],
          @"NEFLTEngineApi api (%@) doesn't respond to @selector(reportCustomEventRequest:error:)",
          api);
      [channel setMessageHandler:^(id _Nullable message, FlutterReply callback) {
        NSArray *args = message;
        NEFLTReportCustomEventRequest *arg_request = GetNullableObjectAtIndex(args, 0);
        FlutterError *error;
        NSNumber *output = [api reportCustomEventRequest:arg_request error:&error];
        callback(wrapResult(output, error));
      }];
    } else {
      [channel setMessageHandler:nil];
    }
  }
  {
    FlutterBasicMessageChannel *channel = [[FlutterBasicMessageChannel alloc]
           initWithName:@"dev.flutter.pigeon.EngineApi.getEffectDuration"
        binaryMessenger:binaryMessenger
                  codec:NEFLTEngineApiGetCodec()];
    if (api) {
      NSCAssert(
          [api respondsToSelector:@selector(getEffectDurationEffectId:error:)],
          @"NEFLTEngineApi api (%@) doesn't respond to @selector(getEffectDurationEffectId:error:)",
          api);
      [channel setMessageHandler:^(id _Nullable message, FlutterReply callback) {
        NSArray *args = message;
        NSNumber *arg_effectId = GetNullableObjectAtIndex(args, 0);
        FlutterError *error;
        NSNumber *output = [api getEffectDurationEffectId:arg_effectId error:&error];
        callback(wrapResult(output, error));
      }];
    } else {
      [channel setMessageHandler:nil];
    }
  }
  {
    FlutterBasicMessageChannel *channel = [[FlutterBasicMessageChannel alloc]
           initWithName:@"dev.flutter.pigeon.EngineApi.startLastmileProbeTest"
        binaryMessenger:binaryMessenger
                  codec:NEFLTEngineApiGetCodec()];
    if (api) {
      NSCAssert([api respondsToSelector:@selector(startLastmileProbeTestRequest:error:)],
                @"NEFLTEngineApi api (%@) doesn't respond to "
                @"@selector(startLastmileProbeTestRequest:error:)",
                api);
      [channel setMessageHandler:^(id _Nullable message, FlutterReply callback) {
        NSArray *args = message;
        NEFLTStartLastmileProbeTestRequest *arg_request = GetNullableObjectAtIndex(args, 0);
        FlutterError *error;
        NSNumber *output = [api startLastmileProbeTestRequest:arg_request error:&error];
        callback(wrapResult(output, error));
      }];
    } else {
      [channel setMessageHandler:nil];
    }
  }
  {
    FlutterBasicMessageChannel *channel = [[FlutterBasicMessageChannel alloc]
           initWithName:@"dev.flutter.pigeon.EngineApi.stopLastmileProbeTest"
        binaryMessenger:binaryMessenger
                  codec:NEFLTEngineApiGetCodec()];
    if (api) {
      NSCAssert(
          [api respondsToSelector:@selector(stopLastmileProbeTestWithError:)],
          @"NEFLTEngineApi api (%@) doesn't respond to @selector(stopLastmileProbeTestWithError:)",
          api);
      [channel setMessageHandler:^(id _Nullable message, FlutterReply callback) {
        FlutterError *error;
        NSNumber *output = [api stopLastmileProbeTestWithError:&error];
        callback(wrapResult(output, error));
      }];
    } else {
      [channel setMessageHandler:nil];
    }
  }
  {
    FlutterBasicMessageChannel *channel = [[FlutterBasicMessageChannel alloc]
           initWithName:@"dev.flutter.pigeon.EngineApi.setVideoCorrectionConfig"
        binaryMessenger:binaryMessenger
                  codec:NEFLTEngineApiGetCodec()];
    if (api) {
      NSCAssert([api respondsToSelector:@selector(setVideoCorrectionConfigRequest:error:)],
                @"NEFLTEngineApi api (%@) doesn't respond to "
                @"@selector(setVideoCorrectionConfigRequest:error:)",
                api);
      [channel setMessageHandler:^(id _Nullable message, FlutterReply callback) {
        NSArray *args = message;
        NEFLTSetVideoCorrectionConfigRequest *arg_request = GetNullableObjectAtIndex(args, 0);
        FlutterError *error;
        NSNumber *output = [api setVideoCorrectionConfigRequest:arg_request error:&error];
        callback(wrapResult(output, error));
      }];
    } else {
      [channel setMessageHandler:nil];
    }
  }
  {
    FlutterBasicMessageChannel *channel = [[FlutterBasicMessageChannel alloc]
           initWithName:@"dev.flutter.pigeon.EngineApi.enableVirtualBackground"
        binaryMessenger:binaryMessenger
                  codec:NEFLTEngineApiGetCodec()];
    if (api) {
      NSCAssert([api respondsToSelector:@selector(enableVirtualBackgroundRequest:error:)],
                @"NEFLTEngineApi api (%@) doesn't respond to "
                @"@selector(enableVirtualBackgroundRequest:error:)",
                api);
      [channel setMessageHandler:^(id _Nullable message, FlutterReply callback) {
        NSArray *args = message;
        NEFLTEnableVirtualBackgroundRequest *arg_request = GetNullableObjectAtIndex(args, 0);
        FlutterError *error;
        NSNumber *output = [api enableVirtualBackgroundRequest:arg_request error:&error];
        callback(wrapResult(output, error));
      }];
    } else {
      [channel setMessageHandler:nil];
    }
  }
  {
    FlutterBasicMessageChannel *channel = [[FlutterBasicMessageChannel alloc]
           initWithName:@"dev.flutter.pigeon.EngineApi.setRemoteHighPriorityAudioStream"
        binaryMessenger:binaryMessenger
                  codec:NEFLTEngineApiGetCodec()];
    if (api) {
      NSCAssert([api respondsToSelector:@selector(setRemoteHighPriorityAudioStreamRequest:error:)],
                @"NEFLTEngineApi api (%@) doesn't respond to "
                @"@selector(setRemoteHighPriorityAudioStreamRequest:error:)",
                api);
      [channel setMessageHandler:^(id _Nullable message, FlutterReply callback) {
        NSArray *args = message;
        NEFLTSetRemoteHighPriorityAudioStreamRequest *arg_request =
            GetNullableObjectAtIndex(args, 0);
        FlutterError *error;
        NSNumber *output = [api setRemoteHighPriorityAudioStreamRequest:arg_request error:&error];
        callback(wrapResult(output, error));
      }];
    } else {
      [channel setMessageHandler:nil];
    }
  }
  {
    FlutterBasicMessageChannel *channel = [[FlutterBasicMessageChannel alloc]
           initWithName:@"dev.flutter.pigeon.EngineApi.setCloudProxy"
        binaryMessenger:binaryMessenger
                  codec:NEFLTEngineApiGetCodec()];
    if (api) {
      NSCAssert(
          [api respondsToSelector:@selector(setCloudProxyProxyType:error:)],
          @"NEFLTEngineApi api (%@) doesn't respond to @selector(setCloudProxyProxyType:error:)",
          api);
      [channel setMessageHandler:^(id _Nullable message, FlutterReply callback) {
        NSArray *args = message;
        NSNumber *arg_proxyType = GetNullableObjectAtIndex(args, 0);
        FlutterError *error;
        NSNumber *output = [api setCloudProxyProxyType:arg_proxyType error:&error];
        callback(wrapResult(output, error));
      }];
    } else {
      [channel setMessageHandler:nil];
    }
  }
  {
    FlutterBasicMessageChannel *channel =
        [[FlutterBasicMessageChannel alloc] initWithName:@"dev.flutter.pigeon.EngineApi.startBeauty"
                                         binaryMessenger:binaryMessenger
                                                   codec:NEFLTEngineApiGetCodec()];
    if (api) {
      NSCAssert([api respondsToSelector:@selector(startBeautyWithError:)],
                @"NEFLTEngineApi api (%@) doesn't respond to @selector(startBeautyWithError:)",
                api);
      [channel setMessageHandler:^(id _Nullable message, FlutterReply callback) {
        FlutterError *error;
        NSNumber *output = [api startBeautyWithError:&error];
        callback(wrapResult(output, error));
      }];
    } else {
      [channel setMessageHandler:nil];
    }
  }
  {
    FlutterBasicMessageChannel *channel =
        [[FlutterBasicMessageChannel alloc] initWithName:@"dev.flutter.pigeon.EngineApi.stopBeauty"
                                         binaryMessenger:binaryMessenger
                                                   codec:NEFLTEngineApiGetCodec()];
    if (api) {
      NSCAssert([api respondsToSelector:@selector(stopBeautyWithError:)],
                @"NEFLTEngineApi api (%@) doesn't respond to @selector(stopBeautyWithError:)", api);
      [channel setMessageHandler:^(id _Nullable message, FlutterReply callback) {
        FlutterError *error;
        [api stopBeautyWithError:&error];
        callback(wrapResult(nil, error));
      }];
    } else {
      [channel setMessageHandler:nil];
    }
  }
  {
    FlutterBasicMessageChannel *channel = [[FlutterBasicMessageChannel alloc]
           initWithName:@"dev.flutter.pigeon.EngineApi.enableBeauty"
        binaryMessenger:binaryMessenger
                  codec:NEFLTEngineApiGetCodec()];
    if (api) {
      NSCAssert([api respondsToSelector:@selector(enableBeautyEnabled:error:)],
                @"NEFLTEngineApi api (%@) doesn't respond to @selector(enableBeautyEnabled:error:)",
                api);
      [channel setMessageHandler:^(id _Nullable message, FlutterReply callback) {
        NSArray *args = message;
        NSNumber *arg_enabled = GetNullableObjectAtIndex(args, 0);
        FlutterError *error;
        NSNumber *output = [api enableBeautyEnabled:arg_enabled error:&error];
        callback(wrapResult(output, error));
      }];
    } else {
      [channel setMessageHandler:nil];
    }
  }
  {
    FlutterBasicMessageChannel *channel = [[FlutterBasicMessageChannel alloc]
           initWithName:@"dev.flutter.pigeon.EngineApi.setBeautyEffect"
        binaryMessenger:binaryMessenger
                  codec:NEFLTEngineApiGetCodec()];
    if (api) {
      NSCAssert([api respondsToSelector:@selector(setBeautyEffectLevel:beautyType:error:)],
                @"NEFLTEngineApi api (%@) doesn't respond to "
                @"@selector(setBeautyEffectLevel:beautyType:error:)",
                api);
      [channel setMessageHandler:^(id _Nullable message, FlutterReply callback) {
        NSArray *args = message;
        NSNumber *arg_level = GetNullableObjectAtIndex(args, 0);
        NSNumber *arg_beautyType = GetNullableObjectAtIndex(args, 1);
        FlutterError *error;
        NSNumber *output = [api setBeautyEffectLevel:arg_level
                                          beautyType:arg_beautyType
                                               error:&error];
        callback(wrapResult(output, error));
      }];
    } else {
      [channel setMessageHandler:nil];
    }
  }
  {
    FlutterBasicMessageChannel *channel = [[FlutterBasicMessageChannel alloc]
           initWithName:@"dev.flutter.pigeon.EngineApi.addBeautyFilter"
        binaryMessenger:binaryMessenger
                  codec:NEFLTEngineApiGetCodec()];
    if (api) {
      NSCAssert(
          [api respondsToSelector:@selector(addBeautyFilterPath:name:error:)],
          @"NEFLTEngineApi api (%@) doesn't respond to @selector(addBeautyFilterPath:name:error:)",
          api);
      [channel setMessageHandler:^(id _Nullable message, FlutterReply callback) {
        NSArray *args = message;
        NSString *arg_path = GetNullableObjectAtIndex(args, 0);
        NSString *arg_name = GetNullableObjectAtIndex(args, 1);
        FlutterError *error;
        NSNumber *output = [api addBeautyFilterPath:arg_path name:arg_name error:&error];
        callback(wrapResult(output, error));
      }];
    } else {
      [channel setMessageHandler:nil];
    }
  }
  {
    FlutterBasicMessageChannel *channel = [[FlutterBasicMessageChannel alloc]
           initWithName:@"dev.flutter.pigeon.EngineApi.removeBeautyFilter"
        binaryMessenger:binaryMessenger
                  codec:NEFLTEngineApiGetCodec()];
    if (api) {
      NSCAssert(
          [api respondsToSelector:@selector(removeBeautyFilterWithError:)],
          @"NEFLTEngineApi api (%@) doesn't respond to @selector(removeBeautyFilterWithError:)",
          api);
      [channel setMessageHandler:^(id _Nullable message, FlutterReply callback) {
        FlutterError *error;
        [api removeBeautyFilterWithError:&error];
        callback(wrapResult(nil, error));
      }];
    } else {
      [channel setMessageHandler:nil];
    }
  }
  {
    FlutterBasicMessageChannel *channel = [[FlutterBasicMessageChannel alloc]
           initWithName:@"dev.flutter.pigeon.EngineApi.setBeautyFilterLevel"
        binaryMessenger:binaryMessenger
                  codec:NEFLTEngineApiGetCodec()];
    if (api) {
      NSCAssert(
          [api respondsToSelector:@selector(setBeautyFilterLevelLevel:error:)],
          @"NEFLTEngineApi api (%@) doesn't respond to @selector(setBeautyFilterLevelLevel:error:)",
          api);
      [channel setMessageHandler:^(id _Nullable message, FlutterReply callback) {
        NSArray *args = message;
        NSNumber *arg_level = GetNullableObjectAtIndex(args, 0);
        FlutterError *error;
        NSNumber *output = [api setBeautyFilterLevelLevel:arg_level error:&error];
        callback(wrapResult(output, error));
      }];
    } else {
      [channel setMessageHandler:nil];
    }
  }
  {
    FlutterBasicMessageChannel *channel = [[FlutterBasicMessageChannel alloc]
           initWithName:@"dev.flutter.pigeon.EngineApi.setLocalVideoWatermarkConfigs"
        binaryMessenger:binaryMessenger
                  codec:NEFLTEngineApiGetCodec()];
    if (api) {
      NSCAssert([api respondsToSelector:@selector(setLocalVideoWatermarkConfigsRequest:error:)],
                @"NEFLTEngineApi api (%@) doesn't respond to "
                @"@selector(setLocalVideoWatermarkConfigsRequest:error:)",
                api);
      [channel setMessageHandler:^(id _Nullable message, FlutterReply callback) {
        NSArray *args = message;
        NEFLTSetLocalVideoWatermarkConfigsRequest *arg_request = GetNullableObjectAtIndex(args, 0);
        FlutterError *error;
        NSNumber *output = [api setLocalVideoWatermarkConfigsRequest:arg_request error:&error];
        callback(wrapResult(output, error));
      }];
    } else {
      [channel setMessageHandler:nil];
    }
  }
  {
    FlutterBasicMessageChannel *channel = [[FlutterBasicMessageChannel alloc]
           initWithName:@"dev.flutter.pigeon.EngineApi.setStreamAlignmentProperty"
        binaryMessenger:binaryMessenger
                  codec:NEFLTEngineApiGetCodec()];
    if (api) {
      NSCAssert([api respondsToSelector:@selector(setStreamAlignmentPropertyEnable:error:)],
                @"NEFLTEngineApi api (%@) doesn't respond to "
                @"@selector(setStreamAlignmentPropertyEnable:error:)",
                api);
      [channel setMessageHandler:^(id _Nullable message, FlutterReply callback) {
        NSArray *args = message;
        NSNumber *arg_enable = GetNullableObjectAtIndex(args, 0);
        FlutterError *error;
        NSNumber *output = [api setStreamAlignmentPropertyEnable:arg_enable error:&error];
        callback(wrapResult(output, error));
      }];
    } else {
      [channel setMessageHandler:nil];
    }
  }
  {
    FlutterBasicMessageChannel *channel = [[FlutterBasicMessageChannel alloc]
           initWithName:@"dev.flutter.pigeon.EngineApi.getNtpTimeOffset"
        binaryMessenger:binaryMessenger
                  codec:NEFLTEngineApiGetCodec()];
    if (api) {
      NSCAssert([api respondsToSelector:@selector(getNtpTimeOffsetWithError:)],
                @"NEFLTEngineApi api (%@) doesn't respond to @selector(getNtpTimeOffsetWithError:)",
                api);
      [channel setMessageHandler:^(id _Nullable message, FlutterReply callback) {
        FlutterError *error;
        NSNumber *output = [api getNtpTimeOffsetWithError:&error];
        callback(wrapResult(output, error));
      }];
    } else {
      [channel setMessageHandler:nil];
    }
  }
  {
    FlutterBasicMessageChannel *channel = [[FlutterBasicMessageChannel alloc]
           initWithName:@"dev.flutter.pigeon.EngineApi.takeLocalSnapshot"
        binaryMessenger:binaryMessenger
                  codec:NEFLTEngineApiGetCodec()];
    if (api) {
      NSCAssert([api respondsToSelector:@selector(takeLocalSnapshotStreamType:path:error:)],
                @"NEFLTEngineApi api (%@) doesn't respond to "
                @"@selector(takeLocalSnapshotStreamType:path:error:)",
                api);
      [channel setMessageHandler:^(id _Nullable message, FlutterReply callback) {
        NSArray *args = message;
        NSNumber *arg_streamType = GetNullableObjectAtIndex(args, 0);
        NSString *arg_path = GetNullableObjectAtIndex(args, 1);
        FlutterError *error;
        NSNumber *output = [api takeLocalSnapshotStreamType:arg_streamType
                                                       path:arg_path
                                                      error:&error];
        callback(wrapResult(output, error));
      }];
    } else {
      [channel setMessageHandler:nil];
    }
  }
  {
    FlutterBasicMessageChannel *channel = [[FlutterBasicMessageChannel alloc]
           initWithName:@"dev.flutter.pigeon.EngineApi.takeRemoteSnapshot"
        binaryMessenger:binaryMessenger
                  codec:NEFLTEngineApiGetCodec()];
    if (api) {
      NSCAssert([api respondsToSelector:@selector(takeRemoteSnapshotUid:streamType:path:error:)],
                @"NEFLTEngineApi api (%@) doesn't respond to "
                @"@selector(takeRemoteSnapshotUid:streamType:path:error:)",
                api);
      [channel setMessageHandler:^(id _Nullable message, FlutterReply callback) {
        NSArray *args = message;
        NSNumber *arg_uid = GetNullableObjectAtIndex(args, 0);
        NSNumber *arg_streamType = GetNullableObjectAtIndex(args, 1);
        NSString *arg_path = GetNullableObjectAtIndex(args, 2);
        FlutterError *error;
        NSNumber *output = [api takeRemoteSnapshotUid:arg_uid
                                           streamType:arg_streamType
                                                 path:arg_path
                                                error:&error];
        callback(wrapResult(output, error));
      }];
    } else {
      [channel setMessageHandler:nil];
    }
  }
}
NSObject<FlutterMessageCodec> *NEFLTVideoRendererApiGetCodec() {
  static FlutterStandardMessageCodec *sSharedObject = nil;
  sSharedObject = [FlutterStandardMessageCodec sharedInstance];
  return sSharedObject;
}

void NEFLTVideoRendererApiSetup(id<FlutterBinaryMessenger> binaryMessenger,
                                NSObject<NEFLTVideoRendererApi> *api) {
  {
    FlutterBasicMessageChannel *channel = [[FlutterBasicMessageChannel alloc]
           initWithName:@"dev.flutter.pigeon.VideoRendererApi.createVideoRenderer"
        binaryMessenger:binaryMessenger
                  codec:NEFLTVideoRendererApiGetCodec()];
    if (api) {
      NSCAssert([api respondsToSelector:@selector(createVideoRendererWithError:)],
                @"NEFLTVideoRendererApi api (%@) doesn't respond to "
                @"@selector(createVideoRendererWithError:)",
                api);
      [channel setMessageHandler:^(id _Nullable message, FlutterReply callback) {
        FlutterError *error;
        NSNumber *output = [api createVideoRendererWithError:&error];
        callback(wrapResult(output, error));
      }];
    } else {
      [channel setMessageHandler:nil];
    }
  }
  {
    FlutterBasicMessageChannel *channel = [[FlutterBasicMessageChannel alloc]
           initWithName:@"dev.flutter.pigeon.VideoRendererApi.setMirror"
        binaryMessenger:binaryMessenger
                  codec:NEFLTVideoRendererApiGetCodec()];
    if (api) {
      NSCAssert([api respondsToSelector:@selector(setMirrorTextureId:mirror:error:)],
                @"NEFLTVideoRendererApi api (%@) doesn't respond to "
                @"@selector(setMirrorTextureId:mirror:error:)",
                api);
      [channel setMessageHandler:^(id _Nullable message, FlutterReply callback) {
        NSArray *args = message;
        NSNumber *arg_textureId = GetNullableObjectAtIndex(args, 0);
        NSNumber *arg_mirror = GetNullableObjectAtIndex(args, 1);
        FlutterError *error;
        NSNumber *output = [api setMirrorTextureId:arg_textureId mirror:arg_mirror error:&error];
        callback(wrapResult(output, error));
      }];
    } else {
      [channel setMessageHandler:nil];
    }
  }
  {
    FlutterBasicMessageChannel *channel = [[FlutterBasicMessageChannel alloc]
           initWithName:@"dev.flutter.pigeon.VideoRendererApi.setupLocalVideoRenderer"
        binaryMessenger:binaryMessenger
                  codec:NEFLTVideoRendererApiGetCodec()];
    if (api) {
      NSCAssert([api respondsToSelector:@selector(setupLocalVideoRendererTextureId:error:)],
                @"NEFLTVideoRendererApi api (%@) doesn't respond to "
                @"@selector(setupLocalVideoRendererTextureId:error:)",
                api);
      [channel setMessageHandler:^(id _Nullable message, FlutterReply callback) {
        NSArray *args = message;
        NSNumber *arg_textureId = GetNullableObjectAtIndex(args, 0);
        FlutterError *error;
        NSNumber *output = [api setupLocalVideoRendererTextureId:arg_textureId error:&error];
        callback(wrapResult(output, error));
      }];
    } else {
      [channel setMessageHandler:nil];
    }
  }
  {
    FlutterBasicMessageChannel *channel = [[FlutterBasicMessageChannel alloc]
           initWithName:@"dev.flutter.pigeon.VideoRendererApi.setupRemoteVideoRenderer"
        binaryMessenger:binaryMessenger
                  codec:NEFLTVideoRendererApiGetCodec()];
    if (api) {
      NSCAssert([api respondsToSelector:@selector(setupRemoteVideoRendererUid:textureId:error:)],
                @"NEFLTVideoRendererApi api (%@) doesn't respond to "
                @"@selector(setupRemoteVideoRendererUid:textureId:error:)",
                api);
      [channel setMessageHandler:^(id _Nullable message, FlutterReply callback) {
        NSArray *args = message;
        NSNumber *arg_uid = GetNullableObjectAtIndex(args, 0);
        NSNumber *arg_textureId = GetNullableObjectAtIndex(args, 1);
        FlutterError *error;
        NSNumber *output = [api setupRemoteVideoRendererUid:arg_uid
                                                  textureId:arg_textureId
                                                      error:&error];
        callback(wrapResult(output, error));
      }];
    } else {
      [channel setMessageHandler:nil];
    }
  }
  {
    FlutterBasicMessageChannel *channel = [[FlutterBasicMessageChannel alloc]
           initWithName:@"dev.flutter.pigeon.VideoRendererApi.setupLocalSubStreamVideoRenderer"
        binaryMessenger:binaryMessenger
                  codec:NEFLTVideoRendererApiGetCodec()];
    if (api) {
      NSCAssert([api respondsToSelector:@selector(setupLocalSubStreamVideoRendererTextureId:
                                                                                      error:)],
                @"NEFLTVideoRendererApi api (%@) doesn't respond to "
                @"@selector(setupLocalSubStreamVideoRendererTextureId:error:)",
                api);
      [channel setMessageHandler:^(id _Nullable message, FlutterReply callback) {
        NSArray *args = message;
        NSNumber *arg_textureId = GetNullableObjectAtIndex(args, 0);
        FlutterError *error;
        NSNumber *output = [api setupLocalSubStreamVideoRendererTextureId:arg_textureId
                                                                    error:&error];
        callback(wrapResult(output, error));
      }];
    } else {
      [channel setMessageHandler:nil];
    }
  }
  {
    FlutterBasicMessageChannel *channel = [[FlutterBasicMessageChannel alloc]
           initWithName:@"dev.flutter.pigeon.VideoRendererApi.setupRemoteSubStreamVideoRenderer"
        binaryMessenger:binaryMessenger
                  codec:NEFLTVideoRendererApiGetCodec()];
    if (api) {
      NSCAssert([api respondsToSelector:@selector(setupRemoteSubStreamVideoRendererUid:
                                                                             textureId:error:)],
                @"NEFLTVideoRendererApi api (%@) doesn't respond to "
                @"@selector(setupRemoteSubStreamVideoRendererUid:textureId:error:)",
                api);
      [channel setMessageHandler:^(id _Nullable message, FlutterReply callback) {
        NSArray *args = message;
        NSNumber *arg_uid = GetNullableObjectAtIndex(args, 0);
        NSNumber *arg_textureId = GetNullableObjectAtIndex(args, 1);
        FlutterError *error;
        NSNumber *output = [api setupRemoteSubStreamVideoRendererUid:arg_uid
                                                           textureId:arg_textureId
                                                               error:&error];
        callback(wrapResult(output, error));
      }];
    } else {
      [channel setMessageHandler:nil];
    }
  }
  {
    FlutterBasicMessageChannel *channel = [[FlutterBasicMessageChannel alloc]
           initWithName:@"dev.flutter.pigeon.VideoRendererApi.disposeVideoRenderer"
        binaryMessenger:binaryMessenger
                  codec:NEFLTVideoRendererApiGetCodec()];
    if (api) {
      NSCAssert([api respondsToSelector:@selector(disposeVideoRendererTextureId:error:)],
                @"NEFLTVideoRendererApi api (%@) doesn't respond to "
                @"@selector(disposeVideoRendererTextureId:error:)",
                api);
      [channel setMessageHandler:^(id _Nullable message, FlutterReply callback) {
        NSArray *args = message;
        NSNumber *arg_textureId = GetNullableObjectAtIndex(args, 0);
        FlutterError *error;
        [api disposeVideoRendererTextureId:arg_textureId error:&error];
        callback(wrapResult(nil, error));
      }];
    } else {
      [channel setMessageHandler:nil];
    }
  }
}
@interface NEFLTNERtcDeviceEventSinkCodecReader : FlutterStandardReader
@end
@implementation NEFLTNERtcDeviceEventSinkCodecReader
- (nullable id)readValueOfType:(UInt8)type {
  switch (type) {
    case 128:
      return [NEFLTCGPoint fromList:[self readValue]];
    default:
      return [super readValueOfType:type];
  }
}
@end

@interface NEFLTNERtcDeviceEventSinkCodecWriter : FlutterStandardWriter
@end
@implementation NEFLTNERtcDeviceEventSinkCodecWriter
- (void)writeValue:(id)value {
  if ([value isKindOfClass:[NEFLTCGPoint class]]) {
    [self writeByte:128];
    [self writeValue:[value toList]];
  } else {
    [super writeValue:value];
  }
}
@end

@interface NEFLTNERtcDeviceEventSinkCodecReaderWriter : FlutterStandardReaderWriter
@end
@implementation NEFLTNERtcDeviceEventSinkCodecReaderWriter
- (FlutterStandardWriter *)writerWithData:(NSMutableData *)data {
  return [[NEFLTNERtcDeviceEventSinkCodecWriter alloc] initWithData:data];
}
- (FlutterStandardReader *)readerWithData:(NSData *)data {
  return [[NEFLTNERtcDeviceEventSinkCodecReader alloc] initWithData:data];
}
@end

NSObject<FlutterMessageCodec> *NEFLTNERtcDeviceEventSinkGetCodec() {
  static FlutterStandardMessageCodec *sSharedObject = nil;
  static dispatch_once_t sPred = 0;
  dispatch_once(&sPred, ^{
    NEFLTNERtcDeviceEventSinkCodecReaderWriter *readerWriter =
        [[NEFLTNERtcDeviceEventSinkCodecReaderWriter alloc] init];
    sSharedObject = [FlutterStandardMessageCodec codecWithReaderWriter:readerWriter];
  });
  return sSharedObject;
}

@interface NEFLTNERtcDeviceEventSink ()
@property(nonatomic, strong) NSObject<FlutterBinaryMessenger> *binaryMessenger;
@end

@implementation NEFLTNERtcDeviceEventSink

- (instancetype)initWithBinaryMessenger:(NSObject<FlutterBinaryMessenger> *)binaryMessenger {
  self = [super init];
  if (self) {
    _binaryMessenger = binaryMessenger;
  }
  return self;
}
- (void)onAudioDeviceChangedSelected:(NSNumber *)arg_selected
                          completion:(void (^)(FlutterError *_Nullable))completion {
  FlutterBasicMessageChannel *channel = [FlutterBasicMessageChannel
      messageChannelWithName:@"dev.flutter.pigeon.NERtcDeviceEventSink.onAudioDeviceChanged"
             binaryMessenger:self.binaryMessenger
                       codec:NEFLTNERtcDeviceEventSinkGetCodec()];
  [channel sendMessage:@[ arg_selected ?: [NSNull null] ]
                 reply:^(id reply) {
                   completion(nil);
                 }];
}
- (void)onAudioDeviceStateChangeDeviceType:(NSNumber *)arg_deviceType
                               deviceState:(NSNumber *)arg_deviceState
                                completion:(void (^)(FlutterError *_Nullable))completion {
  FlutterBasicMessageChannel *channel = [FlutterBasicMessageChannel
      messageChannelWithName:@"dev.flutter.pigeon.NERtcDeviceEventSink.onAudioDeviceStateChange"
             binaryMessenger:self.binaryMessenger
                       codec:NEFLTNERtcDeviceEventSinkGetCodec()];
  [channel sendMessage:@[ arg_deviceType ?: [NSNull null], arg_deviceState ?: [NSNull null] ]
                 reply:^(id reply) {
                   completion(nil);
                 }];
}
- (void)onVideoDeviceStateChangeDeviceType:(NSNumber *)arg_deviceType
                               deviceState:(NSNumber *)arg_deviceState
                                completion:(void (^)(FlutterError *_Nullable))completion {
  FlutterBasicMessageChannel *channel = [FlutterBasicMessageChannel
      messageChannelWithName:@"dev.flutter.pigeon.NERtcDeviceEventSink.onVideoDeviceStateChange"
             binaryMessenger:self.binaryMessenger
                       codec:NEFLTNERtcDeviceEventSinkGetCodec()];
  [channel sendMessage:@[ arg_deviceType ?: [NSNull null], arg_deviceState ?: [NSNull null] ]
                 reply:^(id reply) {
                   completion(nil);
                 }];
}
- (void)onCameraFocusChangedFocusPoint:(NEFLTCGPoint *)arg_focusPoint
                            completion:(void (^)(FlutterError *_Nullable))completion {
  FlutterBasicMessageChannel *channel = [FlutterBasicMessageChannel
      messageChannelWithName:@"dev.flutter.pigeon.NERtcDeviceEventSink.onCameraFocusChanged"
             binaryMessenger:self.binaryMessenger
                       codec:NEFLTNERtcDeviceEventSinkGetCodec()];
  [channel sendMessage:@[ arg_focusPoint ?: [NSNull null] ]
                 reply:^(id reply) {
                   completion(nil);
                 }];
}
- (void)onCameraExposureChangedExposurePoint:(NEFLTCGPoint *)arg_exposurePoint
                                  completion:(void (^)(FlutterError *_Nullable))completion {
  FlutterBasicMessageChannel *channel = [FlutterBasicMessageChannel
      messageChannelWithName:@"dev.flutter.pigeon.NERtcDeviceEventSink.onCameraExposureChanged"
             binaryMessenger:self.binaryMessenger
                       codec:NEFLTNERtcDeviceEventSinkGetCodec()];
  [channel sendMessage:@[ arg_exposurePoint ?: [NSNull null] ]
                 reply:^(id reply) {
                   completion(nil);
                 }];
}
@end

@interface NEFLTDeviceManagerApiCodecReader : FlutterStandardReader
@end
@implementation NEFLTDeviceManagerApiCodecReader
- (nullable id)readValueOfType:(UInt8)type {
  switch (type) {
    case 128:
      return [NEFLTSetCameraPositionRequest fromList:[self readValue]];
    default:
      return [super readValueOfType:type];
  }
}
@end

@interface NEFLTDeviceManagerApiCodecWriter : FlutterStandardWriter
@end
@implementation NEFLTDeviceManagerApiCodecWriter
- (void)writeValue:(id)value {
  if ([value isKindOfClass:[NEFLTSetCameraPositionRequest class]]) {
    [self writeByte:128];
    [self writeValue:[value toList]];
  } else {
    [super writeValue:value];
  }
}
@end

@interface NEFLTDeviceManagerApiCodecReaderWriter : FlutterStandardReaderWriter
@end
@implementation NEFLTDeviceManagerApiCodecReaderWriter
- (FlutterStandardWriter *)writerWithData:(NSMutableData *)data {
  return [[NEFLTDeviceManagerApiCodecWriter alloc] initWithData:data];
}
- (FlutterStandardReader *)readerWithData:(NSData *)data {
  return [[NEFLTDeviceManagerApiCodecReader alloc] initWithData:data];
}
@end

NSObject<FlutterMessageCodec> *NEFLTDeviceManagerApiGetCodec() {
  static FlutterStandardMessageCodec *sSharedObject = nil;
  static dispatch_once_t sPred = 0;
  dispatch_once(&sPred, ^{
    NEFLTDeviceManagerApiCodecReaderWriter *readerWriter =
        [[NEFLTDeviceManagerApiCodecReaderWriter alloc] init];
    sSharedObject = [FlutterStandardMessageCodec codecWithReaderWriter:readerWriter];
  });
  return sSharedObject;
}

void NEFLTDeviceManagerApiSetup(id<FlutterBinaryMessenger> binaryMessenger,
                                NSObject<NEFLTDeviceManagerApi> *api) {
  {
    FlutterBasicMessageChannel *channel = [[FlutterBasicMessageChannel alloc]
           initWithName:@"dev.flutter.pigeon.DeviceManagerApi.isSpeakerphoneOn"
        binaryMessenger:binaryMessenger
                  codec:NEFLTDeviceManagerApiGetCodec()];
    if (api) {
      NSCAssert([api respondsToSelector:@selector(isSpeakerphoneOnWithError:)],
                @"NEFLTDeviceManagerApi api (%@) doesn't respond to "
                @"@selector(isSpeakerphoneOnWithError:)",
                api);
      [channel setMessageHandler:^(id _Nullable message, FlutterReply callback) {
        FlutterError *error;
        NSNumber *output = [api isSpeakerphoneOnWithError:&error];
        callback(wrapResult(output, error));
      }];
    } else {
      [channel setMessageHandler:nil];
    }
  }
  {
    FlutterBasicMessageChannel *channel = [[FlutterBasicMessageChannel alloc]
           initWithName:@"dev.flutter.pigeon.DeviceManagerApi.isCameraZoomSupported"
        binaryMessenger:binaryMessenger
                  codec:NEFLTDeviceManagerApiGetCodec()];
    if (api) {
      NSCAssert([api respondsToSelector:@selector(isCameraZoomSupportedWithError:)],
                @"NEFLTDeviceManagerApi api (%@) doesn't respond to "
                @"@selector(isCameraZoomSupportedWithError:)",
                api);
      [channel setMessageHandler:^(id _Nullable message, FlutterReply callback) {
        FlutterError *error;
        NSNumber *output = [api isCameraZoomSupportedWithError:&error];
        callback(wrapResult(output, error));
      }];
    } else {
      [channel setMessageHandler:nil];
    }
  }
  {
    FlutterBasicMessageChannel *channel = [[FlutterBasicMessageChannel alloc]
           initWithName:@"dev.flutter.pigeon.DeviceManagerApi.isCameraTorchSupported"
        binaryMessenger:binaryMessenger
                  codec:NEFLTDeviceManagerApiGetCodec()];
    if (api) {
      NSCAssert([api respondsToSelector:@selector(isCameraTorchSupportedWithError:)],
                @"NEFLTDeviceManagerApi api (%@) doesn't respond to "
                @"@selector(isCameraTorchSupportedWithError:)",
                api);
      [channel setMessageHandler:^(id _Nullable message, FlutterReply callback) {
        FlutterError *error;
        NSNumber *output = [api isCameraTorchSupportedWithError:&error];
        callback(wrapResult(output, error));
      }];
    } else {
      [channel setMessageHandler:nil];
    }
  }
  {
    FlutterBasicMessageChannel *channel = [[FlutterBasicMessageChannel alloc]
           initWithName:@"dev.flutter.pigeon.DeviceManagerApi.isCameraFocusSupported"
        binaryMessenger:binaryMessenger
                  codec:NEFLTDeviceManagerApiGetCodec()];
    if (api) {
      NSCAssert([api respondsToSelector:@selector(isCameraFocusSupportedWithError:)],
                @"NEFLTDeviceManagerApi api (%@) doesn't respond to "
                @"@selector(isCameraFocusSupportedWithError:)",
                api);
      [channel setMessageHandler:^(id _Nullable message, FlutterReply callback) {
        FlutterError *error;
        NSNumber *output = [api isCameraFocusSupportedWithError:&error];
        callback(wrapResult(output, error));
      }];
    } else {
      [channel setMessageHandler:nil];
    }
  }
  {
    FlutterBasicMessageChannel *channel = [[FlutterBasicMessageChannel alloc]
           initWithName:@"dev.flutter.pigeon.DeviceManagerApi.isCameraExposurePositionSupported"
        binaryMessenger:binaryMessenger
                  codec:NEFLTDeviceManagerApiGetCodec()];
    if (api) {
      NSCAssert([api respondsToSelector:@selector(isCameraExposurePositionSupportedWithError:)],
                @"NEFLTDeviceManagerApi api (%@) doesn't respond to "
                @"@selector(isCameraExposurePositionSupportedWithError:)",
                api);
      [channel setMessageHandler:^(id _Nullable message, FlutterReply callback) {
        FlutterError *error;
        NSNumber *output = [api isCameraExposurePositionSupportedWithError:&error];
        callback(wrapResult(output, error));
      }];
    } else {
      [channel setMessageHandler:nil];
    }
  }
  {
    FlutterBasicMessageChannel *channel = [[FlutterBasicMessageChannel alloc]
           initWithName:@"dev.flutter.pigeon.DeviceManagerApi.setSpeakerphoneOn"
        binaryMessenger:binaryMessenger
                  codec:NEFLTDeviceManagerApiGetCodec()];
    if (api) {
      NSCAssert([api respondsToSelector:@selector(setSpeakerphoneOnEnable:error:)],
                @"NEFLTDeviceManagerApi api (%@) doesn't respond to "
                @"@selector(setSpeakerphoneOnEnable:error:)",
                api);
      [channel setMessageHandler:^(id _Nullable message, FlutterReply callback) {
        NSArray *args = message;
        NSNumber *arg_enable = GetNullableObjectAtIndex(args, 0);
        FlutterError *error;
        NSNumber *output = [api setSpeakerphoneOnEnable:arg_enable error:&error];
        callback(wrapResult(output, error));
      }];
    } else {
      [channel setMessageHandler:nil];
    }
  }
  {
    FlutterBasicMessageChannel *channel = [[FlutterBasicMessageChannel alloc]
           initWithName:@"dev.flutter.pigeon.DeviceManagerApi.switchCamera"
        binaryMessenger:binaryMessenger
                  codec:NEFLTDeviceManagerApiGetCodec()];
    if (api) {
      NSCAssert(
          [api respondsToSelector:@selector(switchCameraWithError:)],
          @"NEFLTDeviceManagerApi api (%@) doesn't respond to @selector(switchCameraWithError:)",
          api);
      [channel setMessageHandler:^(id _Nullable message, FlutterReply callback) {
        FlutterError *error;
        NSNumber *output = [api switchCameraWithError:&error];
        callback(wrapResult(output, error));
      }];
    } else {
      [channel setMessageHandler:nil];
    }
  }
  {
    FlutterBasicMessageChannel *channel = [[FlutterBasicMessageChannel alloc]
           initWithName:@"dev.flutter.pigeon.DeviceManagerApi.setCameraZoomFactor"
        binaryMessenger:binaryMessenger
                  codec:NEFLTDeviceManagerApiGetCodec()];
    if (api) {
      NSCAssert([api respondsToSelector:@selector(setCameraZoomFactorFactor:error:)],
                @"NEFLTDeviceManagerApi api (%@) doesn't respond to "
                @"@selector(setCameraZoomFactorFactor:error:)",
                api);
      [channel setMessageHandler:^(id _Nullable message, FlutterReply callback) {
        NSArray *args = message;
        NSNumber *arg_factor = GetNullableObjectAtIndex(args, 0);
        FlutterError *error;
        NSNumber *output = [api setCameraZoomFactorFactor:arg_factor error:&error];
        callback(wrapResult(output, error));
      }];
    } else {
      [channel setMessageHandler:nil];
    }
  }
  {
    FlutterBasicMessageChannel *channel = [[FlutterBasicMessageChannel alloc]
           initWithName:@"dev.flutter.pigeon.DeviceManagerApi.getCameraMaxZoom"
        binaryMessenger:binaryMessenger
                  codec:NEFLTDeviceManagerApiGetCodec()];
    if (api) {
      NSCAssert([api respondsToSelector:@selector(getCameraMaxZoomWithError:)],
                @"NEFLTDeviceManagerApi api (%@) doesn't respond to "
                @"@selector(getCameraMaxZoomWithError:)",
                api);
      [channel setMessageHandler:^(id _Nullable message, FlutterReply callback) {
        FlutterError *error;
        NSNumber *output = [api getCameraMaxZoomWithError:&error];
        callback(wrapResult(output, error));
      }];
    } else {
      [channel setMessageHandler:nil];
    }
  }
  {
    FlutterBasicMessageChannel *channel = [[FlutterBasicMessageChannel alloc]
           initWithName:@"dev.flutter.pigeon.DeviceManagerApi.setCameraTorchOn"
        binaryMessenger:binaryMessenger
                  codec:NEFLTDeviceManagerApiGetCodec()];
    if (api) {
      NSCAssert(
          [api respondsToSelector:@selector(setCameraTorchOnOn:error:)],
          @"NEFLTDeviceManagerApi api (%@) doesn't respond to @selector(setCameraTorchOnOn:error:)",
          api);
      [channel setMessageHandler:^(id _Nullable message, FlutterReply callback) {
        NSArray *args = message;
        NSNumber *arg_on = GetNullableObjectAtIndex(args, 0);
        FlutterError *error;
        NSNumber *output = [api setCameraTorchOnOn:arg_on error:&error];
        callback(wrapResult(output, error));
      }];
    } else {
      [channel setMessageHandler:nil];
    }
  }
  {
    FlutterBasicMessageChannel *channel = [[FlutterBasicMessageChannel alloc]
           initWithName:@"dev.flutter.pigeon.DeviceManagerApi.setCameraFocusPosition"
        binaryMessenger:binaryMessenger
                  codec:NEFLTDeviceManagerApiGetCodec()];
    if (api) {
      NSCAssert([api respondsToSelector:@selector(setCameraFocusPositionRequest:error:)],
                @"NEFLTDeviceManagerApi api (%@) doesn't respond to "
                @"@selector(setCameraFocusPositionRequest:error:)",
                api);
      [channel setMessageHandler:^(id _Nullable message, FlutterReply callback) {
        NSArray *args = message;
        NEFLTSetCameraPositionRequest *arg_request = GetNullableObjectAtIndex(args, 0);
        FlutterError *error;
        NSNumber *output = [api setCameraFocusPositionRequest:arg_request error:&error];
        callback(wrapResult(output, error));
      }];
    } else {
      [channel setMessageHandler:nil];
    }
  }
  {
    FlutterBasicMessageChannel *channel = [[FlutterBasicMessageChannel alloc]
           initWithName:@"dev.flutter.pigeon.DeviceManagerApi.setCameraExposurePosition"
        binaryMessenger:binaryMessenger
                  codec:NEFLTDeviceManagerApiGetCodec()];
    if (api) {
      NSCAssert([api respondsToSelector:@selector(setCameraExposurePositionRequest:error:)],
                @"NEFLTDeviceManagerApi api (%@) doesn't respond to "
                @"@selector(setCameraExposurePositionRequest:error:)",
                api);
      [channel setMessageHandler:^(id _Nullable message, FlutterReply callback) {
        NSArray *args = message;
        NEFLTSetCameraPositionRequest *arg_request = GetNullableObjectAtIndex(args, 0);
        FlutterError *error;
        NSNumber *output = [api setCameraExposurePositionRequest:arg_request error:&error];
        callback(wrapResult(output, error));
      }];
    } else {
      [channel setMessageHandler:nil];
    }
  }
  {
    FlutterBasicMessageChannel *channel = [[FlutterBasicMessageChannel alloc]
           initWithName:@"dev.flutter.pigeon.DeviceManagerApi.setPlayoutDeviceMute"
        binaryMessenger:binaryMessenger
                  codec:NEFLTDeviceManagerApiGetCodec()];
    if (api) {
      NSCAssert([api respondsToSelector:@selector(setPlayoutDeviceMuteMute:error:)],
                @"NEFLTDeviceManagerApi api (%@) doesn't respond to "
                @"@selector(setPlayoutDeviceMuteMute:error:)",
                api);
      [channel setMessageHandler:^(id _Nullable message, FlutterReply callback) {
        NSArray *args = message;
        NSNumber *arg_mute = GetNullableObjectAtIndex(args, 0);
        FlutterError *error;
        NSNumber *output = [api setPlayoutDeviceMuteMute:arg_mute error:&error];
        callback(wrapResult(output, error));
      }];
    } else {
      [channel setMessageHandler:nil];
    }
  }
  {
    FlutterBasicMessageChannel *channel = [[FlutterBasicMessageChannel alloc]
           initWithName:@"dev.flutter.pigeon.DeviceManagerApi.isPlayoutDeviceMute"
        binaryMessenger:binaryMessenger
                  codec:NEFLTDeviceManagerApiGetCodec()];
    if (api) {
      NSCAssert([api respondsToSelector:@selector(isPlayoutDeviceMuteWithError:)],
                @"NEFLTDeviceManagerApi api (%@) doesn't respond to "
                @"@selector(isPlayoutDeviceMuteWithError:)",
                api);
      [channel setMessageHandler:^(id _Nullable message, FlutterReply callback) {
        FlutterError *error;
        NSNumber *output = [api isPlayoutDeviceMuteWithError:&error];
        callback(wrapResult(output, error));
      }];
    } else {
      [channel setMessageHandler:nil];
    }
  }
  {
    FlutterBasicMessageChannel *channel = [[FlutterBasicMessageChannel alloc]
           initWithName:@"dev.flutter.pigeon.DeviceManagerApi.setRecordDeviceMute"
        binaryMessenger:binaryMessenger
                  codec:NEFLTDeviceManagerApiGetCodec()];
    if (api) {
      NSCAssert([api respondsToSelector:@selector(setRecordDeviceMuteMute:error:)],
                @"NEFLTDeviceManagerApi api (%@) doesn't respond to "
                @"@selector(setRecordDeviceMuteMute:error:)",
                api);
      [channel setMessageHandler:^(id _Nullable message, FlutterReply callback) {
        NSArray *args = message;
        NSNumber *arg_mute = GetNullableObjectAtIndex(args, 0);
        FlutterError *error;
        NSNumber *output = [api setRecordDeviceMuteMute:arg_mute error:&error];
        callback(wrapResult(output, error));
      }];
    } else {
      [channel setMessageHandler:nil];
    }
  }
  {
    FlutterBasicMessageChannel *channel = [[FlutterBasicMessageChannel alloc]
           initWithName:@"dev.flutter.pigeon.DeviceManagerApi.isRecordDeviceMute"
        binaryMessenger:binaryMessenger
                  codec:NEFLTDeviceManagerApiGetCodec()];
    if (api) {
      NSCAssert([api respondsToSelector:@selector(isRecordDeviceMuteWithError:)],
                @"NEFLTDeviceManagerApi api (%@) doesn't respond to "
                @"@selector(isRecordDeviceMuteWithError:)",
                api);
      [channel setMessageHandler:^(id _Nullable message, FlutterReply callback) {
        FlutterError *error;
        NSNumber *output = [api isRecordDeviceMuteWithError:&error];
        callback(wrapResult(output, error));
      }];
    } else {
      [channel setMessageHandler:nil];
    }
  }
  {
    FlutterBasicMessageChannel *channel = [[FlutterBasicMessageChannel alloc]
           initWithName:@"dev.flutter.pigeon.DeviceManagerApi.enableEarback"
        binaryMessenger:binaryMessenger
                  codec:NEFLTDeviceManagerApiGetCodec()];
    if (api) {
      NSCAssert([api respondsToSelector:@selector(enableEarbackEnabled:volume:error:)],
                @"NEFLTDeviceManagerApi api (%@) doesn't respond to "
                @"@selector(enableEarbackEnabled:volume:error:)",
                api);
      [channel setMessageHandler:^(id _Nullable message, FlutterReply callback) {
        NSArray *args = message;
        NSNumber *arg_enabled = GetNullableObjectAtIndex(args, 0);
        NSNumber *arg_volume = GetNullableObjectAtIndex(args, 1);
        FlutterError *error;
        NSNumber *output = [api enableEarbackEnabled:arg_enabled volume:arg_volume error:&error];
        callback(wrapResult(output, error));
      }];
    } else {
      [channel setMessageHandler:nil];
    }
  }
  {
    FlutterBasicMessageChannel *channel = [[FlutterBasicMessageChannel alloc]
           initWithName:@"dev.flutter.pigeon.DeviceManagerApi.setEarbackVolume"
        binaryMessenger:binaryMessenger
                  codec:NEFLTDeviceManagerApiGetCodec()];
    if (api) {
      NSCAssert([api respondsToSelector:@selector(setEarbackVolumeVolume:error:)],
                @"NEFLTDeviceManagerApi api (%@) doesn't respond to "
                @"@selector(setEarbackVolumeVolume:error:)",
                api);
      [channel setMessageHandler:^(id _Nullable message, FlutterReply callback) {
        NSArray *args = message;
        NSNumber *arg_volume = GetNullableObjectAtIndex(args, 0);
        FlutterError *error;
        NSNumber *output = [api setEarbackVolumeVolume:arg_volume error:&error];
        callback(wrapResult(output, error));
      }];
    } else {
      [channel setMessageHandler:nil];
    }
  }
  {
    FlutterBasicMessageChannel *channel = [[FlutterBasicMessageChannel alloc]
           initWithName:@"dev.flutter.pigeon.DeviceManagerApi.setAudioFocusMode"
        binaryMessenger:binaryMessenger
                  codec:NEFLTDeviceManagerApiGetCodec()];
    if (api) {
      NSCAssert([api respondsToSelector:@selector(setAudioFocusModeFocusMode:error:)],
                @"NEFLTDeviceManagerApi api (%@) doesn't respond to "
                @"@selector(setAudioFocusModeFocusMode:error:)",
                api);
      [channel setMessageHandler:^(id _Nullable message, FlutterReply callback) {
        NSArray *args = message;
        NSNumber *arg_focusMode = GetNullableObjectAtIndex(args, 0);
        FlutterError *error;
        NSNumber *output = [api setAudioFocusModeFocusMode:arg_focusMode error:&error];
        callback(wrapResult(output, error));
      }];
    } else {
      [channel setMessageHandler:nil];
    }
  }
  {
    FlutterBasicMessageChannel *channel = [[FlutterBasicMessageChannel alloc]
           initWithName:@"dev.flutter.pigeon.DeviceManagerApi.getCurrentCamera"
        binaryMessenger:binaryMessenger
                  codec:NEFLTDeviceManagerApiGetCodec()];
    if (api) {
      NSCAssert([api respondsToSelector:@selector(getCurrentCameraWithError:)],
                @"NEFLTDeviceManagerApi api (%@) doesn't respond to "
                @"@selector(getCurrentCameraWithError:)",
                api);
      [channel setMessageHandler:^(id _Nullable message, FlutterReply callback) {
        FlutterError *error;
        NSNumber *output = [api getCurrentCameraWithError:&error];
        callback(wrapResult(output, error));
      }];
    } else {
      [channel setMessageHandler:nil];
    }
  }
  {
    FlutterBasicMessageChannel *channel = [[FlutterBasicMessageChannel alloc]
           initWithName:@"dev.flutter.pigeon.DeviceManagerApi.switchCameraWithPosition"
        binaryMessenger:binaryMessenger
                  codec:NEFLTDeviceManagerApiGetCodec()];
    if (api) {
      NSCAssert([api respondsToSelector:@selector(switchCameraWithPositionPosition:error:)],
                @"NEFLTDeviceManagerApi api (%@) doesn't respond to "
                @"@selector(switchCameraWithPositionPosition:error:)",
                api);
      [channel setMessageHandler:^(id _Nullable message, FlutterReply callback) {
        NSArray *args = message;
        NSNumber *arg_position = GetNullableObjectAtIndex(args, 0);
        FlutterError *error;
        NSNumber *output = [api switchCameraWithPositionPosition:arg_position error:&error];
        callback(wrapResult(output, error));
      }];
    } else {
      [channel setMessageHandler:nil];
    }
  }
  {
    FlutterBasicMessageChannel *channel = [[FlutterBasicMessageChannel alloc]
           initWithName:@"dev.flutter.pigeon.DeviceManagerApi.getCameraCurrentZoom"
        binaryMessenger:binaryMessenger
                  codec:NEFLTDeviceManagerApiGetCodec()];
    if (api) {
      NSCAssert([api respondsToSelector:@selector(getCameraCurrentZoomWithError:)],
                @"NEFLTDeviceManagerApi api (%@) doesn't respond to "
                @"@selector(getCameraCurrentZoomWithError:)",
                api);
      [channel setMessageHandler:^(id _Nullable message, FlutterReply callback) {
        FlutterError *error;
        NSNumber *output = [api getCameraCurrentZoomWithError:&error];
        callback(wrapResult(output, error));
      }];
    } else {
      [channel setMessageHandler:nil];
    }
  }
}
@interface NEFLTAudioMixingApiCodecReader : FlutterStandardReader
@end
@implementation NEFLTAudioMixingApiCodecReader
- (nullable id)readValueOfType:(UInt8)type {
  switch (type) {
    case 128:
      return [NEFLTStartAudioMixingRequest fromList:[self readValue]];
    default:
      return [super readValueOfType:type];
  }
}
@end

@interface NEFLTAudioMixingApiCodecWriter : FlutterStandardWriter
@end
@implementation NEFLTAudioMixingApiCodecWriter
- (void)writeValue:(id)value {
  if ([value isKindOfClass:[NEFLTStartAudioMixingRequest class]]) {
    [self writeByte:128];
    [self writeValue:[value toList]];
  } else {
    [super writeValue:value];
  }
}
@end

@interface NEFLTAudioMixingApiCodecReaderWriter : FlutterStandardReaderWriter
@end
@implementation NEFLTAudioMixingApiCodecReaderWriter
- (FlutterStandardWriter *)writerWithData:(NSMutableData *)data {
  return [[NEFLTAudioMixingApiCodecWriter alloc] initWithData:data];
}
- (FlutterStandardReader *)readerWithData:(NSData *)data {
  return [[NEFLTAudioMixingApiCodecReader alloc] initWithData:data];
}
@end

NSObject<FlutterMessageCodec> *NEFLTAudioMixingApiGetCodec() {
  static FlutterStandardMessageCodec *sSharedObject = nil;
  static dispatch_once_t sPred = 0;
  dispatch_once(&sPred, ^{
    NEFLTAudioMixingApiCodecReaderWriter *readerWriter =
        [[NEFLTAudioMixingApiCodecReaderWriter alloc] init];
    sSharedObject = [FlutterStandardMessageCodec codecWithReaderWriter:readerWriter];
  });
  return sSharedObject;
}

void NEFLTAudioMixingApiSetup(id<FlutterBinaryMessenger> binaryMessenger,
                              NSObject<NEFLTAudioMixingApi> *api) {
  {
    FlutterBasicMessageChannel *channel = [[FlutterBasicMessageChannel alloc]
           initWithName:@"dev.flutter.pigeon.AudioMixingApi.startAudioMixing"
        binaryMessenger:binaryMessenger
                  codec:NEFLTAudioMixingApiGetCodec()];
    if (api) {
      NSCAssert([api respondsToSelector:@selector(startAudioMixingRequest:error:)],
                @"NEFLTAudioMixingApi api (%@) doesn't respond to "
                @"@selector(startAudioMixingRequest:error:)",
                api);
      [channel setMessageHandler:^(id _Nullable message, FlutterReply callback) {
        NSArray *args = message;
        NEFLTStartAudioMixingRequest *arg_request = GetNullableObjectAtIndex(args, 0);
        FlutterError *error;
        NSNumber *output = [api startAudioMixingRequest:arg_request error:&error];
        callback(wrapResult(output, error));
      }];
    } else {
      [channel setMessageHandler:nil];
    }
  }
  {
    FlutterBasicMessageChannel *channel = [[FlutterBasicMessageChannel alloc]
           initWithName:@"dev.flutter.pigeon.AudioMixingApi.stopAudioMixing"
        binaryMessenger:binaryMessenger
                  codec:NEFLTAudioMixingApiGetCodec()];
    if (api) {
      NSCAssert(
          [api respondsToSelector:@selector(stopAudioMixingWithError:)],
          @"NEFLTAudioMixingApi api (%@) doesn't respond to @selector(stopAudioMixingWithError:)",
          api);
      [channel setMessageHandler:^(id _Nullable message, FlutterReply callback) {
        FlutterError *error;
        NSNumber *output = [api stopAudioMixingWithError:&error];
        callback(wrapResult(output, error));
      }];
    } else {
      [channel setMessageHandler:nil];
    }
  }
  {
    FlutterBasicMessageChannel *channel = [[FlutterBasicMessageChannel alloc]
           initWithName:@"dev.flutter.pigeon.AudioMixingApi.pauseAudioMixing"
        binaryMessenger:binaryMessenger
                  codec:NEFLTAudioMixingApiGetCodec()];
    if (api) {
      NSCAssert(
          [api respondsToSelector:@selector(pauseAudioMixingWithError:)],
          @"NEFLTAudioMixingApi api (%@) doesn't respond to @selector(pauseAudioMixingWithError:)",
          api);
      [channel setMessageHandler:^(id _Nullable message, FlutterReply callback) {
        FlutterError *error;
        NSNumber *output = [api pauseAudioMixingWithError:&error];
        callback(wrapResult(output, error));
      }];
    } else {
      [channel setMessageHandler:nil];
    }
  }
  {
    FlutterBasicMessageChannel *channel = [[FlutterBasicMessageChannel alloc]
           initWithName:@"dev.flutter.pigeon.AudioMixingApi.resumeAudioMixing"
        binaryMessenger:binaryMessenger
                  codec:NEFLTAudioMixingApiGetCodec()];
    if (api) {
      NSCAssert(
          [api respondsToSelector:@selector(resumeAudioMixingWithError:)],
          @"NEFLTAudioMixingApi api (%@) doesn't respond to @selector(resumeAudioMixingWithError:)",
          api);
      [channel setMessageHandler:^(id _Nullable message, FlutterReply callback) {
        FlutterError *error;
        NSNumber *output = [api resumeAudioMixingWithError:&error];
        callback(wrapResult(output, error));
      }];
    } else {
      [channel setMessageHandler:nil];
    }
  }
  {
    FlutterBasicMessageChannel *channel = [[FlutterBasicMessageChannel alloc]
           initWithName:@"dev.flutter.pigeon.AudioMixingApi.setAudioMixingSendVolume"
        binaryMessenger:binaryMessenger
                  codec:NEFLTAudioMixingApiGetCodec()];
    if (api) {
      NSCAssert([api respondsToSelector:@selector(setAudioMixingSendVolumeVolume:error:)],
                @"NEFLTAudioMixingApi api (%@) doesn't respond to "
                @"@selector(setAudioMixingSendVolumeVolume:error:)",
                api);
      [channel setMessageHandler:^(id _Nullable message, FlutterReply callback) {
        NSArray *args = message;
        NSNumber *arg_volume = GetNullableObjectAtIndex(args, 0);
        FlutterError *error;
        NSNumber *output = [api setAudioMixingSendVolumeVolume:arg_volume error:&error];
        callback(wrapResult(output, error));
      }];
    } else {
      [channel setMessageHandler:nil];
    }
  }
  {
    FlutterBasicMessageChannel *channel = [[FlutterBasicMessageChannel alloc]
           initWithName:@"dev.flutter.pigeon.AudioMixingApi.getAudioMixingSendVolume"
        binaryMessenger:binaryMessenger
                  codec:NEFLTAudioMixingApiGetCodec()];
    if (api) {
      NSCAssert([api respondsToSelector:@selector(getAudioMixingSendVolumeWithError:)],
                @"NEFLTAudioMixingApi api (%@) doesn't respond to "
                @"@selector(getAudioMixingSendVolumeWithError:)",
                api);
      [channel setMessageHandler:^(id _Nullable message, FlutterReply callback) {
        FlutterError *error;
        NSNumber *output = [api getAudioMixingSendVolumeWithError:&error];
        callback(wrapResult(output, error));
      }];
    } else {
      [channel setMessageHandler:nil];
    }
  }
  {
    FlutterBasicMessageChannel *channel = [[FlutterBasicMessageChannel alloc]
           initWithName:@"dev.flutter.pigeon.AudioMixingApi.setAudioMixingPlaybackVolume"
        binaryMessenger:binaryMessenger
                  codec:NEFLTAudioMixingApiGetCodec()];
    if (api) {
      NSCAssert([api respondsToSelector:@selector(setAudioMixingPlaybackVolumeVolume:error:)],
                @"NEFLTAudioMixingApi api (%@) doesn't respond to "
                @"@selector(setAudioMixingPlaybackVolumeVolume:error:)",
                api);
      [channel setMessageHandler:^(id _Nullable message, FlutterReply callback) {
        NSArray *args = message;
        NSNumber *arg_volume = GetNullableObjectAtIndex(args, 0);
        FlutterError *error;
        NSNumber *output = [api setAudioMixingPlaybackVolumeVolume:arg_volume error:&error];
        callback(wrapResult(output, error));
      }];
    } else {
      [channel setMessageHandler:nil];
    }
  }
  {
    FlutterBasicMessageChannel *channel = [[FlutterBasicMessageChannel alloc]
           initWithName:@"dev.flutter.pigeon.AudioMixingApi.getAudioMixingPlaybackVolume"
        binaryMessenger:binaryMessenger
                  codec:NEFLTAudioMixingApiGetCodec()];
    if (api) {
      NSCAssert([api respondsToSelector:@selector(getAudioMixingPlaybackVolumeWithError:)],
                @"NEFLTAudioMixingApi api (%@) doesn't respond to "
                @"@selector(getAudioMixingPlaybackVolumeWithError:)",
                api);
      [channel setMessageHandler:^(id _Nullable message, FlutterReply callback) {
        FlutterError *error;
        NSNumber *output = [api getAudioMixingPlaybackVolumeWithError:&error];
        callback(wrapResult(output, error));
      }];
    } else {
      [channel setMessageHandler:nil];
    }
  }
  {
    FlutterBasicMessageChannel *channel = [[FlutterBasicMessageChannel alloc]
           initWithName:@"dev.flutter.pigeon.AudioMixingApi.getAudioMixingDuration"
        binaryMessenger:binaryMessenger
                  codec:NEFLTAudioMixingApiGetCodec()];
    if (api) {
      NSCAssert([api respondsToSelector:@selector(getAudioMixingDurationWithError:)],
                @"NEFLTAudioMixingApi api (%@) doesn't respond to "
                @"@selector(getAudioMixingDurationWithError:)",
                api);
      [channel setMessageHandler:^(id _Nullable message, FlutterReply callback) {
        FlutterError *error;
        NSNumber *output = [api getAudioMixingDurationWithError:&error];
        callback(wrapResult(output, error));
      }];
    } else {
      [channel setMessageHandler:nil];
    }
  }
  {
    FlutterBasicMessageChannel *channel = [[FlutterBasicMessageChannel alloc]
           initWithName:@"dev.flutter.pigeon.AudioMixingApi.getAudioMixingCurrentPosition"
        binaryMessenger:binaryMessenger
                  codec:NEFLTAudioMixingApiGetCodec()];
    if (api) {
      NSCAssert([api respondsToSelector:@selector(getAudioMixingCurrentPositionWithError:)],
                @"NEFLTAudioMixingApi api (%@) doesn't respond to "
                @"@selector(getAudioMixingCurrentPositionWithError:)",
                api);
      [channel setMessageHandler:^(id _Nullable message, FlutterReply callback) {
        FlutterError *error;
        NSNumber *output = [api getAudioMixingCurrentPositionWithError:&error];
        callback(wrapResult(output, error));
      }];
    } else {
      [channel setMessageHandler:nil];
    }
  }
  {
    FlutterBasicMessageChannel *channel = [[FlutterBasicMessageChannel alloc]
           initWithName:@"dev.flutter.pigeon.AudioMixingApi.setAudioMixingPosition"
        binaryMessenger:binaryMessenger
                  codec:NEFLTAudioMixingApiGetCodec()];
    if (api) {
      NSCAssert([api respondsToSelector:@selector(setAudioMixingPositionPosition:error:)],
                @"NEFLTAudioMixingApi api (%@) doesn't respond to "
                @"@selector(setAudioMixingPositionPosition:error:)",
                api);
      [channel setMessageHandler:^(id _Nullable message, FlutterReply callback) {
        NSArray *args = message;
        NSNumber *arg_position = GetNullableObjectAtIndex(args, 0);
        FlutterError *error;
        NSNumber *output = [api setAudioMixingPositionPosition:arg_position error:&error];
        callback(wrapResult(output, error));
      }];
    } else {
      [channel setMessageHandler:nil];
    }
  }
  {
    FlutterBasicMessageChannel *channel = [[FlutterBasicMessageChannel alloc]
           initWithName:@"dev.flutter.pigeon.AudioMixingApi.setAudioMixingPitch"
        binaryMessenger:binaryMessenger
                  codec:NEFLTAudioMixingApiGetCodec()];
    if (api) {
      NSCAssert([api respondsToSelector:@selector(setAudioMixingPitchPitch:error:)],
                @"NEFLTAudioMixingApi api (%@) doesn't respond to "
                @"@selector(setAudioMixingPitchPitch:error:)",
                api);
      [channel setMessageHandler:^(id _Nullable message, FlutterReply callback) {
        NSArray *args = message;
        NSNumber *arg_pitch = GetNullableObjectAtIndex(args, 0);
        FlutterError *error;
        NSNumber *output = [api setAudioMixingPitchPitch:arg_pitch error:&error];
        callback(wrapResult(output, error));
      }];
    } else {
      [channel setMessageHandler:nil];
    }
  }
  {
    FlutterBasicMessageChannel *channel = [[FlutterBasicMessageChannel alloc]
           initWithName:@"dev.flutter.pigeon.AudioMixingApi.getAudioMixingPitch"
        binaryMessenger:binaryMessenger
                  codec:NEFLTAudioMixingApiGetCodec()];
    if (api) {
      NSCAssert([api respondsToSelector:@selector(getAudioMixingPitchWithError:)],
                @"NEFLTAudioMixingApi api (%@) doesn't respond to "
                @"@selector(getAudioMixingPitchWithError:)",
                api);
      [channel setMessageHandler:^(id _Nullable message, FlutterReply callback) {
        FlutterError *error;
        NSNumber *output = [api getAudioMixingPitchWithError:&error];
        callback(wrapResult(output, error));
      }];
    } else {
      [channel setMessageHandler:nil];
    }
  }
}
NSObject<FlutterMessageCodec> *NEFLTNERtcAudioMixingEventSinkGetCodec() {
  static FlutterStandardMessageCodec *sSharedObject = nil;
  sSharedObject = [FlutterStandardMessageCodec sharedInstance];
  return sSharedObject;
}

@interface NEFLTNERtcAudioMixingEventSink ()
@property(nonatomic, strong) NSObject<FlutterBinaryMessenger> *binaryMessenger;
@end

@implementation NEFLTNERtcAudioMixingEventSink

- (instancetype)initWithBinaryMessenger:(NSObject<FlutterBinaryMessenger> *)binaryMessenger {
  self = [super init];
  if (self) {
    _binaryMessenger = binaryMessenger;
  }
  return self;
}
- (void)onAudioMixingStateChangedReason:(NSNumber *)arg_reason
                             completion:(void (^)(FlutterError *_Nullable))completion {
  FlutterBasicMessageChannel *channel = [FlutterBasicMessageChannel
      messageChannelWithName:
          @"dev.flutter.pigeon.NERtcAudioMixingEventSink.onAudioMixingStateChanged"
             binaryMessenger:self.binaryMessenger
                       codec:NEFLTNERtcAudioMixingEventSinkGetCodec()];
  [channel sendMessage:@[ arg_reason ?: [NSNull null] ]
                 reply:^(id reply) {
                   completion(nil);
                 }];
}
- (void)onAudioMixingTimestampUpdateTimestampMs:(NSNumber *)arg_timestampMs
                                     completion:(void (^)(FlutterError *_Nullable))completion {
  FlutterBasicMessageChannel *channel = [FlutterBasicMessageChannel
      messageChannelWithName:
          @"dev.flutter.pigeon.NERtcAudioMixingEventSink.onAudioMixingTimestampUpdate"
             binaryMessenger:self.binaryMessenger
                       codec:NEFLTNERtcAudioMixingEventSinkGetCodec()];
  [channel sendMessage:@[ arg_timestampMs ?: [NSNull null] ]
                 reply:^(id reply) {
                   completion(nil);
                 }];
}
@end

@interface NEFLTAudioEffectApiCodecReader : FlutterStandardReader
@end
@implementation NEFLTAudioEffectApiCodecReader
- (nullable id)readValueOfType:(UInt8)type {
  switch (type) {
    case 128:
      return [NEFLTPlayEffectRequest fromList:[self readValue]];
    default:
      return [super readValueOfType:type];
  }
}
@end

@interface NEFLTAudioEffectApiCodecWriter : FlutterStandardWriter
@end
@implementation NEFLTAudioEffectApiCodecWriter
- (void)writeValue:(id)value {
  if ([value isKindOfClass:[NEFLTPlayEffectRequest class]]) {
    [self writeByte:128];
    [self writeValue:[value toList]];
  } else {
    [super writeValue:value];
  }
}
@end

@interface NEFLTAudioEffectApiCodecReaderWriter : FlutterStandardReaderWriter
@end
@implementation NEFLTAudioEffectApiCodecReaderWriter
- (FlutterStandardWriter *)writerWithData:(NSMutableData *)data {
  return [[NEFLTAudioEffectApiCodecWriter alloc] initWithData:data];
}
- (FlutterStandardReader *)readerWithData:(NSData *)data {
  return [[NEFLTAudioEffectApiCodecReader alloc] initWithData:data];
}
@end

NSObject<FlutterMessageCodec> *NEFLTAudioEffectApiGetCodec() {
  static FlutterStandardMessageCodec *sSharedObject = nil;
  static dispatch_once_t sPred = 0;
  dispatch_once(&sPred, ^{
    NEFLTAudioEffectApiCodecReaderWriter *readerWriter =
        [[NEFLTAudioEffectApiCodecReaderWriter alloc] init];
    sSharedObject = [FlutterStandardMessageCodec codecWithReaderWriter:readerWriter];
  });
  return sSharedObject;
}

void NEFLTAudioEffectApiSetup(id<FlutterBinaryMessenger> binaryMessenger,
                              NSObject<NEFLTAudioEffectApi> *api) {
  {
    FlutterBasicMessageChannel *channel = [[FlutterBasicMessageChannel alloc]
           initWithName:@"dev.flutter.pigeon.AudioEffectApi.playEffect"
        binaryMessenger:binaryMessenger
                  codec:NEFLTAudioEffectApiGetCodec()];
    if (api) {
      NSCAssert(
          [api respondsToSelector:@selector(playEffectRequest:error:)],
          @"NEFLTAudioEffectApi api (%@) doesn't respond to @selector(playEffectRequest:error:)",
          api);
      [channel setMessageHandler:^(id _Nullable message, FlutterReply callback) {
        NSArray *args = message;
        NEFLTPlayEffectRequest *arg_request = GetNullableObjectAtIndex(args, 0);
        FlutterError *error;
        NSNumber *output = [api playEffectRequest:arg_request error:&error];
        callback(wrapResult(output, error));
      }];
    } else {
      [channel setMessageHandler:nil];
    }
  }
  {
    FlutterBasicMessageChannel *channel = [[FlutterBasicMessageChannel alloc]
           initWithName:@"dev.flutter.pigeon.AudioEffectApi.stopEffect"
        binaryMessenger:binaryMessenger
                  codec:NEFLTAudioEffectApiGetCodec()];
    if (api) {
      NSCAssert(
          [api respondsToSelector:@selector(stopEffectEffectId:error:)],
          @"NEFLTAudioEffectApi api (%@) doesn't respond to @selector(stopEffectEffectId:error:)",
          api);
      [channel setMessageHandler:^(id _Nullable message, FlutterReply callback) {
        NSArray *args = message;
        NSNumber *arg_effectId = GetNullableObjectAtIndex(args, 0);
        FlutterError *error;
        NSNumber *output = [api stopEffectEffectId:arg_effectId error:&error];
        callback(wrapResult(output, error));
      }];
    } else {
      [channel setMessageHandler:nil];
    }
  }
  {
    FlutterBasicMessageChannel *channel = [[FlutterBasicMessageChannel alloc]
           initWithName:@"dev.flutter.pigeon.AudioEffectApi.stopAllEffects"
        binaryMessenger:binaryMessenger
                  codec:NEFLTAudioEffectApiGetCodec()];
    if (api) {
      NSCAssert(
          [api respondsToSelector:@selector(stopAllEffectsWithError:)],
          @"NEFLTAudioEffectApi api (%@) doesn't respond to @selector(stopAllEffectsWithError:)",
          api);
      [channel setMessageHandler:^(id _Nullable message, FlutterReply callback) {
        FlutterError *error;
        NSNumber *output = [api stopAllEffectsWithError:&error];
        callback(wrapResult(output, error));
      }];
    } else {
      [channel setMessageHandler:nil];
    }
  }
  {
    FlutterBasicMessageChannel *channel = [[FlutterBasicMessageChannel alloc]
           initWithName:@"dev.flutter.pigeon.AudioEffectApi.pauseEffect"
        binaryMessenger:binaryMessenger
                  codec:NEFLTAudioEffectApiGetCodec()];
    if (api) {
      NSCAssert(
          [api respondsToSelector:@selector(pauseEffectEffectId:error:)],
          @"NEFLTAudioEffectApi api (%@) doesn't respond to @selector(pauseEffectEffectId:error:)",
          api);
      [channel setMessageHandler:^(id _Nullable message, FlutterReply callback) {
        NSArray *args = message;
        NSNumber *arg_effectId = GetNullableObjectAtIndex(args, 0);
        FlutterError *error;
        NSNumber *output = [api pauseEffectEffectId:arg_effectId error:&error];
        callback(wrapResult(output, error));
      }];
    } else {
      [channel setMessageHandler:nil];
    }
  }
  {
    FlutterBasicMessageChannel *channel = [[FlutterBasicMessageChannel alloc]
           initWithName:@"dev.flutter.pigeon.AudioEffectApi.resumeEffect"
        binaryMessenger:binaryMessenger
                  codec:NEFLTAudioEffectApiGetCodec()];
    if (api) {
      NSCAssert(
          [api respondsToSelector:@selector(resumeEffectEffectId:error:)],
          @"NEFLTAudioEffectApi api (%@) doesn't respond to @selector(resumeEffectEffectId:error:)",
          api);
      [channel setMessageHandler:^(id _Nullable message, FlutterReply callback) {
        NSArray *args = message;
        NSNumber *arg_effectId = GetNullableObjectAtIndex(args, 0);
        FlutterError *error;
        NSNumber *output = [api resumeEffectEffectId:arg_effectId error:&error];
        callback(wrapResult(output, error));
      }];
    } else {
      [channel setMessageHandler:nil];
    }
  }
  {
    FlutterBasicMessageChannel *channel = [[FlutterBasicMessageChannel alloc]
           initWithName:@"dev.flutter.pigeon.AudioEffectApi.pauseAllEffects"
        binaryMessenger:binaryMessenger
                  codec:NEFLTAudioEffectApiGetCodec()];
    if (api) {
      NSCAssert(
          [api respondsToSelector:@selector(pauseAllEffectsWithError:)],
          @"NEFLTAudioEffectApi api (%@) doesn't respond to @selector(pauseAllEffectsWithError:)",
          api);
      [channel setMessageHandler:^(id _Nullable message, FlutterReply callback) {
        FlutterError *error;
        NSNumber *output = [api pauseAllEffectsWithError:&error];
        callback(wrapResult(output, error));
      }];
    } else {
      [channel setMessageHandler:nil];
    }
  }
  {
    FlutterBasicMessageChannel *channel = [[FlutterBasicMessageChannel alloc]
           initWithName:@"dev.flutter.pigeon.AudioEffectApi.resumeAllEffects"
        binaryMessenger:binaryMessenger
                  codec:NEFLTAudioEffectApiGetCodec()];
    if (api) {
      NSCAssert(
          [api respondsToSelector:@selector(resumeAllEffectsWithError:)],
          @"NEFLTAudioEffectApi api (%@) doesn't respond to @selector(resumeAllEffectsWithError:)",
          api);
      [channel setMessageHandler:^(id _Nullable message, FlutterReply callback) {
        FlutterError *error;
        NSNumber *output = [api resumeAllEffectsWithError:&error];
        callback(wrapResult(output, error));
      }];
    } else {
      [channel setMessageHandler:nil];
    }
  }
  {
    FlutterBasicMessageChannel *channel = [[FlutterBasicMessageChannel alloc]
           initWithName:@"dev.flutter.pigeon.AudioEffectApi.setEffectSendVolume"
        binaryMessenger:binaryMessenger
                  codec:NEFLTAudioEffectApiGetCodec()];
    if (api) {
      NSCAssert([api respondsToSelector:@selector(setEffectSendVolumeEffectId:volume:error:)],
                @"NEFLTAudioEffectApi api (%@) doesn't respond to "
                @"@selector(setEffectSendVolumeEffectId:volume:error:)",
                api);
      [channel setMessageHandler:^(id _Nullable message, FlutterReply callback) {
        NSArray *args = message;
        NSNumber *arg_effectId = GetNullableObjectAtIndex(args, 0);
        NSNumber *arg_volume = GetNullableObjectAtIndex(args, 1);
        FlutterError *error;
        NSNumber *output = [api setEffectSendVolumeEffectId:arg_effectId
                                                     volume:arg_volume
                                                      error:&error];
        callback(wrapResult(output, error));
      }];
    } else {
      [channel setMessageHandler:nil];
    }
  }
  {
    FlutterBasicMessageChannel *channel = [[FlutterBasicMessageChannel alloc]
           initWithName:@"dev.flutter.pigeon.AudioEffectApi.getEffectSendVolume"
        binaryMessenger:binaryMessenger
                  codec:NEFLTAudioEffectApiGetCodec()];
    if (api) {
      NSCAssert([api respondsToSelector:@selector(getEffectSendVolumeEffectId:error:)],
                @"NEFLTAudioEffectApi api (%@) doesn't respond to "
                @"@selector(getEffectSendVolumeEffectId:error:)",
                api);
      [channel setMessageHandler:^(id _Nullable message, FlutterReply callback) {
        NSArray *args = message;
        NSNumber *arg_effectId = GetNullableObjectAtIndex(args, 0);
        FlutterError *error;
        NSNumber *output = [api getEffectSendVolumeEffectId:arg_effectId error:&error];
        callback(wrapResult(output, error));
      }];
    } else {
      [channel setMessageHandler:nil];
    }
  }
  {
    FlutterBasicMessageChannel *channel = [[FlutterBasicMessageChannel alloc]
           initWithName:@"dev.flutter.pigeon.AudioEffectApi.setEffectPlaybackVolume"
        binaryMessenger:binaryMessenger
                  codec:NEFLTAudioEffectApiGetCodec()];
    if (api) {
      NSCAssert([api respondsToSelector:@selector(setEffectPlaybackVolumeEffectId:volume:error:)],
                @"NEFLTAudioEffectApi api (%@) doesn't respond to "
                @"@selector(setEffectPlaybackVolumeEffectId:volume:error:)",
                api);
      [channel setMessageHandler:^(id _Nullable message, FlutterReply callback) {
        NSArray *args = message;
        NSNumber *arg_effectId = GetNullableObjectAtIndex(args, 0);
        NSNumber *arg_volume = GetNullableObjectAtIndex(args, 1);
        FlutterError *error;
        NSNumber *output = [api setEffectPlaybackVolumeEffectId:arg_effectId
                                                         volume:arg_volume
                                                          error:&error];
        callback(wrapResult(output, error));
      }];
    } else {
      [channel setMessageHandler:nil];
    }
  }
  {
    FlutterBasicMessageChannel *channel = [[FlutterBasicMessageChannel alloc]
           initWithName:@"dev.flutter.pigeon.AudioEffectApi.getEffectPlaybackVolume"
        binaryMessenger:binaryMessenger
                  codec:NEFLTAudioEffectApiGetCodec()];
    if (api) {
      NSCAssert([api respondsToSelector:@selector(getEffectPlaybackVolumeEffectId:error:)],
                @"NEFLTAudioEffectApi api (%@) doesn't respond to "
                @"@selector(getEffectPlaybackVolumeEffectId:error:)",
                api);
      [channel setMessageHandler:^(id _Nullable message, FlutterReply callback) {
        NSArray *args = message;
        NSNumber *arg_effectId = GetNullableObjectAtIndex(args, 0);
        FlutterError *error;
        NSNumber *output = [api getEffectPlaybackVolumeEffectId:arg_effectId error:&error];
        callback(wrapResult(output, error));
      }];
    } else {
      [channel setMessageHandler:nil];
    }
  }
  {
    FlutterBasicMessageChannel *channel = [[FlutterBasicMessageChannel alloc]
           initWithName:@"dev.flutter.pigeon.AudioEffectApi.getEffectDuration"
        binaryMessenger:binaryMessenger
                  codec:NEFLTAudioEffectApiGetCodec()];
    if (api) {
      NSCAssert([api respondsToSelector:@selector(getEffectDurationEffectId:error:)],
                @"NEFLTAudioEffectApi api (%@) doesn't respond to "
                @"@selector(getEffectDurationEffectId:error:)",
                api);
      [channel setMessageHandler:^(id _Nullable message, FlutterReply callback) {
        NSArray *args = message;
        NSNumber *arg_effectId = GetNullableObjectAtIndex(args, 0);
        FlutterError *error;
        NSNumber *output = [api getEffectDurationEffectId:arg_effectId error:&error];
        callback(wrapResult(output, error));
      }];
    } else {
      [channel setMessageHandler:nil];
    }
  }
  {
    FlutterBasicMessageChannel *channel = [[FlutterBasicMessageChannel alloc]
           initWithName:@"dev.flutter.pigeon.AudioEffectApi.getEffectCurrentPosition"
        binaryMessenger:binaryMessenger
                  codec:NEFLTAudioEffectApiGetCodec()];
    if (api) {
      NSCAssert([api respondsToSelector:@selector(getEffectCurrentPositionEffectId:error:)],
                @"NEFLTAudioEffectApi api (%@) doesn't respond to "
                @"@selector(getEffectCurrentPositionEffectId:error:)",
                api);
      [channel setMessageHandler:^(id _Nullable message, FlutterReply callback) {
        NSArray *args = message;
        NSNumber *arg_effectId = GetNullableObjectAtIndex(args, 0);
        FlutterError *error;
        NSNumber *output = [api getEffectCurrentPositionEffectId:arg_effectId error:&error];
        callback(wrapResult(output, error));
      }];
    } else {
      [channel setMessageHandler:nil];
    }
  }
  {
    FlutterBasicMessageChannel *channel = [[FlutterBasicMessageChannel alloc]
           initWithName:@"dev.flutter.pigeon.AudioEffectApi.setEffectPitch"
        binaryMessenger:binaryMessenger
                  codec:NEFLTAudioEffectApiGetCodec()];
    if (api) {
      NSCAssert([api respondsToSelector:@selector(setEffectPitchEffectId:pitch:error:)],
                @"NEFLTAudioEffectApi api (%@) doesn't respond to "
                @"@selector(setEffectPitchEffectId:pitch:error:)",
                api);
      [channel setMessageHandler:^(id _Nullable message, FlutterReply callback) {
        NSArray *args = message;
        NSNumber *arg_effectId = GetNullableObjectAtIndex(args, 0);
        NSNumber *arg_pitch = GetNullableObjectAtIndex(args, 1);
        FlutterError *error;
        NSNumber *output = [api setEffectPitchEffectId:arg_effectId pitch:arg_pitch error:&error];
        callback(wrapResult(output, error));
      }];
    } else {
      [channel setMessageHandler:nil];
    }
  }
  {
    FlutterBasicMessageChannel *channel = [[FlutterBasicMessageChannel alloc]
           initWithName:@"dev.flutter.pigeon.AudioEffectApi.getEffectPitch"
        binaryMessenger:binaryMessenger
                  codec:NEFLTAudioEffectApiGetCodec()];
    if (api) {
      NSCAssert([api respondsToSelector:@selector(getEffectPitchEffectId:error:)],
                @"NEFLTAudioEffectApi api (%@) doesn't respond to "
                @"@selector(getEffectPitchEffectId:error:)",
                api);
      [channel setMessageHandler:^(id _Nullable message, FlutterReply callback) {
        NSArray *args = message;
        NSNumber *arg_effectId = GetNullableObjectAtIndex(args, 0);
        FlutterError *error;
        NSNumber *output = [api getEffectPitchEffectId:arg_effectId error:&error];
        callback(wrapResult(output, error));
      }];
    } else {
      [channel setMessageHandler:nil];
    }
  }
  {
    FlutterBasicMessageChannel *channel = [[FlutterBasicMessageChannel alloc]
           initWithName:@"dev.flutter.pigeon.AudioEffectApi.setEffectPosition"
        binaryMessenger:binaryMessenger
                  codec:NEFLTAudioEffectApiGetCodec()];
    if (api) {
      NSCAssert([api respondsToSelector:@selector(setEffectPositionEffectId:position:error:)],
                @"NEFLTAudioEffectApi api (%@) doesn't respond to "
                @"@selector(setEffectPositionEffectId:position:error:)",
                api);
      [channel setMessageHandler:^(id _Nullable message, FlutterReply callback) {
        NSArray *args = message;
        NSNumber *arg_effectId = GetNullableObjectAtIndex(args, 0);
        NSNumber *arg_position = GetNullableObjectAtIndex(args, 1);
        FlutterError *error;
        NSNumber *output = [api setEffectPositionEffectId:arg_effectId
                                                 position:arg_position
                                                    error:&error];
        callback(wrapResult(output, error));
      }];
    } else {
      [channel setMessageHandler:nil];
    }
  }
}
NSObject<FlutterMessageCodec> *NEFLTNERtcAudioEffectEventSinkGetCodec() {
  static FlutterStandardMessageCodec *sSharedObject = nil;
  sSharedObject = [FlutterStandardMessageCodec sharedInstance];
  return sSharedObject;
}

@interface NEFLTNERtcAudioEffectEventSink ()
@property(nonatomic, strong) NSObject<FlutterBinaryMessenger> *binaryMessenger;
@end

@implementation NEFLTNERtcAudioEffectEventSink

- (instancetype)initWithBinaryMessenger:(NSObject<FlutterBinaryMessenger> *)binaryMessenger {
  self = [super init];
  if (self) {
    _binaryMessenger = binaryMessenger;
  }
  return self;
}
- (void)onAudioEffectFinishedEffectId:(NSNumber *)arg_effectId
                           completion:(void (^)(FlutterError *_Nullable))completion {
  FlutterBasicMessageChannel *channel = [FlutterBasicMessageChannel
      messageChannelWithName:@"dev.flutter.pigeon.NERtcAudioEffectEventSink.onAudioEffectFinished"
             binaryMessenger:self.binaryMessenger
                       codec:NEFLTNERtcAudioEffectEventSinkGetCodec()];
  [channel sendMessage:@[ arg_effectId ?: [NSNull null] ]
                 reply:^(id reply) {
                   completion(nil);
                 }];
}
- (void)onAudioEffectTimestampUpdateId:(NSNumber *)arg_id
                           timestampMs:(NSNumber *)arg_timestampMs
                            completion:(void (^)(FlutterError *_Nullable))completion {
  FlutterBasicMessageChannel *channel = [FlutterBasicMessageChannel
      messageChannelWithName:
          @"dev.flutter.pigeon.NERtcAudioEffectEventSink.onAudioEffectTimestampUpdate"
             binaryMessenger:self.binaryMessenger
                       codec:NEFLTNERtcAudioEffectEventSinkGetCodec()];
  [channel sendMessage:@[ arg_id ?: [NSNull null], arg_timestampMs ?: [NSNull null] ]
                 reply:^(id reply) {
                   completion(nil);
                 }];
}
@end

@interface NEFLTNERtcStatsEventSinkCodecReader : FlutterStandardReader
@end
@implementation NEFLTNERtcStatsEventSinkCodecReader
- (nullable id)readValueOfType:(UInt8)type {
  switch (type) {
    case 128:
      return [NEFLTAddOrUpdateLiveStreamTaskRequest fromList:[self readValue]];
    case 129:
      return [NEFLTAdjustUserPlaybackSignalVolumeRequest fromList:[self readValue]];
    case 130:
      return [NEFLTAudioRecordingConfigurationRequest fromList:[self readValue]];
    case 131:
      return [NEFLTAudioVolumeInfo fromList:[self readValue]];
    case 132:
      return [NEFLTCGPoint fromList:[self readValue]];
    case 133:
      return [NEFLTCreateEngineRequest fromList:[self readValue]];
    case 134:
      return [NEFLTDeleteLiveStreamTaskRequest fromList:[self readValue]];
    case 135:
      return [NEFLTEnableAudioVolumeIndicationRequest fromList:[self readValue]];
    case 136:
      return [NEFLTEnableEncryptionRequest fromList:[self readValue]];
    case 137:
      return [NEFLTEnableLocalVideoRequest fromList:[self readValue]];
    case 138:
      return [NEFLTEnableVirtualBackgroundRequest fromList:[self readValue]];
    case 139:
      return [NEFLTFirstVideoDataReceivedEvent fromList:[self readValue]];
    case 140:
      return [NEFLTFirstVideoFrameDecodedEvent fromList:[self readValue]];
    case 141:
      return [NEFLTJoinChannelOptions fromList:[self readValue]];
    case 142:
      return [NEFLTJoinChannelRequest fromList:[self readValue]];
    case 143:
      return [NEFLTNERtcLastmileProbeOneWayResult fromList:[self readValue]];
    case 144:
      return [NEFLTNERtcLastmileProbeResult fromList:[self readValue]];
    case 145:
      return [NEFLTNERtcUserJoinExtraInfo fromList:[self readValue]];
    case 146:
      return [NEFLTNERtcUserLeaveExtraInfo fromList:[self readValue]];
    case 147:
      return [NEFLTNERtcVersion fromList:[self readValue]];
    case 148:
      return [NEFLTPlayEffectRequest fromList:[self readValue]];
    case 149:
      return [NEFLTRemoteAudioVolumeIndicationEvent fromList:[self readValue]];
    case 150:
      return [NEFLTReportCustomEventRequest fromList:[self readValue]];
    case 151:
      return [NEFLTRtcServerAddresses fromList:[self readValue]];
    case 152:
      return [NEFLTSendSEIMsgRequest fromList:[self readValue]];
    case 153:
      return [NEFLTSetAudioProfileRequest fromList:[self readValue]];
    case 154:
      return [NEFLTSetAudioSubscribeOnlyByRequest fromList:[self readValue]];
    case 155:
      return [NEFLTSetCameraCaptureConfigRequest fromList:[self readValue]];
    case 156:
      return [NEFLTSetCameraPositionRequest fromList:[self readValue]];
    case 157:
      return [NEFLTSetLocalMediaPriorityRequest fromList:[self readValue]];
    case 158:
      return [NEFLTSetLocalVideoConfigRequest fromList:[self readValue]];
    case 159:
      return [NEFLTSetLocalVideoWatermarkConfigsRequest fromList:[self readValue]];
    case 160:
      return [NEFLTSetLocalVoiceEqualizationRequest fromList:[self readValue]];
    case 161:
      return [NEFLTSetLocalVoiceReverbParamRequest fromList:[self readValue]];
    case 162:
      return [NEFLTSetRemoteHighPriorityAudioStreamRequest fromList:[self readValue]];
    case 163:
      return [NEFLTSetVideoCorrectionConfigRequest fromList:[self readValue]];
    case 164:
      return [NEFLTStartAudioMixingRequest fromList:[self readValue]];
    case 165:
      return [NEFLTStartAudioRecordingRequest fromList:[self readValue]];
    case 166:
      return [NEFLTStartLastmileProbeTestRequest fromList:[self readValue]];
    case 167:
      return [NEFLTStartOrUpdateChannelMediaRelayRequest fromList:[self readValue]];
    case 168:
      return [NEFLTStartScreenCaptureRequest fromList:[self readValue]];
    case 169:
      return [NEFLTStartorStopVideoPreviewRequest fromList:[self readValue]];
    case 170:
      return [NEFLTSubscribeRemoteAudioRequest fromList:[self readValue]];
    case 171:
      return [NEFLTSubscribeRemoteSubStreamAudioRequest fromList:[self readValue]];
    case 172:
      return [NEFLTSubscribeRemoteSubStreamVideoRequest fromList:[self readValue]];
    case 173:
      return [NEFLTSubscribeRemoteVideoStreamRequest fromList:[self readValue]];
    case 174:
      return [NEFLTSwitchChannelRequest fromList:[self readValue]];
    case 175:
      return [NEFLTUserJoinedEvent fromList:[self readValue]];
    case 176:
      return [NEFLTUserLeaveEvent fromList:[self readValue]];
    case 177:
      return [NEFLTUserVideoMuteEvent fromList:[self readValue]];
    case 178:
      return [NEFLTVideoWatermarkConfig fromList:[self readValue]];
    case 179:
      return [NEFLTVideoWatermarkImageConfig fromList:[self readValue]];
    case 180:
      return [NEFLTVideoWatermarkTextConfig fromList:[self readValue]];
    case 181:
      return [NEFLTVideoWatermarkTimestampConfig fromList:[self readValue]];
    case 182:
      return [NEFLTVirtualBackgroundSourceEnabledEvent fromList:[self readValue]];
    default:
      return [super readValueOfType:type];
  }
}
@end

@interface NEFLTNERtcStatsEventSinkCodecWriter : FlutterStandardWriter
@end
@implementation NEFLTNERtcStatsEventSinkCodecWriter
- (void)writeValue:(id)value {
  if ([value isKindOfClass:[NEFLTAddOrUpdateLiveStreamTaskRequest class]]) {
    [self writeByte:128];
    [self writeValue:[value toList]];
  } else if ([value isKindOfClass:[NEFLTAdjustUserPlaybackSignalVolumeRequest class]]) {
    [self writeByte:129];
    [self writeValue:[value toList]];
  } else if ([value isKindOfClass:[NEFLTAudioRecordingConfigurationRequest class]]) {
    [self writeByte:130];
    [self writeValue:[value toList]];
  } else if ([value isKindOfClass:[NEFLTAudioVolumeInfo class]]) {
    [self writeByte:131];
    [self writeValue:[value toList]];
  } else if ([value isKindOfClass:[NEFLTCGPoint class]]) {
    [self writeByte:132];
    [self writeValue:[value toList]];
  } else if ([value isKindOfClass:[NEFLTCreateEngineRequest class]]) {
    [self writeByte:133];
    [self writeValue:[value toList]];
  } else if ([value isKindOfClass:[NEFLTDeleteLiveStreamTaskRequest class]]) {
    [self writeByte:134];
    [self writeValue:[value toList]];
  } else if ([value isKindOfClass:[NEFLTEnableAudioVolumeIndicationRequest class]]) {
    [self writeByte:135];
    [self writeValue:[value toList]];
  } else if ([value isKindOfClass:[NEFLTEnableEncryptionRequest class]]) {
    [self writeByte:136];
    [self writeValue:[value toList]];
  } else if ([value isKindOfClass:[NEFLTEnableLocalVideoRequest class]]) {
    [self writeByte:137];
    [self writeValue:[value toList]];
  } else if ([value isKindOfClass:[NEFLTEnableVirtualBackgroundRequest class]]) {
    [self writeByte:138];
    [self writeValue:[value toList]];
  } else if ([value isKindOfClass:[NEFLTFirstVideoDataReceivedEvent class]]) {
    [self writeByte:139];
    [self writeValue:[value toList]];
  } else if ([value isKindOfClass:[NEFLTFirstVideoFrameDecodedEvent class]]) {
    [self writeByte:140];
    [self writeValue:[value toList]];
  } else if ([value isKindOfClass:[NEFLTJoinChannelOptions class]]) {
    [self writeByte:141];
    [self writeValue:[value toList]];
  } else if ([value isKindOfClass:[NEFLTJoinChannelRequest class]]) {
    [self writeByte:142];
    [self writeValue:[value toList]];
  } else if ([value isKindOfClass:[NEFLTNERtcLastmileProbeOneWayResult class]]) {
    [self writeByte:143];
    [self writeValue:[value toList]];
  } else if ([value isKindOfClass:[NEFLTNERtcLastmileProbeResult class]]) {
    [self writeByte:144];
    [self writeValue:[value toList]];
  } else if ([value isKindOfClass:[NEFLTNERtcUserJoinExtraInfo class]]) {
    [self writeByte:145];
    [self writeValue:[value toList]];
  } else if ([value isKindOfClass:[NEFLTNERtcUserLeaveExtraInfo class]]) {
    [self writeByte:146];
    [self writeValue:[value toList]];
  } else if ([value isKindOfClass:[NEFLTNERtcVersion class]]) {
    [self writeByte:147];
    [self writeValue:[value toList]];
  } else if ([value isKindOfClass:[NEFLTPlayEffectRequest class]]) {
    [self writeByte:148];
    [self writeValue:[value toList]];
  } else if ([value isKindOfClass:[NEFLTRemoteAudioVolumeIndicationEvent class]]) {
    [self writeByte:149];
    [self writeValue:[value toList]];
  } else if ([value isKindOfClass:[NEFLTReportCustomEventRequest class]]) {
    [self writeByte:150];
    [self writeValue:[value toList]];
  } else if ([value isKindOfClass:[NEFLTRtcServerAddresses class]]) {
    [self writeByte:151];
    [self writeValue:[value toList]];
  } else if ([value isKindOfClass:[NEFLTSendSEIMsgRequest class]]) {
    [self writeByte:152];
    [self writeValue:[value toList]];
  } else if ([value isKindOfClass:[NEFLTSetAudioProfileRequest class]]) {
    [self writeByte:153];
    [self writeValue:[value toList]];
  } else if ([value isKindOfClass:[NEFLTSetAudioSubscribeOnlyByRequest class]]) {
    [self writeByte:154];
    [self writeValue:[value toList]];
  } else if ([value isKindOfClass:[NEFLTSetCameraCaptureConfigRequest class]]) {
    [self writeByte:155];
    [self writeValue:[value toList]];
  } else if ([value isKindOfClass:[NEFLTSetCameraPositionRequest class]]) {
    [self writeByte:156];
    [self writeValue:[value toList]];
  } else if ([value isKindOfClass:[NEFLTSetLocalMediaPriorityRequest class]]) {
    [self writeByte:157];
    [self writeValue:[value toList]];
  } else if ([value isKindOfClass:[NEFLTSetLocalVideoConfigRequest class]]) {
    [self writeByte:158];
    [self writeValue:[value toList]];
  } else if ([value isKindOfClass:[NEFLTSetLocalVideoWatermarkConfigsRequest class]]) {
    [self writeByte:159];
    [self writeValue:[value toList]];
  } else if ([value isKindOfClass:[NEFLTSetLocalVoiceEqualizationRequest class]]) {
    [self writeByte:160];
    [self writeValue:[value toList]];
  } else if ([value isKindOfClass:[NEFLTSetLocalVoiceReverbParamRequest class]]) {
    [self writeByte:161];
    [self writeValue:[value toList]];
  } else if ([value isKindOfClass:[NEFLTSetRemoteHighPriorityAudioStreamRequest class]]) {
    [self writeByte:162];
    [self writeValue:[value toList]];
  } else if ([value isKindOfClass:[NEFLTSetVideoCorrectionConfigRequest class]]) {
    [self writeByte:163];
    [self writeValue:[value toList]];
  } else if ([value isKindOfClass:[NEFLTStartAudioMixingRequest class]]) {
    [self writeByte:164];
    [self writeValue:[value toList]];
  } else if ([value isKindOfClass:[NEFLTStartAudioRecordingRequest class]]) {
    [self writeByte:165];
    [self writeValue:[value toList]];
  } else if ([value isKindOfClass:[NEFLTStartLastmileProbeTestRequest class]]) {
    [self writeByte:166];
    [self writeValue:[value toList]];
  } else if ([value isKindOfClass:[NEFLTStartOrUpdateChannelMediaRelayRequest class]]) {
    [self writeByte:167];
    [self writeValue:[value toList]];
  } else if ([value isKindOfClass:[NEFLTStartScreenCaptureRequest class]]) {
    [self writeByte:168];
    [self writeValue:[value toList]];
  } else if ([value isKindOfClass:[NEFLTStartorStopVideoPreviewRequest class]]) {
    [self writeByte:169];
    [self writeValue:[value toList]];
  } else if ([value isKindOfClass:[NEFLTSubscribeRemoteAudioRequest class]]) {
    [self writeByte:170];
    [self writeValue:[value toList]];
  } else if ([value isKindOfClass:[NEFLTSubscribeRemoteSubStreamAudioRequest class]]) {
    [self writeByte:171];
    [self writeValue:[value toList]];
  } else if ([value isKindOfClass:[NEFLTSubscribeRemoteSubStreamVideoRequest class]]) {
    [self writeByte:172];
    [self writeValue:[value toList]];
  } else if ([value isKindOfClass:[NEFLTSubscribeRemoteVideoStreamRequest class]]) {
    [self writeByte:173];
    [self writeValue:[value toList]];
  } else if ([value isKindOfClass:[NEFLTSwitchChannelRequest class]]) {
    [self writeByte:174];
    [self writeValue:[value toList]];
  } else if ([value isKindOfClass:[NEFLTUserJoinedEvent class]]) {
    [self writeByte:175];
    [self writeValue:[value toList]];
  } else if ([value isKindOfClass:[NEFLTUserLeaveEvent class]]) {
    [self writeByte:176];
    [self writeValue:[value toList]];
  } else if ([value isKindOfClass:[NEFLTUserVideoMuteEvent class]]) {
    [self writeByte:177];
    [self writeValue:[value toList]];
  } else if ([value isKindOfClass:[NEFLTVideoWatermarkConfig class]]) {
    [self writeByte:178];
    [self writeValue:[value toList]];
  } else if ([value isKindOfClass:[NEFLTVideoWatermarkImageConfig class]]) {
    [self writeByte:179];
    [self writeValue:[value toList]];
  } else if ([value isKindOfClass:[NEFLTVideoWatermarkTextConfig class]]) {
    [self writeByte:180];
    [self writeValue:[value toList]];
  } else if ([value isKindOfClass:[NEFLTVideoWatermarkTimestampConfig class]]) {
    [self writeByte:181];
    [self writeValue:[value toList]];
  } else if ([value isKindOfClass:[NEFLTVirtualBackgroundSourceEnabledEvent class]]) {
    [self writeByte:182];
    [self writeValue:[value toList]];
  } else {
    [super writeValue:value];
  }
}
@end

@interface NEFLTNERtcStatsEventSinkCodecReaderWriter : FlutterStandardReaderWriter
@end
@implementation NEFLTNERtcStatsEventSinkCodecReaderWriter
- (FlutterStandardWriter *)writerWithData:(NSMutableData *)data {
  return [[NEFLTNERtcStatsEventSinkCodecWriter alloc] initWithData:data];
}
- (FlutterStandardReader *)readerWithData:(NSData *)data {
  return [[NEFLTNERtcStatsEventSinkCodecReader alloc] initWithData:data];
}
@end

NSObject<FlutterMessageCodec> *NEFLTNERtcStatsEventSinkGetCodec() {
  static FlutterStandardMessageCodec *sSharedObject = nil;
  static dispatch_once_t sPred = 0;
  dispatch_once(&sPred, ^{
    NEFLTNERtcStatsEventSinkCodecReaderWriter *readerWriter =
        [[NEFLTNERtcStatsEventSinkCodecReaderWriter alloc] init];
    sSharedObject = [FlutterStandardMessageCodec codecWithReaderWriter:readerWriter];
  });
  return sSharedObject;
}

@interface NEFLTNERtcStatsEventSink ()
@property(nonatomic, strong) NSObject<FlutterBinaryMessenger> *binaryMessenger;
@end

@implementation NEFLTNERtcStatsEventSink

- (instancetype)initWithBinaryMessenger:(NSObject<FlutterBinaryMessenger> *)binaryMessenger {
  self = [super init];
  if (self) {
    _binaryMessenger = binaryMessenger;
  }
  return self;
}
- (void)onRtcStatsArguments:(NSDictionary<id, id> *)arg_arguments
                 completion:(void (^)(FlutterError *_Nullable))completion {
  FlutterBasicMessageChannel *channel = [FlutterBasicMessageChannel
      messageChannelWithName:@"dev.flutter.pigeon.NERtcStatsEventSink.onRtcStats"
             binaryMessenger:self.binaryMessenger
                       codec:NEFLTNERtcStatsEventSinkGetCodec()];
  [channel sendMessage:@[ arg_arguments ?: [NSNull null] ]
                 reply:^(id reply) {
                   completion(nil);
                 }];
}
- (void)onLocalAudioStatsArguments:(NSDictionary<id, id> *)arg_arguments
                        completion:(void (^)(FlutterError *_Nullable))completion {
  FlutterBasicMessageChannel *channel = [FlutterBasicMessageChannel
      messageChannelWithName:@"dev.flutter.pigeon.NERtcStatsEventSink.onLocalAudioStats"
             binaryMessenger:self.binaryMessenger
                       codec:NEFLTNERtcStatsEventSinkGetCodec()];
  [channel sendMessage:@[ arg_arguments ?: [NSNull null] ]
                 reply:^(id reply) {
                   completion(nil);
                 }];
}
- (void)onRemoteAudioStatsArguments:(NSDictionary<id, id> *)arg_arguments
                         completion:(void (^)(FlutterError *_Nullable))completion {
  FlutterBasicMessageChannel *channel = [FlutterBasicMessageChannel
      messageChannelWithName:@"dev.flutter.pigeon.NERtcStatsEventSink.onRemoteAudioStats"
             binaryMessenger:self.binaryMessenger
                       codec:NEFLTNERtcStatsEventSinkGetCodec()];
  [channel sendMessage:@[ arg_arguments ?: [NSNull null] ]
                 reply:^(id reply) {
                   completion(nil);
                 }];
}
- (void)onLocalVideoStatsArguments:(NSDictionary<id, id> *)arg_arguments
                        completion:(void (^)(FlutterError *_Nullable))completion {
  FlutterBasicMessageChannel *channel = [FlutterBasicMessageChannel
      messageChannelWithName:@"dev.flutter.pigeon.NERtcStatsEventSink.onLocalVideoStats"
             binaryMessenger:self.binaryMessenger
                       codec:NEFLTNERtcStatsEventSinkGetCodec()];
  [channel sendMessage:@[ arg_arguments ?: [NSNull null] ]
                 reply:^(id reply) {
                   completion(nil);
                 }];
}
- (void)onRemoteVideoStatsArguments:(NSDictionary<id, id> *)arg_arguments
                         completion:(void (^)(FlutterError *_Nullable))completion {
  FlutterBasicMessageChannel *channel = [FlutterBasicMessageChannel
      messageChannelWithName:@"dev.flutter.pigeon.NERtcStatsEventSink.onRemoteVideoStats"
             binaryMessenger:self.binaryMessenger
                       codec:NEFLTNERtcStatsEventSinkGetCodec()];
  [channel sendMessage:@[ arg_arguments ?: [NSNull null] ]
                 reply:^(id reply) {
                   completion(nil);
                 }];
}
- (void)onNetworkQualityArguments:(NSDictionary<id, id> *)arg_arguments
                       completion:(void (^)(FlutterError *_Nullable))completion {
  FlutterBasicMessageChannel *channel = [FlutterBasicMessageChannel
      messageChannelWithName:@"dev.flutter.pigeon.NERtcStatsEventSink.onNetworkQuality"
             binaryMessenger:self.binaryMessenger
                       codec:NEFLTNERtcStatsEventSinkGetCodec()];
  [channel sendMessage:@[ arg_arguments ?: [NSNull null] ]
                 reply:^(id reply) {
                   completion(nil);
                 }];
}
@end

NSObject<FlutterMessageCodec> *NEFLTNERtcLiveStreamEventSinkGetCodec() {
  static FlutterStandardMessageCodec *sSharedObject = nil;
  sSharedObject = [FlutterStandardMessageCodec sharedInstance];
  return sSharedObject;
}

@interface NEFLTNERtcLiveStreamEventSink ()
@property(nonatomic, strong) NSObject<FlutterBinaryMessenger> *binaryMessenger;
@end

@implementation NEFLTNERtcLiveStreamEventSink

- (instancetype)initWithBinaryMessenger:(NSObject<FlutterBinaryMessenger> *)binaryMessenger {
  self = [super init];
  if (self) {
    _binaryMessenger = binaryMessenger;
  }
  return self;
}
- (void)onUpdateLiveStreamTaskTaskId:(NSString *)arg_taskId
                             errCode:(NSNumber *)arg_errCode
                          completion:(void (^)(FlutterError *_Nullable))completion {
  FlutterBasicMessageChannel *channel = [FlutterBasicMessageChannel
      messageChannelWithName:@"dev.flutter.pigeon.NERtcLiveStreamEventSink.onUpdateLiveStreamTask"
             binaryMessenger:self.binaryMessenger
                       codec:NEFLTNERtcLiveStreamEventSinkGetCodec()];
  [channel sendMessage:@[ arg_taskId ?: [NSNull null], arg_errCode ?: [NSNull null] ]
                 reply:^(id reply) {
                   completion(nil);
                 }];
}
- (void)onAddLiveStreamTaskTaskId:(NSString *)arg_taskId
                          errCode:(NSNumber *)arg_errCode
                       completion:(void (^)(FlutterError *_Nullable))completion {
  FlutterBasicMessageChannel *channel = [FlutterBasicMessageChannel
      messageChannelWithName:@"dev.flutter.pigeon.NERtcLiveStreamEventSink.onAddLiveStreamTask"
             binaryMessenger:self.binaryMessenger
                       codec:NEFLTNERtcLiveStreamEventSinkGetCodec()];
  [channel sendMessage:@[ arg_taskId ?: [NSNull null], arg_errCode ?: [NSNull null] ]
                 reply:^(id reply) {
                   completion(nil);
                 }];
}
- (void)onDeleteLiveStreamTaskTaskId:(NSString *)arg_taskId
                             errCode:(NSNumber *)arg_errCode
                          completion:(void (^)(FlutterError *_Nullable))completion {
  FlutterBasicMessageChannel *channel = [FlutterBasicMessageChannel
      messageChannelWithName:@"dev.flutter.pigeon.NERtcLiveStreamEventSink.onDeleteLiveStreamTask"
             binaryMessenger:self.binaryMessenger
                       codec:NEFLTNERtcLiveStreamEventSinkGetCodec()];
  [channel sendMessage:@[ arg_taskId ?: [NSNull null], arg_errCode ?: [NSNull null] ]
                 reply:^(id reply) {
                   completion(nil);
                 }];
}
@end
